// Aplicacion con el 16f628A ver 1.0 que sirve como transmisor de datos WX hacia
// el PIC3C2 formado con el 16f877A, el cual est a la espera de dichas
// seales de Temperatura, Humedad y Presin, por lo que el 16f628A 
// simular esos datos envindolos por el puerto RS232. El tren de 
// datos inicia con una <ESC> (0x1B), luego una 'w' minscula y seguida por 3
// bytes de temperatura en C, para decenas, unidades y una parte decimal,
// seguidas por los datos de Humedad Relativa, con 2 bytes, decenas y unidades
// y finalmente 4 ltimos bytes para la presin atmosfrica, en centenas, decenas,
// unidades y un decimal, luego otra serie de datos que exige el formato.
// En total son 32 bytes por el puerto serie, transmitidos a 9600 bps.
// El proceso se ejecuta con handshaking donde al enviar un dato, se 
// espera recibir la confirmacin con un caracter <ESC> (0x1B), para poder
// enviar un siguiente byte.
// El iniciador es el PIC3C2 quien manda a este software un comando <ESC>
// Alfredo Segura Queretaro,Mxico, agosto de 2017.

#include <16f628a.h>
#fuses HS, NOMCLR, NOBROWNOUT, NOPUT, NOCPD, NOLVP, NOWDT, NOPROTECT
#use delay (clock = 12000000)
#use rs232(baud=9600, bits=8, stop=1, parity=n, xmit=pin_b2, rcv=pin_b1)

#use fast_io(a)

#byte porta = 0x05
#byte portb = 0x06
#byte rcsta = 0x18

#bit rb2 = 0x06.2          // Tx
#bit rb4 = 0x06.4          // LED de actividad
#bit rb5 = 0x06.5          // LED Rx en puerta y Tx activado
#bit spen = 0x18.7
#bit cren = 0x18.4
#bit oerr = 0x18.1

int j, k;
int dato;
int thp1[32] = {0x1B,0x77,0x30,0x32,0x37,0x2F,0x34,0x38,0x33,0x67,0x31,0x37,0x35,0x74,0x31,0x32,0x33,0x62,0x30,0x38,0x38,0x32,0x39,0x68,0x36,0x37,0x0D,0x30,0x32,0x35,0x33,0x43};
// formato     ESC   w    0     2    7   /    4     8    3    g    1    7    5    t    1    2    3    b    0    8    8    2    9    h    6    7    CR   0    2    5    3    C 
// Nmero thp1   0    1     2   3    4    5    6     7    8    9   10   11   12   13   14   15   16   17   18   19   20   21   22   23   24   25   26  27   28   29    30   31      
// 0x1B = ESC, necesario al inicio de la trama
// 0x77 = w, se van a enviar datos de temp, humedad y presion
// 0x43 = 'C' caracter de terminacin de cadena de bytes
// Temperatura: thp1[27, 28, 29, . , 30]
// Humedad: thp1[24, 25]
// Presion: thp1[18, 19, 20, 21, . , 22]

#int_timer1
void actividad() {      // Indicacion de Actividad del PIC
j++;
if (j >= 50) {
thp1[30]++;             // hacemos variar los decimales de Temperatura
thp1[25]++;             // igual variamos las unidades de Humedad
thp1[22]++;             // y las de Presin
if (thp1[30] >= 0x39) {
thp1[30] = 0x30;
}
if (thp1[25] >= 0x39) {
thp1[25] = 0x30;
}
if (thp1[22] >= 0x39) {
thp1[22] = 0x30;
}

j = 0;
rb4++;               // LED de actividad
}
set_timer1(0);
}

void ini() {
enable_interrupts(global);
enable_interrupts(int_timer1);
setup_timer_1(t1_internal  | t1_div_by_2);
setup_ccp1(ccp_off);
setup_comparator(nc_nc_nc_nc);

set_tris_a(0b11100000);    //Solo el XTAL y /MCLR
set_tris_b(0b00000010);    //Rx = rb1, Tx = rb2, LED = rb4
porta = 0x00;           //
portb = 0x00;
set_timer1(0);
}

void main(void) {
ini();
delay_ms(300);

while (true) {

if (kbhit()) {
dato = getc();
delay_us(10);
if (dato == 0x1B) {    // Codigo de <ESC>
for (k = 0; k <= 31; k++) {
rb5 = 1;
putc(thp1[k]);        // Enva <ESC> + w + datos...
delay_us(5);
}
}
rb5 = 0;
}

if (oerr == 1) {
cren = 0;
oerr = 0;
spen = 0;
delay_ms(30);
spen = 1;
cren = 1;
}

delay_ms(30);


}
}

