; Programa Pic3C3.asm 
; Para communicacion usando la tarjeta de red 3c509 y el procesador PIC16F877
; Reprogramacion basada en el codigo:
; "interfac IP sur carte reseau ISA" by "jean-pierre MANDON 17 jun 2001"
; La fuente fue corregido, modificada para un protocolo de 16 bits, reacondicionada
; por OK1VCF 09-12/2005 y Alfredo Segura. rutinas de TCP y otras fueron adicionadas.
;
; Tamao del paquete Ethernet = 2 bytes
; Encabezado Ethernet = 14 bytes (6MAC+6MAC+2type)
; Encabezado ARP =  8 bytes, Data = 20 bytes (6MAC+4IP+6MAC+4IP)
; Encabezado IP = 20 bytes (Sin opciones)
; Encabezado UDP =  8 bytes (2SrcPort+2DstPort+2Length+2Checksum)
; Encabezado TCP = 20 bytes (Sin opciones)
;

; Si la IP recibida tiene opciones, el paquete ser erroneamente interpretado. Esto no es
; cierto en caso de que TCP SYN sea con opciones - SYN puede tener opciones.

; Se definen niveles de debug (los cuales se enviarn al puerto RS232)
; DEBUG0 - inicializacion basica para informes y tipos de paquetes RX
; DEBUG1 - inicializacion completa de informes
; DEBUG2 - Todos los paquetes de bytes son enviados (paquetes RX y TX)
; DEBUG3 - Despus de que el paquete es transmitido,los registros RAM 0x20 a 0x7f son enviados
; DEBUG4 - Despues de que el paquete es recibido, los registros RAM 0x20 a 0x7f son enviados

	LIST P=PIC16F877, R=HEX
	include<p16f877A.inc>

        __config _LVP_OFF & _HS_OSC & _WDT_OFF & _PWRTE_ON & _CP_OFF & _BODEN_OFF & _DEBUG_OFF

	errorlevel -302	; Suprimido: El registro en el operando no est en el banco 0. Asegurese que el bit de banco sea correcto.
	errorlevel -306	; Suprimido: Cruce del limite de pgina -- asegrese que el bit de gina est a 1.

#define DEBUG0
;#define DEBUG1
;#define DEBUG2
;#define DEBUG3
;#define DEBUG4

; Selecciones la velocidad de su cristal aqui para temporizacion correcta (4, 10 o 20 MHz)
; SOLAMENTE UNO DEBERA SER DESCOMENTADO !!! El resto debe ser comentado (;) - INACTIVOS !!!
;#define	XTAL4MHZ		
;#define	XTAL10MHZ
;#define	XTAL12MHZ		
#define	XTAL20MHZ		

; Define REMOTE_RESET si quiere tener la opcion de resetear el PIC via Telnet!
#define	REMOTE_RESET

; Define RTC_TRCHARGE_ENABLE si quiere habilitar la funcion de mantenimiento
#define	RTC_TRCHARGE_ENABLE

	cblock	0x20			
		pktdata			;70 bytes si el que sigue es 0x66
					;64 necesarios.
	endc
	cblock	0x60
		serialData, serialCount, delay_time, delay_time3
		serialDelayCount, reg_al
		MAC0,MAC1,MAC2,MAC3,MAC4,MAC5,IP0,IP1,IP2,IP3
		
	endc
	cblock	0x70
		AA,BB,CC,DD,TMP,byte_tmp,iohigh,iolow,g_val_hi,g_val_low
		g_len_hi,g_len_low,adresseH,adresseL,dataF,Flag
	endc
	cblock	0xA0	; bank 1 (80 bytes for pktdata)
		pktdata2
	endc
	cblock	0x10	; bank 2 (16 bytes for registers, 80 for pktdata)
		GW_MAC0,GW_MAC1,GW_MAC2,GW_MAC3,GW_MAC4,GW_MAC5
		GW_IP0,GW_IP1,GW_IP2,GW_IP3
		seqn1, seqn2, seqn3, seqn4, FREEreg1, FREEreg2
		
	endc
	cblock	0x20	; bank 2
		pktdata3
	endc
	cblock	0x190	; bank 3 (34 bytes for registers, 62 for serial data)
		wx_data_endR
		AARGB5,AARGB4,AARGB3,AARGB2,AARGB1,AARGB0,AEXP
		BARGB3,BARGB2,BARGB1,BARGB0,BEXP,SIGN,TEMP,LOOPCOUNT,FPFLAGS
		SOUCET,horniSOUCET,poc_mer,pod_nulou,TeplH,TeplL,desetiny
		temp0,temp1,temp2,digit_count,smt_act,rtc_act,tmr_counter
		timingH, timingL,srvs_flags 
	endc
	cblock	0xB2	; bank 3 (B2-CF)
		wx_data			; 30 bytes necesarios 
	endc
	cblock	0xD0	; bank 3 (D0-D7)
		rtc_data		; 8 bytes necesarios
	endc
	cblock	0xD8	; bank 3 
		ackn1, ackn2, ackn3, ackn4, FREEreg3
	endc
	cblock	0xDD	; bank 3 (DD-EF)
		time_data		; 19 bytes necesarios para formateo
					; DD.MM.YY HH:MM:SS d
	endc

;##################################################################3
;
; Ajustes del Paquete
;
;##################################################################3

			; Encabezado MAC (Ethernet)
eth_len_h		equ	0x00	; Ajuste de longitud del byte alto del paquete
eth_len_l		equ	0x01	; ajuste de longitud del byte bajo del paquete

eth_dest_0		equ	0x02	; MAC address del destinatario
eth_dest_1		equ	0x03	; MAC address del destinatario
eth_dest_2		equ	0x04	; MAC address del destinatario
eth_dest_3		equ	0x05	; MAC address del destinatario
eth_dest_4		equ	0x06	; MAC address del destinatario
eth_dest_5		equ	0x07	; MAC address del destinatario
eth_src_0		equ	0x08	; MAC address del emisor
eth_src_1		equ	0x09	; MAC address del emisor
eth_src_2		equ	0x0a	; MAC address del emisor
eth_src_3		equ	0x0b	; MAC address del emisor
eth_src_4		equ	0x0c	; MAC address del emisor
eth_src_5		equ	0x0d	; MAC address del emisor
eth_typ_0		equ	0x0e	; Bytes tipo Ethernet (0x0800=IP,
eth_typ_1		equ	0x0f	;  0x0806=ARP, ...)

			; Protocolo ARP
arp_hw			equ	0x10	; Encabezado ARP. Tipo Hardware (1=Ethernet)
arp_prot		equ	0x12	; Encabezado del protocolo ARP (0x0800 = IP)
arp_hwlen		equ	0x14	; Encabezado ARP longitud de la direccion HW
arp_protlen		equ	0x15	; Encabezado ARP longitud de la direcciondel Protocolo
arp_op			equ	0x16	; Encabezado ARP Opcode (1=Req, 2=Resp)
arp_src_hw		equ	0x18	; Dato MAC de ARP del emisor
arp_src_ip		equ	0x1e   	; Dato IP ARP del emisor
arp_dst_hw		equ	0x22   	; Dato MAC ARP del destinatario
arp_dst_ip		equ	0x28   	; Dato IP ARP del destinatario

			; Protocolo IP
ip_verlen 		equ	0x10	; IP Vers.+Int.HeaderLength (4x=IP v4)
ip_tos			equ	0x11	; IP TOS Tipo de Servicio
ip_len			equ	0x12	; IP Longitud total (del datagrama IP)
ip_id			equ	0x14	; IP Identificacion (para fragmentos)
ip_froff		equ	0x16	; IP flags y offset de fragmentacion
ip_ttl			equ	0x18	; IP TTL (Tiempo de vida)
ip_prot			equ	0x19	; IP Protocolo (1=ICMP,6=TCP,17=UDP,...)
ip_cksum		equ	0x1a	; IP checksum encabezado
ip_src			equ	0x1c	; IP Direccion IP del origen
ip_dst			equ	0x20	; IP Direccion IP del destino
ip_data			equ	0x24	; IP Optiones (se asume que ninguna) 
					;    seguidas pr e dato IP

			; Protocolo UDP
udp_src			equ	0x24	; UDP puerto origen
udp_dst			equ	0x26	; UDP puerto destino
udp_len			equ	0x28	; UDP Longitud de UDP header+data
udp_cksum		equ	0x2a	; UDP Checksum (0xFFFF=disabled)
udp_data		equ	0x2c	; UDP El dato inicia aqui

			; Protocolo ICMP
icmp_typ		equ	0x24	; ICMP Tipo (8=Echo req, 0=Echo repl)
icmp_func		equ	0x25	; ICMP Codigo (0 for Echo req/repl)
icmp_cksum		equ	0x26	; ICMP Header checksum
icmp_id			equ	0x28	; ICMP Identificador, puede ser puesto a 0
icmp_seq		equ	0x2a	; ICMP numero de secuencia, puede ser 0

			; Protocolo TCP
tcp_src_port		equ	0x24	; TCP Puerto de origen
tcp_dst_port		equ	0x26	; TCP Puerto de destino
tcp_seqn		equ	0x28	; TCP Numero de secuencia
tcp_ackn		equ	0x2c	; TCP Numero de reconocimiento
tcp_doffset		equ	0x30	; TCP Data offset (en palabras de 32 bits)
tcp_ctrl		equ	0x31	; TCP Banderas de Control
tcp_window		equ	0x32	; TCP tamao de la ventana
tcp_cksum		equ	0x34	; TCP Checksum
tcp_urge		equ	0x36	; TCP Apuntador urgente
tcp_data		equ	0x38	; TCP Los datos u opciones inician aqui

#define	TCP_URG		tcp_ctrl,5	; Bandera vlida del apuntador urgente
#define	TCP_ACK		tcp_ctrl,4	; Bandera vlida del nmero de reconocimiento
#define	TCP_PSH		tcp_ctrl,3	; Bandera Push
#define	TCP_RST		tcp_ctrl,2	; Bandera de conexin reseteada
#define	TCP_SYN		tcp_ctrl,1	; Bandera de nmeros de sincrona de secuencia
#define	TCP_FIN		tcp_ctrl,0	; Bandera de Fin de datos

#define carry		STATUS,C
#define zero		STATUS,Z
#define gw_mac_ok	Flag,0		; Bandera que se obtuvo del gateway MAC
#define end_tcp_req	Flag,1		; Bandera que se recibi al final del comando TCP
#define html_file_rep	Flag,2		; Paquete que incluye el archivo html
#define html_odd_len	Flag,3		; La longitud del archivo html es impar
#define html_pkt_snd_f	Flag,4		; Encabezado enviado para MAC,IP,TCP, ahora para el archivo html
#define html_wx_rep	Flag,5		; Bandera que indica que el archivo html es WX
#define tcp_len_ovfl	Flag,6		; Bandera que indica que IPlen-IPhead tuvo sobreflujo
#define aprsd_act	Flag,7		; Bandera que indica que Aprsd esta conectado

#define tel_syn_ack	reg_al,0	; Bandera - hemos enviado un paquete mas
#define smt_syn_ack	reg_al,1	; Bandera - hemos enviado un paquete mas

;#define sda		PORTB,3		; Datos por el canal I2C
;#define scl		PORTB,2		; reloj por el canal I2C
;#define int		PORTB,0		; interrupcin ante un cambio en I2C

#define	reset		PORTC,0		; 3c509b ISA a4-7, 10-19 puestas a Cero
#define	iow		PORTC,2		; a8 puesta a UNO
#define	ior		PORTC,1		; da un rango de puertos de 0x100-0x10F
#define a0		PORTE,0		; con a9 en estado BAJO para
#define	a1		PORTE,1		; activacion
#define a2		PORTE,2		; y rango de puertos 0x300-0x30F 
#define	a3		PORTA,1		; con a9 en estado ALTO para 
#define	a9		PORTA,2		; operacion normal via la direccin base I/O.

#define led		PORTA,3
#define int		PORTA,4

#define	userdata0	PORTD,0
#define	userdata1	PORTD,1
#define	userdata2	PORTD,2
#define	userdata3	PORTD,3
#define	userdata4	PORTD,4
#define	userdata5	PORTD,5
#define	userdata6	PORTD,6
#define	userdata7	PORTD,7
#define	userdata8	PORTB,0
#define	userdata9	PORTB,1
#define	userdataA	PORTB,2
#define	userdataB	PORTB,3
#define	userdataC	PORTB,4
#define	userdataD	PORTB,5
#define	userdataE	PORTB,6
#define	userdataF	PORTB,7

#define	serial_out	PORTC,6		; RS232 Salida Tx
#define	serial_in	PORTC,7		; RS232 Entrada Rx

#define	rtcres		PORTA,0		; Seal de Reset para el RTC
#define	rtcdata		PORTC,5		; Linea de datos para el RTC
#define	sclk		PORTC,3		; Clock para SPI,I2C,RTC,...

#define	smt_0		PORTA,0		; SMT160 sensor uno (RTC RES,A/D)
#define	smt_1		PORTA,5		; SMT160 sensor dos (SPI SS/CS)
#define	smt_2		PORTC,3		; SMT160 sensor tres (sclk)
#define	smt_3		PORTC,4		; SMT160 sensor cuatro (SPI IN,I2C I/O)
#define	smt_4		PORTC,5		; SMT160 sensor cinco (SPI OUT,RTC I/O)

#define	smt_5		PORTC,6		; SMT160 sensor seis (RS232 OUT)
#define	smt_6		PORTC,7		; SMT160 sensor siete (RS232 IN)


HTML_FILES_H		equ	0x18	; punto donde los archivos html inician en
HTML_FILES_L		equ	0x00	; el area de memoria de programa

EE_LEN			equ	0xff	; La EEPROM accesible tiene 255 bytes 

WX_DATA_MAX_LEN		equ	0x1e	; Nmero mximo de bytes WX par recibir
wx_page_req		equ	a'w'	; Primera letra desde GET para la pgina WX
wx_pattern		equ	0x1a	; 0x1a es el patrn en el archivo wx html
					; el cual cambiaremos dinamicamente debido a
					; el dato recibido desde WX
wx_tempf		equ	0x0c	; pre-ajuste de temperatura en farenheit
wx_press		equ	0x10	; pre-ajuste de presin
wx_humi			equ	0x16	; pre-ajuste de humedad
wx_tempc		equ	0x19	; pre-ajuste de temperatura en Celsius
wx_bytes_to_replace	equ	0x0b	; 4tempr+2humi+5press=11

rtc_clock_len		equ	0x08	; Longitud de datos en RAM del reloj RTC
					; RTC bytes
rtc_sec			equ	0x00	; pre-ajuste de segundos del RTC
rtc_min			equ	0x01	; pre-ajuste de minutos del RTC
rtc_hr			equ	0x02	; pre-ajuste de horas del RTC
rtc_date		equ	0x03	; pre-ajuste de fecha del RTC
rtc_month		equ	0x04	; pre-ajuste del mes del RTC
rtc_day			equ	0x05	; pre-ajuste del dia de la semana del RTC
rtc_year		equ	0x06	; RTC year offset
rtc_cntr		equ	0x07	; RTC control offset
rtc_trchrge		equ	b'10100101' ; Carga lenta (1diode+2k res.)

ASCII_OFFSET		equ	0x30	; pre-ajuste para numeros ascii

#define aprsd_conn	srvs_flags,0	; Bandera que debemos conectar a aprsd
#define dtime_conn	srvs_flags,1	; Bandera que debemos sincronizar via DayTime
#define aprsd_tout	srvs_flags,2	; Bandera indica que aprsd no est enviando datos
#define aprsd_auth	srvs_flags,3	; Bandera para enviar autorizacin aprsd
#define aprsd_form	srvs_flags,4	; formato de dato de salida 1=UIView, 0=normal
#define aprsd_UIsnd	srvs_flags,5	; Formato UIView, <UI> fu enviado
#define aprsd_TXena	srvs_flags,6	; Enva dato a aprsd 1=habilitado 0=deshabilitado
#define reset_flag	srvs_flags,7	; Bandera para SW de reset del programa

bank0	macro
	bcf STATUS,RP0
	bcf STATUS,RP1
	endm

bank1	macro
	bsf STATUS,RP0
	bcf STATUS,RP1
	endm


bank2	macro
	bsf STATUS,RP1
	bcf STATUS,RP0
	endm

bank3	macro
	bsf STATUS,RP1
	bsf STATUS,RP0
	endm
	
page0	macro
	bcf PCLATH, 3
	bcf PCLATH, 4
	endm
	
page1	macro
	bsf PCLATH, 3
	bcf PCLATH, 4
	endm
	
page2	macro
	bcf PCLATH, 3
	bsf PCLATH, 4
	endm
	
page3	macro
	bsf PCLATH, 3
	bsf PCLATH, 4
	endm

;##################################################################
;
; Inicializacin del Sistema
;
;##################################################################
        org	0x0000
	page2				; Poner PCLATCH para programar la pagina 2
        goto    Start

        org	0x0004
Interrupt
        retfie

tcp_Version_Data:	
	addwf	PCL,F		; Direccionamiento Relativo
	retlw	0x0d
	retlw	0x0a
	retlw	a'P'
	retlw	a'I'
	retlw	a'C'
	retlw	a'3'
	retlw	a'C'
	retlw	a'o'
	retlw	a'm'
	retlw	a'>'
	retlw	a' '
	retlw	000H

Version_Data:	
	addwf	PCL,F		; Direccionamiento Relativo
	retlw	a'P'
	retlw	a'I'
	retlw	a'C'
	retlw	a'-'
	retlw	a'3'
	retlw	a'C'
	retlw	a'o'
	retlw	a'm'
	retlw	a' '
	retlw	a'v'
	retlw	a'0'
	retlw	a'.'
	retlw	a'1'
	retlw	000H


wx_byte_table				; Devuelve el byte WX debido al valor en W
	addwf	PCL, F			; Toma el byte apropiado
	goto	wx_tempc0		; el cual es simplemente contado desde ya
	goto	wx_tempc1		; por los bytes reemplazados
	goto	wx_tempc2		; Saltaremos a la parte apropiada
	goto	wx_tempc3		; desde aqu.
	goto	wx_humi0		; 
	goto	wx_humi1		; 
	goto	wx_press0		; 
	goto	wx_press1		; 
	goto	wx_press2		; 
	goto	wx_press3		; 
	goto	wx_press4		; 

wx_tempc0				; Y aqui, movemos el valor apropiado
	movf	wx_data+wx_tempc+0, W	; desde los registros WX RAM dentro de W
	return				; 
wx_tempc1				; 
	movf	wx_data+wx_tempc+1, W	; 
	return				; 
wx_tempc2				; 
	movf	wx_data+wx_tempc+2, W	; 
	return				; 
wx_tempc3				; 
	movf	wx_data+wx_tempc+3, W	; 
	return				; 
wx_humi0				; 
	movf	wx_data+wx_humi+0, W	; 
	return				; 
wx_humi1				; 
	movf	wx_data+wx_humi+1, W	; 
	return				; 
wx_press0				; 
	movf	wx_data+wx_press+0, W	; 
	return				; 
wx_press1				; 
	movf	wx_data+wx_press+1, W	; 
	return				; 
wx_press2				; 
	movf	wx_data+wx_press+2, W	; 
	return				; 
wx_press3				; 
	movf	wx_data+wx_press+3, W	; 
	return				; 
wx_press4				; 
	movf	wx_data+wx_press+4, W	; 
	return				; 


;##################################################################3
;
; Main Loop
;
;##################################################################3
Main
	ifdef	REMOTE_RESET		; 
	bank3				;
	btfss	reset_flag		; Si la bandera de reset no es 1
	goto	Main_loop		; continua el bucle
	page0				; Poner PCLATCH para programar la pgina 0
	bank0				; 
	goto	0x0000			; Activa el sw de reset
	endif				; 

Main_loop				;
	bank0				; Elegir el banco por default

Main_RX232:				; Rutina de recepcin va RS-232
	call	serial_Receive		; lee la entrada RS232
	btfss	serialData,00		; Recibimos algo?
	goto	Main_RX_Eth		; No, contina...
	movwf	TMP			; Un byte es recibido, lo almacena aqu...
	xorlw	0x1b			; Es ESC ?
	btfsc	STATUS,Z		; Salta si no es cero
	goto	Serial_Receive_ESC	; Se recibi ESC, que haremos en seguida?
	movf	TMP,W			; No es ESC, asi que
	goto	Main_RX_Eth		; continuamos con pruebas desde Eth RX

Main_RX_Eth				; Rutina de recepcin va Ethernet
	call	rx_complete		; Recibimos algn paquete?
	xorlw	0x01			; Devuelve un en W si es SI, o 0 si es NO
	btfss	STATUS, Z		; Salta si un paquete es recibido
	goto	Main_timechk		; Ningn paquete recibido
	call	rx_packet		; Recibe un paquete si hay uno listo
	xorlw	0x01			; Devuelve 0x01 en W si es sI, o 0 si es NO
	btfss	STATUS, Z		; Salta si un paquete es recibido
	goto	Main_timechk		; Ningn paquete recibido
	ifdef	DEBUG4
	page1				; Poner PCLATCH para programar la pgina 1
	call	send_RAM_reg		; Para debug
	page0				; Poner PCLATCH para programr la pgina 0
	endif
	call	process_packet		; Analiza el paquete recibido
	page0				; Poner PCLATCH para programr la pgina 0
	bcf 	led			; Apaga el LED
	ifdef	DEBUG3
	page1				; Poner PCLATCH para programar la pgina 1
	call	send_RAM_reg		; Para debug
	page0				; Poner PCLATCH para programr la pgina 0
	endif

Main_timechk				; 
	btfsc	gw_mac_ok		; Si tenemos una MAC GW
	goto 	Main_timechk2		; Contina con la funcin time check
	btfss	PIR1, TMR1IF		; Si no - verifica si TMR1 tuvo owerflow
	goto 	Main			; No. Contina el bucle
	clrf	TMR1H			; Borra el timer
	bcf	PIR1, TMR1IF		; Borra la bandera de interrupcin
	bank3				; 
	incf	tmr_counter, F		; Incrementa el contador de sobreflujo
	movlw	0x1f			; mscara para Repeat
	andwf	tmr_counter, W		; Repeat enviado cada 32avo
	bank0				; 
	iorlw	0x00			; sobreflujo => una vez por 16s a 4MHz
	btfss	zero			; 
	goto 	Main			; De otra manera - Contina el bucle
	page2				; Poner PCLATCH para programar la pgina 2
	goto 	Send_arp_gw_req		; El 32avo sobreflujo, enva otro req
	
Main_timechk2				; 
	btfss	PIR1, TMR1IF		; TMR1 tiene sobreflujo?
	goto 	Main			; No. Contina el bucle
	bcf	PIR1, TMR1IF		; Borra la bandera de interrupcin

	bank3				; 
	btfss	aprsd_conn		; Si no debemos conectar a aprsd
	goto 	Main_timechk4		; Contina con la funcin time check
	btfsc	aprsd_act		; Si estamos conectados
	goto 	Main_timechk3		; Contina con la funcin time check
	incf	tmr_counter, F		; Incrementa el contador de sobreflujos
	ifdef	XTAL4MHZ		; 
	movlw	0x1f			; enmascara Repeat una vez cada 16s a 4MHz
	endif				;
	ifdef	XTAL10MHZ		; 
	movlw	0x3f			; enmascara Repeat una vez cada 10s a 10MHz
	endif				;
	ifdef	XTAL12MHZ		; 
	movlw	0x4f			; enmascara Repeat una vez cada 10s a 12MHz
	endif				;
	ifdef	XTAL20MHZ		; 
	movlw	0x7f			; enmascara Repeat una vez cada 10s a 20MHz
	endif				;
	andwf	tmr_counter, W		; Se envi Repeat?
	;iorlw	0x00			; 
	btfss	zero			; 
	goto 	Main_timechk4		; Contina con la funcin time check
	page2				; Poner PCLATCH para programar la pgina 2
	goto 	TCP_send_aprsd_SYN_pkt	; 32avo sobreflujo, enviar otro req

Main_timechk3				; 
	bank3				; 
	btfss	aprsd_auth		; Si debemos autorizar
	call	Authorize_aprsd		; 
	bank3				; 
	ifdef	XTAL4MHZ		; 
	movlw	0x03			; Para 4Mhz el Timeout es de aproximadamente 9 min
	endif				;
	ifdef	XTAL10MHZ		; 
	movlw	0x07			; enmascara para un Timeout de 8 min para 10 MHz
	endif				;
	ifdef	XTAL12MHZ		; 
	movlw	0x07			; enmascara para un Timeout de 8 min para 12 MHz
	endif				;
	ifdef	XTAL20MHZ		; 
	movlw	0x0f			; enmascara para un Timeout de 8 min para 20 MHz
	endif				;
	andwf	timingH, W		; Enviar Repeat cada 9 min
	;iorlw	0x00			; 
	btfss	zero			; 
	goto 	Main_timechk4		; Continue with time check function
	movlw	0x00			; Is it really 32000-th
	iorwf	timingL, W		; owerflow?
	btfss	zero			; If not
	goto 	Main_timechk4		; Continue with time check function
	btfss	aprsd_tout		; if we are receiving data!
	goto	Main_timechk3a		; 
	bcf	aprsd_act		; Data are not received, reconnect!
	bcf	aprsd_auth		; And reauthorize.
	goto 	Main_timechk4		; Continue with time check function
	
Main_timechk3a				; 
	bsf	aprsd_tout		; Set that we are not receiving data!
	page2				; Set PCLATCH to program page 1
	call	TCP_send_aprsd_WX_pkt	; Send WX report to aprsd server

	ifdef	DEBUG0			; Print send time onto RS-232
	bank3				; 
	incf	timingL, F		; Increase timing counter
	bcf	aprsd_tout		; Set that we are receiving data!
	page1				; Set PCLATCH to program page 1
	goto	RTC_get_232 		; get RTC time and send it to RS232
	endif				; 

Main_timechk4				; 
	bank3				; 
	incfsz	timingL, F		; Increase timing counter
	goto 	Main			; If not owerflowed - Continue looping
	incfsz	timingH, F		; Increase timing counter
	goto 	Main			; If not owerflowed - Continue looping
	btfss	dtime_conn		; If we should not sync. via DayTime
	goto 	Main_timechk5		; Continue with time check function
	page1				; Set PCLATCH to program page 1
	call	RTC_send_sync_pkt	; Synchronize RTC to Internet time srv
	page0				; Set PCLATCH to program page 0
	bank3				; 
	ifdef	XTAL4MHZ		; 
	movlw	0xCF			; For 4Mhz sync aprox. every 2hours
	endif				;
	ifdef	XTAL10MHZ		; 
	movlw	0x85			; For 10Mhz sync aprox. every 2hours
	endif				;
	ifdef	XTAL12MHZ		; 
	movlw	0xA5			; For 12Mhz sync aprox. every 2hours
	endif				;
	ifdef	XTAL20MHZ		; 
	movlw	0x0A			; For 20Mhz sync aprox. every 2hours
	endif				;
	movwf	timingH			; 

Main_timechk5				; 
	goto 	Main			; Continue looping

Serial_Receive_ESC:			; ESC received, it may be a command...
	call	serial_Receive_BYTE	; read next byte from RS232
	movwf	TMP			; store it here
	xorlw	0x1b			; ESC received?
	btfsc	STATUS,Z		; Skip if not zero
	goto	Serial_Receive_ESC	; another ESC received, read next byte
	page1				; Set PCLATCH to program page 1
	movf	TMP,W			; It was not ESC, what it was?
	xorlw	a'D'			; D received?
	btfsc	STATUS,Z		; Skip if not zero
	goto	Write_Config_to_EEPROM	; 'D'=write configuration to EEPROM
	movf	TMP,W			; Restore received byte
	xorlw	a'U'			; U received?
	btfsc	STATUS,Z		; Skip if not zero
	goto	Send_Config_from_EEPROM	; 'U'=send configuration from EEPROM
	movf	TMP,W			; Restore received byte
	xorlw	a'R'			; R received?
	btfsc	STATUS,Z		; Skip if not zero
	goto	RAMsend			; 'R'=send RAM registers content
	movf	TMP,W			; Restore received byte
	xorlw	a'W'			; W received?
	btfsc	STATUS,Z		; Skip if not zero
	goto	WX_RAMsend		; 'W'=send WX RAM registers content  
	movf	TMP,W			; Restore received byte
	xorlw	a't'			; t received?
	btfsc	STATUS,Z		; Skip if not zero
	goto	RTC_set_232 		; 't'=set RTC time
	movf	TMP,W			; Restore received byte
	xorlw	a'T'			; T received?
	btfsc	STATUS,Z		; Skip if not zero
	goto	RTC_get_232 		; 'T'=get RTC time and send it to RS232
	page2				; Set PCLATCH to program page 2
	movf	TMP,W			; Restore received byte
	xorlw	a'O'			; O received?
	btfsc	STATUS,Z		; Skip if not zero
	goto	Change_aprs_format	; 'O'=change 
	page0				; Set PCLATCH to program page 0
	movf	TMP,W			; Restore received byte
	xorlw	a'w'			; w received?
	btfsc	STATUS,Z		; Skip if not zero
	goto	Write_WX_to_RAM		; 'w'=write WX data string to RAM regs
	movf	TMP,W			; Restore received byte
	xorlw	a'r'			; r received?
	btfsc	STATUS,Z		; Skip if not zero
	goto	send_RAM_asc		; 'r'=Send RAM regs in ascii format
	movf	TMP,W			; Restore received byte
	xorlw	a'V'			; V received?
	btfsc	STATUS,Z		; Skip if not zero
	call	Send_Version		; 'V'=send RS232 FirmWare version
	goto	Main_RX_Eth		; continue with Eth RX tests
	
send_RAM_asc				; Sends RAM 0+2 registers content
	page1				; Set PCLATCH to program page 1
	call	send_RAM_reg		; 
	movlw	0x10			; 
	movwf	FSR			; 
	bsf	STATUS, IRP		; indirect bank 2+3
	clrf	CC			; 
	page1				; Set PCLATCH to program page 1
	call	send_RAM_reg_loop	; 
	bcf	STATUS, IRP		; set indirect page 0+1
	page0				; Set PCLATCH to program page 0
	goto	Main_RX_Eth		; continue with Eth RX tests
	
Authorize_aprsd				;
	page2				; Set PCLATCH to program page 2
	goto 	TCP_send_aprsd_AUTH_pkt	; Send authorize packet
	
;##################################################################3
;
; Start of packet-related subroutines
;
;##################################################################3

oc_startsum				; Calculates checksum
	clrf	AA			; clear temp registers
	clrf	BB			; 
	clrf	CC			; 

oc_cumsum				; 
	movwf	DD			; incoming: len in W, start in FSR
	bcf	STATUS,C		; 
	rrf	DD,f			; length/2

oc_hi_byte				; 
	movf	INDF,W			; 
	addwf	BB,f			; 
	skpc				; 
	goto	oc_low_byte		; 
	incf	CC,f			; 
oc_low_byte				; 
	incf	FSR,f			; 
	movf	INDF,W			; 
	addwf	AA,f			; 
	skpc				; 
	goto	oc_test_done		; 
	movlw	0x01			; 
	addwf	BB,f			; 
	skpnc				; 
	incf	CC,f			; 
oc_test_done				; 
	incf	FSR,f			; 

	call	do_indirect_bank_tests	; indirect address bank maintenance

oc_len_tests				; 
	decfsz	DD,f			; 
	goto	oc_hi_byte		; 
	bcf	STATUS, IRP		; Set indirect page 0+1
	return				; All done

oc_getsum				; Place computed checksum in AA BB
	movf	CC,W			; 
	addwf	AA,f			; 
	skpnc				; 
	incf	BB,f			; 
	comf	AA,f			; 
	comf	BB,f			; 
	return				; All done

;------------------------------------------------------------------------------
; Analyze the received packet
;------------------------------------------------------------------------------
process_packet				; Analyze the received packet
	bsf led				; Switch the LED On
	tstf	pktdata+eth_len_l	; Is packet length = 0?
	skpnz				; No, continue
	return				; Packet length=0, bad packet!

	movf	pktdata+eth_typ_0,W	; Is Ethernet type high byte
	sublw	0x08			; 0x08 (IP or ARP)?
	skpz				; Yes, continue
	return				; For other types return
	bcf	html_file_rep		; clear flag that reply is html file
	clrf	reg_al			; clear TCP protocol syn flags
	movf	pktdata+eth_typ_1,W	; So is Ethernet type 
	sublw	0x06			; 0x0806 (ARP)?
	bz	do_arp			; Yes, so process ARP Response
	movf	pktdata+eth_typ_1,W	; Or is Ethernet type
	sublw	0x00			; 0x0800 (IP)?
	bz	do_ip			; Yes, so process IP decoding
	return				; For other types return!

;------------------------------------------------------------------------------
; Received IP packet, continue analyzing
;------------------------------------------------------------------------------
do_ip
	ifdef	DEBUG0
	movlw	A'I'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'P'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif
	
	movlw	0x01			; Is IP protocol ICMP (0x01)?
	subwf	pktdata+ip_prot,W	; 
	skpz				; 
	goto	check_udp		; Continue with UDP type check
	movlw	0x08			; Is it ICPM Echo Request?
	subwf	pktdata+icmp_typ,W	; 
	skpz				; Skip if ICMP Ping
	goto	check_udp		; It is not ICMP Echo Request!
	page2				; Set PCLATCH to program page 2
	goto	do_ping			; ICMP Echo Request received

check_udp				; Check if the IP proto. is UDP
	movlw	0x11			; UDP protocol id is 17 (0x11)
	subwf	pktdata+ip_prot,W	; 
	skpnz				; It is not UDP protocol
	goto	do_udp			; UDP protocol received

check_tcp				; Check if the IP proto. is TCP
	movlw	0x06			; TCP protocol id is 6 (0x06)
	subwf	pktdata+ip_prot,W	; 
	skpnz				; It is not TCP protocol
	goto	do_tcp			; TCP protocol received
	
check_other				; Other protocols unsupported
	return				; All done


;------------------------------------------------------------------------------
; Set (swap) destination and source IP addresses in IP reply packet
;------------------------------------------------------------------------------
reply_set_ip
        movf    pktdata+ip_src,W		; Place sender ip into
        movwf   pktdata+ip_dst			; destimantion fields
        movf    pktdata+ip_src+1,W		; when constructing 
        movwf   pktdata+ip_dst+1		; reply packet
        movf    pktdata+ip_src+2,W		; 
        movwf   pktdata+ip_dst+2		; 
        movf    pktdata+ip_src+3,W		; 
        movwf   pktdata+ip_dst+3 		; 

set_my_src_ip					; 
        movf   	IP0,W				; Place my IP as 
        movwf   pktdata+ip_src			; source addres
        movf   	IP1,W				; 
        movwf   pktdata+ip_src+1		; 
        movf   	IP2,W				; 
        movwf   pktdata+ip_src+2		; 
        movf   	IP3,W				; 
        movwf   pktdata+ip_src+3		; 
	return					; All done

;------------------------------------------------------------------------------
; Set (swap) Ethernet (MAC) source and destination adresses in reply packet
;------------------------------------------------------------------------------
reply_set_mac					; Place source MAC from  
        movf    pktdata+eth_src_0,W		; received packet to
        movwf   pktdata+eth_dest_0		; destination place
        movf    pktdata+eth_src_1,W		; because we are 
        movwf   pktdata+eth_dest_1		; preparing the answer
        movf    pktdata+eth_src_2,W		; packet
        movwf   pktdata+eth_dest_2		; 
        movf    pktdata+eth_src_3,W		; 
        movwf   pktdata+eth_dest_3		; 
        movf    pktdata+eth_src_4,W		; 
        movwf   pktdata+eth_dest_4		; 
        movf    pktdata+eth_src_5,W		; 
        movwf   pktdata+eth_dest_5		; 

reply_set_my_mac				; 
        movf   	MAC0,w				; Place my own MAC
        movwf   pktdata+eth_src_0		; (obtained from card's
        movf   	MAC1,w				; EEPROM during initialization)
        movwf   pktdata+eth_src_1		; as source MAC in the
        movf   	MAC2,w				; answer packet
        movwf   pktdata+eth_src_2		; 
        movf   	MAC3,w				; 
        movwf   pktdata+eth_src_3		; 
        movf   	MAC4,w				; 
        movwf   pktdata+eth_src_4		; 
        movf   	MAC5,w				; 
        movwf   pktdata+eth_src_5		; 
	return					; All done 

;------------------------------------------------------------------------------
ip_check_IP				; 
	movf	pktdata+ip_dst,W	; Verify that arp packet is 
	subwf	IP0,W			; really for my IP
	skpz				; 
	retlw	0x00			; IP differs, return
	movf	pktdata+ip_dst+1,W	; 
	subwf	IP1,W			; 
	skpz				; 
	retlw	0x00			; IP differs, return 
	movf	pktdata+ip_dst+2,W	; 
	subwf	IP2,W			; 
	skpz				; 
	retlw	0x00			; IP differs, return 
	movf	pktdata+ip_dst+3,W	; 
	subwf	IP3,W			; 
	skpz				; 
	retlw	0x00			; IP differs, return 
	retlw	0x01			; IP is mine!

;------------------------------------------------------------------------------
; Process received TCP packet
;------------------------------------------------------------------------------
do_tcp					; expecting 8-byte packet on any port
	ifdef	DEBUG0
	movlw	A'T'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'C'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'P'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif
	
	call	ip_check_IP		; Verify if packet is for my IP
	xorlw	0x01			; 0x01 means Ok
	skpz				; 
	return				; IP differs, return 
	
	clrf 	pktdata+ip_cksum	; clear IP checksum 
	clrf 	pktdata+ip_cksum+1	; 
        clrf    pktdata+tcp_cksum	; clear TCP cheksum
        clrf    pktdata+tcp_cksum+1	; 
	bcf	end_tcp_req		; Clear flag for FIN in reply packet

	call	reply_set_ip		; Set dest. and sender IP
	call	reply_set_mac		; Set dest. and sender MAC
	call	reply_set_tcp_port	; Set dest. and sender port
	call	reply_swap_tcp_seqnackn	; Set AckNumber and SeqNumber
	clrf	pktdata+tcp_window+0	; Set window size
	movlw	D'168'			; to 168 bytes (80+80+8)
	movwf	pktdata+tcp_window+1	; 

	call	tcp_test_http		; Test if port = 80 (HTTP)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfsc	zero			; If not zero, port is not 80! 
	decf	pktdata+tcp_window+0,F	; Set big window for http connection

tcp_window_done				; 
	clrf	pktdata+tcp_urge+0	; Clear urgent pointer
	clrf	pktdata+tcp_urge+1	; Clear urgent pointer
	

	movlw	b'00010011'		; Mask ACK, SYN and FIN bits
	andwf	pktdata+tcp_ctrl, F	; in Control Bits byte
	movlw	b'00000010'		; is it only SYN?
	xorwf	pktdata+tcp_ctrl, W	; 
	btfsc	STATUS, Z		; 
	goto	tcp_syn_received	; Yes, only SYN flag!
	movlw	b'00010010'		; is it ACK + SYN?
	xorwf	pktdata+tcp_ctrl, W	; 
	btfsc	STATUS, Z		; 
	goto	tcp_ack_syn_received	; Yes, ACK + SYN flag!
	btfsc	pktdata+tcp_ctrl, 0	; is FIN flag set?
	goto	tcp_fin_received	; Yes, FIN flag!
	
		; normal packet (ack, data) received
	movlw	0xf0			; mask out reserved bits
	andwf	pktdata+tcp_doffset, F	; and divide Data Offset
	bcf	STATUS, C		; by 4 to have TCPhead length
	rrf	pktdata+tcp_doffset, F	; of the received packet in
	rrf	pktdata+tcp_doffset, F	; bytes in pktdata+tcp_doffset
	movf	pktdata+ip_verlen, W	; place head length 
	andlw	0x0f			; in W
	bcf	STATUS, C		; clear carry
	movwf	byte_tmp		;
	rlf	byte_tmp, F		; multiply it by 4 to get 
	rlf	byte_tmp, W		; number of bytes in ip header
	subwf	pktdata+ip_len+1, W	; substract it from ip_len
	movwf	TMP			; and store
	movf	pktdata+tcp_doffset, W	; take TCP header length
	subwf	TMP, F			; substract it from stored val.
	btfsc	STATUS, Z		; data length is zero ?
	goto	tcp_end			; Yes, ACK without data, end!
					; ACK with data received,
	movf	TMP, W			; Take data length
	call	inc_ackn		; inc ack number by data len.
	call	set_fr_ttl		; Set do not fragment and ttl
	movlw	b'00010000'		; Set ACK in answer
	movwf	pktdata+tcp_ctrl	; packet

	movf	pktdata+tcp_doffset, W	; take TCP header length
	addlw	pktdata+tcp_src_port	; add it to TCP header start
	movwf	FSR			; set it to FSR pointer
	bcf	STATUS, IRP		; set indirect page 0+1
	goto	tcp_parse_data		; Look what we received, 
	
tcp_syn_received			; SYN request received...
	movlw	0x01			; increase ACK by one
	call	inc_ackn		; 

tcp_syn_HTTP_tst			; 
	call	tcp_test_http		; Test if port = 80 (HTTP)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfsc	zero			; If not zero, port is not 80! 
	goto	tcp_syn_ack_send	; Send SYN+ACK packet
tcp_syn_TELNET_tst				; 
	call	tcp_test_telnet		; Test if port = 23 (telnet)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfss	zero			; If not zero, port is not 23! 
	goto	tcp_syn_SMT_tst		; Continute testing
	bsf	tel_syn_ack		; set flag that TELNET syn received
	goto	tcp_syn_ack_send	; Send SYN+ACK packet
tcp_syn_SMT_tst				; 
	call	tcp_test_SMT		; Test if port = 18624 (SMTtelnet)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfss	zero			; If not zero, port is not 18624! 
	goto	tcp_syn_END_tst		; Continute testing
	bsf	smt_syn_ack		; set flag that SMT syn received
	goto	tcp_syn_ack_send	; Send SYN+ACK packet
tcp_syn_END_tst				; 
	goto	tcp_fin_send		; For other ports send FIN

tcp_syn_ack_send			; 
	movlw	b'00010010'		; Set ACK+SYN in answer
	movwf	pktdata+tcp_ctrl	; packet
	movlw	0x02			; Option Max Segment size (4B)
	movwf	pktdata+tcp_data+0	; kind = 0x02 (MSS)
	movlw	0x04			;
	movwf	pktdata+tcp_data+1	; length = 0x04 bytes
	clrf	pktdata+tcp_data+2	; We have small buffer
	movlw	D'188'			; bank 1,2 = 80+80
	movwf	pktdata+tcp_data+3	; bank 0 = 28 TCP bytes => 188

	call	tcp_test_http		; Test if port = 80 (HTTP)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfsc	zero			; If not zero, port is not 80! 
	decf	pktdata+tcp_data+2,F	; Set big buffer for http connection

tcp_syn_MSS_done			; 
	clrf	pktdata+ip_len+0	; 
	movlw	D'44'			; Length of IP+TCP = 20+20+4
	movwf	pktdata+ip_len+1	; 
	call	set_fr_ttl		; Set do not fragment and ttl
	movlw	0x60			; Set TCP header length to
	movwf	pktdata+tcp_doffset	; 6x4=24 bytes
	movlw	D'24'			; Length of TCP part = 20+4
	call	tcp_do_cksum		; Count TPC checksum
	clrf	pktdata+eth_len_h	; Length of Ethernet packet
	movlw	D'58'			; tcp24+ip20+mac14=58
	movwf	pktdata+eth_len_l	; 
	goto	tcp_send		; And send it

	
tcp_fin_received			; FIN received...
	movlw	0x01			; increase ACK by one
	call	inc_ackn		; 
tcp_fin_send				; Entered when we want to end session
	movlw	b'00010001'		; Set ACK+FIN in answer
	movwf	pktdata+tcp_ctrl	; packet
	clrf	pktdata+ip_len+0	; 
	movlw	D'40'			; Length of IP+TCP = 40
	movwf	pktdata+ip_len+1	; 
	call	set_fr_ttl		; Set do not fragment and ttl
	movlw	0x50			; Set TCP header length to
	movwf	pktdata+tcp_doffset	; 5x4=20 bytes
	movlw	D'20'			; Length of TCP part = 20
	call	tcp_do_cksum		; Count TPC checksum
	clrf	pktdata+eth_len_h	; Length of Ethernet packet
	movlw	D'54'			; tcp20+ip20+mac14=54
	movwf	pktdata+eth_len_l	; 
	goto	tcp_send		; And send it
	

tcp_ack_syn_received			; ACK + SYN received...
	movlw	0x01			; increase ACK by one
	call	inc_ackn		; 

	call	tcp_test_aprsd_telnet	; Test if port = 18673 (aprsd)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfsc	zero			; If not zero, port is not 18673! 
	bsf	aprsd_act		; Set TCP connect flag!

	goto	tcp_send_ack		; Send an ACK packet


tcp_send_ack				; Sends ACK packet, expects
					; that ackn is set already
	movlw	b'00010000'		; Set ACK in answer
	movwf	pktdata+tcp_ctrl	; packet
	clrf	pktdata+ip_len+0	; 
	movlw	D'40'			; Length of IP+TCP = 40
	movwf	pktdata+ip_len+1	; 
	call	set_fr_ttl		; Set do not fragment and ttl
	movlw	0x50			; Set TCP header length to
	movwf	pktdata+tcp_doffset	; 5x4=20 bytes
	movlw	D'20'			; Length of TCP part = 20
	call	tcp_do_cksum		; Count TPC checksum
	clrf	pktdata+eth_len_h	; Length of Ethernet packet
	movlw	D'54'			; tcp20+ip20+mac14=54
	movwf	pktdata+eth_len_l	; 
	goto	tcp_send		; And send it
	

tcp_send				;
	;bank0				; bank0
	call	send_pkt		; Send prepared packet 
tcp_send_next_tst1			;
	movlw	0x00			; Test if we have to send another
	iorwf	reg_al, W		; packet after syn+ack - it is for
	btfsc	zero			; TELNET and SMTtelnet ports
	goto	tcp_end			; We do not have to send any!
	incf	pktdata+tcp_seqn+3, F	; inc SEQn, we should not overflowed
	movlw	b'00010000'		; Set ACK in answer
	movwf	pktdata+tcp_ctrl	; packet
	btfss	tel_syn_ack		; Do we have to send Telnet prompt?
	goto	tcp_send_next_tst2	; No.
	clrf	reg_al			; clear TCP protocol syn flags
	movlw	pktdata+tcp_data	; First reply data byte offset
	movwf	FSR			; place here
	goto	Send_telnet_first_pkt	; and send packet
tcp_send_next_tst2			;
	btfss	smt_syn_ack		; Do we have to send SMT reply?
	goto	tcp_send_next_tst3	; No.
	clrf	reg_al			; clear TCP protocol syn flags
	goto	tcp_parse_SMT		; and send packet
tcp_send_next_tst3			;

tcp_end					; 
	call	tcp_test_aprsd_telnet	; Test if port = 18673 (aprsd)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfss	zero			; If not zero, port is not 18673! 
	return				; All done
	page2				; Set PCLATCH to program page 2
	call	store_TCP_ackn_seqn	; Store ackn and seqn
	page0				; Set PCLATCH to program page 0
	return				; All done


;------------------------------------------------------------------------------
; TCP with data received, parse it.
;------------------------------------------------------------------------------
tcp_parse_data				; Parse received TCP data
	call	tcp_test_http		; Test if port = 80 (HTTP)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfsc	zero			; If not zero, port is not 80! 
	goto	tcp_parse_http		; Else parse http packet.
	call	tcp_test_telnet		; Test if port = 23 (telnet)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfsc	zero			; If not zero, port is not 23! 
	goto	tcp_parse_telnet	; Else parse telnet packet.
	call	tcp_test_SMT		; Test if port = 18624 (SMTtelnet)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfsc	zero			; If not zero, port is not 18624! 
	goto	tcp_parse_SMT		; Else parse smttelnet packet.
	call	tcp_test_aprsd_telnet	; Test if port = 18673 (aprsd)
	xorlw	0x01			; Returns 0x01 if Yes.
	btfsc	zero			; If not zero, port is not 18673! 
	goto	tcp_parse_aprsd		; Else parse aprsdtelnet packet.

	goto	tcp_fin_send		; For other packets do nothing!

tcp_parse_telnet			;
	ifdef	DEBUG0
	movlw	A'T'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'E'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'L'			;
	call	serial_SEND_byte	; For debug 
	movlw	A's'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'r'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'v'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	movf	INDF, W			; Take the first data byte
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif
	
	movf	INDF, W			; Take the first data byte
	xorlw	a'q'			; Is it 'q'? = quit
	btfsc	STATUS, Z		; No, skip
	bsf	end_tcp_req		; Set Quit flag
	
	ifdef	REMOTE_RESET		; 
	movf	INDF, W			; Take the first data byte
	xorlw	a'^'			; Is it '^'? = reset
	btfss	STATUS, Z		; Yes, skip
	goto	Send_telnet_first_pkt	; this
	bsf	end_tcp_req		; Set Quit flag
	bank3				;
	bsf	reset_flag		; Set the reset flag!
	bank0				;
	endif

Send_telnet_first_pkt			; 
	clrf	AA			; clear bytes sent counter

Send_Version_tcp:			;
	movf	AA, W			; take byte offset
	call	tcp_Version_Data	; get a byte
	iorlw	000H			; zero = end of sending
	btfsc	STATUS,Z		; 
	goto	Send_Version_tcp_END	; All text send!
	movwf	INDF			; 
	incf	AA, F			; increase pointer
	incf	FSR, F			; increase pointer
	call	do_indirect_bank_tests	; indirect address bank maintenance
	goto	Send_Version_tcp	; and process next byte

Send_Version_tcp_END			;
	clrf	INDF			; clear the byte for checksumming

Send_tcp_final				;
	movf	AA,W			; data size into W
					; Return answer data len in W.
	movwf	TMP			; So store it here.

	btfsc	end_tcp_req		; Was Quit in the RX packet? 
	bsf	pktdata+tcp_ctrl, 0	; Quit received, set FIN flag.

	addlw	D'40'			; IP+TCP heads have 40 bytes
	movwf	pktdata+ip_len+1	; So set IP length
	clrf	pktdata+ip_len+0	; in the packet
	clrf	pktdata+eth_len_h	; Length of Ethernet packet
	addlw	D'14'			; Eth. header has next 14 bytes
	movwf	pktdata+eth_len_l	; 
	movlw	0x50			; TCP header length is 5x4bytes
	movwf	pktdata+tcp_doffset	; set it here
	movf	TMP, W			; Take stored tcp data lenght
	addlw	D'20'			; TCP head has 20 bytes
	btfsc	TMP,0			; is the result even?
	addlw	0x01			; is odd, make it even 
	call	tcp_do_cksum		; Count TCP checksum

	goto	tcp_send		; And send it

;------------------------------------------------------------------------------
tcp_test_http				; 
	movlw	D'80'			; TCP port 80 = HTTP protocol
	xorwf	pktdata+tcp_src_port+1,W	; Ports are swapped for answer!
	btfss	zero			;  
	goto	tcp_test_http_aprsd	; If port is not 80, return 0x00
	movf	pktdata+tcp_src_port,F	; Ensure that high nibble is zero
	btfsc	zero			; If not zero, port is not 80! 
	retlw	0x01			; Port is 80, return 0x01

tcp_test_http_aprsd			; Test port 14501 (0x38a5)
	movlw	0xa5			; TCP port 14501 = HTTP protocol
	xorwf	pktdata+tcp_src_port+1,W	; Ports are swapped for answer!
	btfss	zero			;  
	retlw	0x00			; If port is not 14501 or 80, ret 0x00
	movlw	0x38			; TCP port 14501 = HTTP protocol
	xorwf	pktdata+tcp_src_port+0,W	; Ports are swapped for answer!
	btfsc	zero			; If not zero, port is not 80! 
	retlw	0x01			; Port is 14501, return 0x01
	retlw	0x00			; If port is not 14501 or 80, ret 0x00

;------------------------------------------------------------------------------
tcp_test_telnet				; 
	movlw	D'23'			; TCP port 23 = TELNET protocol
	xorwf	pktdata+tcp_src_port+1,W	; Ports are swapped for answer!
	btfss	zero			;  
	retlw	0x00			; If port is not 23, return 0x00
	movf	pktdata+tcp_src_port,F	; Ensure that high nibble is zero
	btfsc	zero			; If not zero, port is not 80! 
	retlw	0x01			; Port is 23, return 0x01
	retlw	0x00			; If port is not 23, return 0x00

;------------------------------------------------------------------------------
tcp_test_SMT				; 
	movlw	0xc0			; TCP port 18624 = SMT-TELNET protocol
	xorwf	pktdata+tcp_src_port+1,W	; Ports are swapped for answer!
	btfss	zero			;  
	retlw	0x00			; If port is not 18624, return 0x00
	movlw	0x48			; TCP port 18624 = SMT-TELNET protocol
	xorwf	pktdata+tcp_src_port+0,W	; Ports are swapped for answer!
	btfsc	zero			; If not zero, port is not 18624! 
	retlw	0x01			; Port is 18624 (0x48c0), return 0x01
	retlw	0x00			; If port is not 18624, return 0x00

;------------------------------------------------------------------------------
tcp_test_aprsd_telnet			; 
	movlw	0xf1			; TCP port 18673 (my favourite :-))
	xorwf	pktdata+tcp_src_port+1,W	; Ports are swapped for answer!
	btfss	zero			;  
	retlw	0x00			; If port is not 18673, return 0x00
	movlw	0x48			; TCP port 18673 (my favourite :-))
	xorwf	pktdata+tcp_src_port+0,W	; Ports are swapped for answer!
	btfss	zero			;  
	retlw	0x00			; If port is not 18673, return 0x00
	retlw	0x01			; Port is 18673, return 0x01

;------------------------------------------------------------------------------
; TCP with HTTP port received, parse it.
;------------------------------------------------------------------------------
tcp_parse_http				; Packet is for port 80, parse it!
	ifdef	DEBUG0
	movlw	A'H'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'T'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'T'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'P'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif
	
	bcf	html_wx_rep		; Clear html WX page reply flag
	movlw	a'G'			; 
	xorwf	INDF, W			; Test if GET was issued
	btfss	zero			; 
	goto	tcp_parse_http_bad	; Unexpected HTTP request...
	incf	FSR, F			; Point to the next byte
	movlw	a'E'			; 
	xorwf	INDF, W			; Test if GET was issued
	btfss	zero			; 
	goto	tcp_parse_http_bad	; Unexpected HTTP request...
	incf	FSR, F			; Point to the next byte
	movlw	a'T'			; 
	xorwf	INDF, W			; Test if GET was issued
	btfss	zero			; 
	goto	tcp_parse_http_bad	; Unexpected HTTP request...
	incf	FSR, F			; Point to the next byte
	movlw	0x04			; 
	movwf	AA			; Look for slash in next 4 bytes
		
tcp_parse_http_slash_loop		; 
	movlw	a'/'			; Next: search for slash
	xorwf	INDF, W			; is it slash?
	btfsc	zero			; 
	goto	tcp_parse_http_get_slash	; We have got slash!
	incf	FSR, F			; Point to the next byte
	decfsz	AA, F			; Decrease counter
	goto	tcp_parse_http_slash_loop	; Loop for another byte
	goto	tcp_parse_http_bad	; Unexpected HTTP request...
	
tcp_parse_http_get_slash		; We have got slash, what is the next
	incf	FSR, F			; byte?
	movlw	0x20			; is it a space?
	xorwf	INDF, W			; 
	btfss	zero			; 
	goto	http_find_reply		; it was not space, hope it is a letter
	movlw	a'i'			; Change space to 'i' it means
	movwf	INDF			; reply with main page...

http_find_reply				; 
	movf	INDF, W			; Take first letter
	movwf	byte_tmp		; store it here
	ifdef	DEBUG0
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif
	
	movlw	HTML_FILES_H		; offset of html files in 
	movwf	adresseH		; program memory
	movlw	HTML_FILES_L		; area 
	movwf	adresseL		; 

http_find_reply_loop			;
	call	readF			; compare identificator with byte from 
	xorwf	byte_tmp, W		; received packet
	btfsc	zero			; 
	goto	http_reply_found	; we found desired file
	xorwf	byte_tmp, W		; Recover identification byte
	xorlw	0xff			; are we at the end of files section?
	btfsc	zero			; 
	goto	http_reply_not_found	; Yes, end od files reached!
	call	seek_to_next_file	; Seek for the next file
	goto	http_find_reply_loop	; and process it

http_reply_not_found			; Set the first file as an reply
	movlw	HTML_FILES_H		; offset of html files in 
	movwf	adresseH		; program memory
	movlw	HTML_FILES_L		; area 
	movwf	adresseL		; 

http_reply_found			; We are on id byte of reply file now
	call	readF			; Get a byte (id)
	xorlw	wx_page_req		; is the appropriate page WX?
	btfsc	zero			; skip if not
	bsf	html_wx_rep		; Set the flag if Yes
	call	inc_file_address	; go to the next byte
	clrf	pktdata+eth_len_l	; clear low size bit
	call	readF			; Get a byte (size high value)
	movwf	g_len_hi		; 
	movwf	pktdata+ip_len+0	; Place it as IP length in the packet
	movwf	pktdata+eth_len_h	; and length of Ethernet packet
	call	inc_file_address	; go to the next byte
	call	readF			; Get a byte (size low value)
	movwf	g_len_low		; 
	addlw	D'40'			; IP+TCP heads have 40 bytes
	btfss	carry			; if we overflowed
	goto	http_reply_set_len	; we do not overflowed
	incf	pktdata+ip_len+0, F	; increase the high byte
	incf	pktdata+eth_len_h, F	; increase the high byte

http_reply_set_len			; 
	movwf	pktdata+ip_len+1	; Place it as IP length in the packet
	addlw	D'14'			; Eth. header has next 14 bytes
	btfsc	carry			; if we overflowed
	incf	pktdata+eth_len_h, F	; increase the high byte
	movwf	pktdata+eth_len_l	; 
	movlw	0x50			; TCP header length is 5x4bytes
	movwf	pktdata+tcp_doffset	; set it here
	bcf	html_odd_len		; clear flag
	btfsc	g_len_low, 0		; is html file length odd?
	bsf	html_odd_len		; Yes, so set the flag!
	bsf	html_file_rep		; Reply includes hmtl file!
	call	inc_file_address	; go to the next byte (checksum high)
	bsf	pktdata+tcp_ctrl, 0	; Reply is one page, set FIN flag.
	movlw	D'20'			; TCP len for checksum (header only)
	call	tcp_do_cksum		; Count TCP checksum
	goto	tcp_send		; And send it
	
tcp_parse_http_bad			; Send FIN as a reply to other then
	goto	tcp_fin_send		; GET method or if slash not found
	

;------------------------------------------------------------------------------
seek_to_next_file			; Jumps to the head of the next file
	call	store_file_address	; Store actual position
	movlw	0x05			; add 5 bytes (1id+2length+2checksum)
	addwf	iolow, F		; add to low byte of address
	btfsc	carry			; if we overflowed
	incf	iohigh, F		; increment high byte

	call	inc_file_address	; Length of act file are next 2 bytes
	call	readF			; High byte of length
	addwf	iohigh, F		; add to high byte of address
	call	inc_file_address	; go to the next byte
	call	readF			; Low byte of length
	addwf	iolow, F		; add to low byte of address
	btfsc	carry			; if we overflowed
	incf	iohigh, F		; increment high byte
	call	restore_file_address	; Set stored position
	return				; All done

inc_file_address			; Increment pointer to files area
	incf	adresseL, F		; Length of act file are next 2 bytes
	btfsc	zero			; if we overflowed
	incf	adresseH, F		; increment high byte
	return				; All done

store_file_address			;
	movf	adresseH, W		; Store actual position
	movwf	iohigh			; for future use
	movf	adresseL, W		; 
	movwf	iolow			; 
	return				; All done

restore_file_address			;
	movf	iohigh, W		; Put stored position
	movwf	adresseH		; into adresse registers
	movf	iolow, W		;
	movwf	adresseL		; 
	return				; All done

;------------------------------------------------------------------------------
; TCP with SMTtelnet port received, parse it.
;------------------------------------------------------------------------------
tcp_parse_SMT				; Packet is for port 80, parse it!
	ifdef	DEBUG0
	movlw	A'S'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'M'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'T'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif

	;movlw	pktdata+tcp_data	; First reply data byte offset
	;movwf	BB			; will be here
	clrf	AA			; clear bytes counter

	page1				; Set PCLATCH to program page 1
	call	RTC_get_time		; Place time from RTC to RAM regs
	movlw	pktdata+tcp_data	; First reply data byte offset
	movwf	FSR			; place it here
	call	time_to_smt_pkt		; Place time on the top of smt packet
	movf	FSR, W			; First reply temper byte offset
	movwf	BB			; will be here
	page0				; Set PCLATCH to program page 0

	bank3				; bank3
	movlw	0x7f			; we have max 7 ports for SMTs
	andwf	smt_act, W		; Where are SMTs connected?
	movwf	TMP			; place here

tcp_parse_SMT_loop			; measure temp. from configured SMTs
	movf	TMP, W			; take configuration
	iorlw	0x00			; are some?
	btfsc	zero			; skip if Yes
	goto	tcp_parse_SMT_end	; No SMTs configured!
	
	page1				; Set PCLATCH to program page 1
	call	Start_Mereni		; Measure temperature from SMT sensors
	page0				; Set PCLATCH to program page 0
	bank3				; bank3
	movf	BB, W			; Take data offset
	movwf	FSR			; place it here
	movf	pod_nulou, W		; Take measured byte
	call	SMT_place_tempr		; Place measured byte into packet
	movf	TeplH, W		; Take measured byte
	call	SMT_place_tempr		; Place measured byte into packet
	movf	TeplL, W		; Take measured byte
	call	SMT_place_tempr		; Place measured byte into packet
	movlw	a'.'			; Take tens delimitter
	call	SMT_place_tempr		; Place it into packet
	movf	desetiny, W		; Take measured byte
	call	SMT_place_tempr		; Place measured byte into packet
	movf	FSR, W			; Take data offset
	movwf	BB			; place it here
	goto	tcp_parse_SMT_loop	; and go for the next SMT

tcp_parse_SMT_end			; 
	bank0				; bank0
	btfss	STATUS, IRP		; if in bank3, jump
	goto	tcp_parse_SMT_end2	; goto if not
	movlw	0x70			; bank3, address after the last dataB
	subwf	FSR, W			; If FSR points here, 
	btfss	zero			; jump over next clrf INDF instruction

tcp_parse_SMT_end2			; 
	clrf	INDF			; clr next byte (for odd length cksum)
	bsf	end_tcp_req		; Set Quit/disconnect flag
	goto	Send_tcp_final		; Send reply
	
SMT_place_tempr				; Places measured temperature bytes
	movwf	INDF			; to FSR position in reply packet
	incf	AA, F			; increase counters
	incf	FSR, F			; 
	call	do_indirect_bank_tests	; indirect address bank maintenance
	return				; 

;------------------------------------------------------------------------------
; TCP with APRSDtelnet port received, parse it.
;------------------------------------------------------------------------------
tcp_parse_aprsd				;
	ifdef	DEBUG0
	movlw	A'A'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'P'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'R'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'S'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'D'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif
	bsf	aprsd_act		; Set TCP connect flag! Just for sure.
	page2				; Set PCLATCH to program page 2
	goto	tcp_do_aprsd		; And go to aprsd routine
	
;------------------------------------------------------------------------------
; Set (swap) source and destination ports in TCP reply packet 
;------------------------------------------------------------------------------
reply_set_tcp_port				; Swap source and destination
	movf	pktdata+tcp_src_port, W		; ports in TCP reply packet
	movwf	TMP				; 
	movf	pktdata+tcp_dst_port, W		; 
	movwf	pktdata+tcp_src_port		; 
	movf	TMP, W				; 
	movwf	pktdata+tcp_dst_port		; 
	movf	pktdata+tcp_src_port+1, W	; 
	movwf	TMP				; 
	movf	pktdata+tcp_dst_port+1, W	; 
	movwf	pktdata+tcp_src_port+1		; 
	movf	TMP, W				; 
	movwf	pktdata+tcp_dst_port+1		; 
	return					; All done

;------------------------------------------------------------------------------
; Swap sequenece and acknowledge numbers in TCP reply packet
;------------------------------------------------------------------------------
reply_swap_tcp_seqnackn			; Swap sequence and acknowledge
	movf	pktdata+tcp_seqn, W	; numbers in TCP reply packet
	movwf	TMP			; 
	movf	pktdata+tcp_ackn, W	; 
	movwf	pktdata+tcp_seqn	; 
	movf	TMP, W			; 
	movwf	pktdata+tcp_ackn	; 
	movf	pktdata+tcp_seqn+1, W	; 
	movwf	TMP			; 
	movf	pktdata+tcp_ackn+1, W	; 
	movwf	pktdata+tcp_seqn+1	; 
	movf	TMP, W			; 
	movwf	pktdata+tcp_ackn+1	; 
	movf	pktdata+tcp_seqn+2, W	; 
	movwf	TMP			; 
	movf	pktdata+tcp_ackn+2, W	; 
	movwf	pktdata+tcp_seqn+2	; 
	movf	TMP, W			; 
	movwf	pktdata+tcp_ackn+2	; 
	movf	pktdata+tcp_seqn+3, W	; 
	movwf	TMP			; 
	movf	pktdata+tcp_ackn+3, W	; 
	movwf	pktdata+tcp_seqn+3	; 
	movf	TMP, W			; 
	movwf	pktdata+tcp_ackn+3	; 
	return				; All done

;------------------------------------------------------------------------------
; Increment acknowledgement number in TPC reply packet by number of RX bytes
;------------------------------------------------------------------------------
inc_ackn				; increments ack number by W contents
	bcf	STATUS, C		; In 32bit arithmetic!
	addwf	pktdata+tcp_ackn+3, f	; 
	btfss	carry			; Jump if overflowed
	goto	incackn_done		; 
	incfsz	pktdata+tcp_ackn+2, f	; Jump if overflowed
	goto	incackn_done		; 
	incfsz	pktdata+tcp_ackn+1, f	; Jump if overflowed
	goto	incackn_done		; 
	incfsz	pktdata+tcp_ackn+0, f	; Jump if overflowed
	goto	incackn_done		; 
	incf	pktdata+tcp_ackn+3, f	; Is this really right? Suppose Yes!
incackn_done				; 
	return				; All done
	
;------------------------------------------------------------------------------
; Set do not fragment and Time to live in TCP reply packet
;------------------------------------------------------------------------------
set_fr_ttl				; 
	movlw	0x40			; Set do not fragment flag
	movwf	pktdata+ip_froff	; 
	clrf	pktdata+ip_froff+1	; 
	movlw	0x80			; Set TTL to 128
	movwf	pktdata+ip_ttl		; 
	return


;------------------------------------------------------------------------------
; UDP packet received, continue analyzing
;------------------------------------------------------------------------------

do_udp					; expecting 8-byte packet on any port
	ifdef	DEBUG0
	movlw	A'U'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'D'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'P'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif

	call	ip_check_IP		; Verify if packet is for my IP
	xorlw	0x01			; 0x01 means Ok
	skpz				; 
	return				; IP differs, return 

	call	udp_test_daytime	; Test if port is DayTime
	xorlw	0x01			; Returns 0x01 if Yes.
	btfsc	zero			; If not zero, port is not DayTime! 
	goto	udp_parse_dtime		; Else parse daytime packet.
	call	udp_test_snmp		; Test if port = SNMP
	return				; All done

udp_parse_dtime				; DayTime datagram received, parse it
	ifdef	DEBUG0
	movlw	A'D'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'T'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	call	serialTXcr		; For debug
	endif
	page2				; Set PCLATCH to program page 2
	goto	UDP_synchonize_RTC	; And go to synchronization routine
	
	

	
	;movf	pktdata+udp_data,W		; Take the first UDP data byte
	;xorlw 'S'				; is it instruction SET?
	;btfsc zero				; skip if not
	;goto setsorties				; Process command SET
	;xorlw 'S'				; Restore original value
	;xorlw 'R'				; is it instruction READ?
	;btfsc zero				; skip if not
	;goto read				; Process command READ
	;return					; All done

;------------------------------------------------------------------------------
udp_test_daytime			; Is received UDP daytime datagram?
	movlw	EE_DTIME_PORTL		; Load dtime server port
	page1				; Set PCLATCH to program page 1
	call	EEPROM_Read_Byte	; 
	page0				; Set PCLATCH to program page 0
	xorwf	pktdata+udp_src+1,W	;
	btfss	zero			;  If not zero, port is different!
	retlw	0x00			; If port does not equals EEPROM exit
	page1				; Set PCLATCH to program page 1
	movlw	EE_DTIME_PORTH		; Load dtime server port
	call	EEPROM_Read_Byte	; 
	page0				; Set PCLATCH to program page 0
	xorwf	pktdata+udp_src+0,W	;
	btfss	zero			; If not zero, port is different!
	retlw	0x00			; If port does not equals EEPROM exit
	retlw	0x01			; Port is EEPROM dtime, return 0x01

;------------------------------------------------------------------------------
udp_test_snmp				; Is received UDP SNMP datagram?
	retlw	0x00			; If port does not equals set 0x00

;------------------------------------------------------------------------------
; La trame est constitue, l'envoyer
;------------------------------------------------------------------------------

reponse:
	call	reply_set_mac			; Set source and dest MAC
	call	reply_set_ip			; Set source and dest IP
	call	reply_set_udp_port		; Set destination UDP port
	call	udp_do_cksum			; Count and fill UDP checksum
	call	send_pkt			; Send reply packet
	return					; All done

;------------------------------------------------------------------------------
; positionnement des octets de la zone UDP
;------------------------------------------------------------------------------

reply_set_udp_port				; Set UDP port in reply packet
        movf    pktdata+udp_src,W		; Take the source port
        movwf   pktdata+udp_dst			; and set it as destination
        movf    pktdata+udp_src+1,W		; Take the source port
        movwf   pktdata+udp_dst+1		; and set it as destination
	return					; All done


;------------------------------------------------------------------------------
; count IP checksum of IP header
;------------------------------------------------------------------------------
ip_do_cksum					; Calculate IP checksum
	clrf	pktdata+ip_cksum		; Clear IP checksum bytes
	clrf	pktdata+ip_cksum+1		; 
	movlw	pktdata+ip_verlen		; do ip checksum
	movwf	FSR				; start of IP part into FSR
	movlw	D'20'				; length into W
	call	oc_startsum			; Calculate checksum
	call	oc_getsum			; Get checksum (into AA and BB)
	movf	BB,W				; Place checksum bytes
	movwf	pktdata+ip_cksum		; to it's places
	movf	AA,W				; in IP part of the
	movwf	pktdata+ip_cksum+1		; packet
	return					; All done

;------------------------------------------------------------------------------
; count IP and TCP checksums of TCP packet
;------------------------------------------------------------------------------
tcp_do_cksum					; Calculate IP and TCP checksum
	movwf	TMP				; TCP data length
	clrf 	pktdata+ip_cksum		; clear IP checksum 
	clrf 	pktdata+ip_cksum+1		; 
	call	ip_do_cksum			; Calculate IP checksum
	clrf	pktdata+tcp_cksum		; Clear TCP checksum bytes
	clrf	pktdata+tcp_cksum+1		; 
	bcf	tcp_len_ovfl			; Clear the flag!

				; Count TCP pseudo header checksum start
	movlw	pktdata+ip_src			; ip src & dst addres into
	movwf	FSR				; FSR (for TCP part checksum)
	movlw	D'8'				; length is 8 bytes
	call	oc_startsum			; Calculate checksum

tcp_protosum					; Protocol value to checksum
	movf	pktdata+ip_prot,W		; Take IP protocol value
	addwf	AA,f				; add it to AA
	skpc					; if overflowed
	goto	tcp_lensum			; if not owerflowed, continue
	movlw	0x01				; add 1 to 
	addwf	BB,f				; BB
	skpnc					; if not overflowed, continue
	incf	CC,f				; if overflowed, add 1 to CC

tcp_lensum					; Add length value to checksum
	movlw	D'20'				; IP header length
	subwf	pktdata+ip_len+1, W		; Take TPC part length
	btfss	carry				; if we overflowed
	bsf	tcp_len_ovfl			; set the flag!
	addwf	AA,f				; add it to AA
	skpc					; if overflowed
	goto	tcp_len_hi_sum			; if not owerflowed, continue
	movlw	0x01				; add 1 to 
	addwf	BB,f				; BB
	skpnc					; if not overflowed, continue
	incf	CC,f				; if overflowed, add 1 to CC

tcp_len_hi_sum					; 
	movlw	0x01				; Substract one from 
	subwf	pktdata+ip_len+0, W		; TPC part length high byte
	btfss	tcp_len_ovfl			; do we overflowed in low byte?
	addlw	0x01				; If not, add one back!
	addwf	BB,f				; BB
	skpnc					; if not overflowed, continue
	incf	CC,f				; if overflowed, add 1 to CC
				; Count TCP pseudo header checksum end

tcp_pktsum					; continue data checksumming
	movlw	pktdata+tcp_src_port		; Place the first TCP byte 
	movwf	FSR				; into FSR
	movf	TMP,W				; length of the TCP part into W
	call	oc_cumsum			; Calculate checksum

	btfsc	html_file_rep			; if reply is html file, go 
	goto	tcp_html_file_sum		; to the appropriate section
	goto	tcp_getsum			; Otherwise get checksum

tcp_html_file_sum				; Add precounted file chksum
	call	readF				; Get a byte (chksum hi value)
	addwf	BB,f				; BB
	skpnc					; if not overflowed, continue
	incf	CC,f				; if overflowed, add 1 to CC
	call	inc_file_address		; go to the next byte
	call	readF				; Get a byte (chksum lo value)
	addwf	AA,f				; add it to AA
	skpc					; if overflowed
	goto	tcp_html_file_sum_done		; if not owerflowed, continue
	movlw	0x01				; add 1 to 
	addwf	BB,f				; BB
	skpnc					; if not overflowed, continue
	incf	CC,f				; if overflowed, add 1 to CC

tcp_html_file_sum_done				; Set address to 1st file byte
	call	inc_file_address		; go to the next byte does it
				; count checksum for dynamicly filled (WX) bytes
	btfss	html_wx_rep			; Is the reply html WX page?
	goto	tcp_getsum			; No, be happy!
	
	movf	g_len_low, W			; Take stored file size
	movwf	g_val_low			; and place it here
	movf	g_len_hi, W			; for seeking through the
	movwf	g_val_hi			; file
	call	store_file_address		; Store actual position
	incf	g_val_hi, F			; for decfsz cycle!

tcp_tempc_sum					; 
	call	seek_wx_pattern			; find the first wx pattern
	xorlw	0x01				; was pattern found?
	btfss	zero				; skip if yes
	goto	tcp_getsum			; Pattern was not found in file
	call	wx_pattern_odd_even		; Returns 0x00=even, 0x01=odd
	xorlw	0x01				; 
	btfsc	zero				; 
	goto	tcp_tempc_sum_odd		; 
tcp_tempc_sum_even				;
	bank3					; bank3
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_tempc+0, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_tempc+1, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_tempc+2, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
		; next byte is tens delimitter!
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_tempc+3, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
	goto	tcp_humi_sum			; 
tcp_tempc_sum_odd				;
	bank3					; bank3
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_tempc+0, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_tempc+1, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_tempc+2, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
		; next byte is tens delimitter!
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_tempc+3, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
	
tcp_humi_sum					; 
	bank0					; bank0
	movlw	0x05				; skip 5 tempc bytes
	movwf	dataF				; 
        call	inc_file_address		; go to the next byte
	decfsz	dataF, F			; 
	goto	$-2				; 
	call	seek_wx_pattern			; find the first wx pattern
	xorlw	0x01				; was pattern found?
	btfss	zero				; skip if yes
	goto	tcp_getsum			; Pattern was not found in file
	call	wx_pattern_odd_even		; Returns 0x00=even, 0x01=odd
	xorlw	0x01				; 
	btfsc	zero				; 
	goto	tcp_humi_sum_odd		; 
tcp_humi_sum_even				;
	bank3					; bank3
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_humi+0, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_humi+1, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
	goto	tcp_press_sum			; 
tcp_humi_sum_odd				;
	bank3					; bank3
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_humi+0, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_humi+1, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
	
tcp_press_sum					; 
	bank0					; bank0
        call	inc_file_address		; go to the next byte
        call	inc_file_address		; go to the next byte
	call	seek_wx_pattern			; find the first wx pattern
	xorlw	0x01				; was pattern found?
	btfss	zero				; skip if yes
	goto	tcp_getsum			; Pattern was not found in file
	call	wx_pattern_odd_even		; Returns 0x00=even, 0x01=odd
	xorlw	0x01				; 
	btfsc	zero				; 
	goto	tcp_press_sum_odd		; 
tcp_press_sum_even				;
	bank3					; bank3
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_press+0, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_press+1, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_press+2, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_press+3, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
		; next byte is tens delimitter!
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_press+4, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
	goto	tcp_getsum			; 
tcp_press_sum_odd				;
	bank3					; bank3
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_press+0, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_press+1, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_press+2, W		; substract pattern for cksum
	call	tcp_add_odd_sum			; 
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_press+3, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
		; next byte is tens delimitter!
	movlw	wx_pattern			; Take a byte from WX RAM, but
	subwf	wx_data+wx_press+4, W		; substract pattern for cksum
	call	tcp_add_even_sum		; 
	goto	tcp_getsum			; 
	

tcp_getsum					; And finally get calculated
	bank0					; bank0
	btfsc	html_wx_rep			; Is the reply html WX page?
	call	restore_file_address		; Position to the top of file
	call	oc_getsum			; checksum and palce it
	movf	BB,W				; into the TCP checksum
	movwf	pktdata+tcp_cksum		; bytes in the
	movf	AA,W				; reply
	movwf	pktdata+tcp_cksum+1		; packet
	return					; Checksums in place, return

;------------------------------------------------------------------------------
tcp_add_odd_sum					; Add WX byte to checksum
	addwf	AA,f				; it lays on the odd byte 
	skpc					; in the tcp data area
	return					; 
	movlw	0x01				; 
	addwf	BB,f				; 
	skpnc					; 
	incf	CC,f				; 
	return					; 

;------------------------------------------------------------------------------
tcp_add_even_sum				; Add WX byte to checksum
	addwf	BB,f				; it lays on the even byte
	skpc					; in the tcp datat area
	return					; 
	incf	CC,f				; 
	return					; 

;------------------------------------------------------------------------------
wx_pattern_odd_even				; In which byte is the pattern?
	btfsc	adresseL, 0			; Is pattern in odd byte?
	goto	tcp_wx_seek_odd			; Yes, is odd!
	btfsc	iolow, 0			; Starts html file on odd byte?
	goto	tcp_wx_start_odd		; Yes, file starts on odd byte!
	retlw	0x00				; Start, Seek are even...

tcp_wx_start_odd				; Start=odd, Seek=even
	retlw	0x01				; 

tcp_wx_seek_odd					; Seek=even
	btfss	iolow, 0			; Starts html file on odd byte?
	goto	tcp_wx_start_even_seek_odd	; Yes, file starts on odd byte!
	retlw	0x00				; Start, Seek are odd

tcp_wx_start_even_seek_odd			; Start=even, Seek=odd
	retlw	0x01				; 
	
;------------------------------------------------------------------------------
seek_wx_pattern					; 
	call	readF				; read a byte from file
	xorlw	wx_pattern			; is this byte wx_pattern??
	btfsc	zero				; 
	retlw	0x01				; If pattern is found return 1
        call	inc_file_address		; go to the next byte
	decfsz	g_val_low, F			; decrease file length
	goto	seek_wx_pattern			; and go for the next byte
	decfsz	g_val_hi, F			; decrease file length
	goto	seek_wx_pattern			; and go for the next byte
	call	restore_file_address		; Position to the top of file
	retlw	0x00				; Pattern was not found!
	
;------------------------------------------------------------------------------
; count IP and UDP checksums of UDP packet
;------------------------------------------------------------------------------
udp_do_cksum					; Calculate IP and UDP checksum
	call	ip_do_cksum			; Calculate IP checksum
	clrf	pktdata+udp_cksum		; Clear UDP checksum bytes
	clrf	pktdata+udp_cksum+1		; 

				; Count UDP pseudo header checksum start
	movlw	pktdata+ip_src			; ip src & dst addres into
	movwf	FSR				; FSR (for UDP part checksum)
	movlw	D'8'				; length is 8 bytes
	call	oc_startsum			; Calculate checksum

udp_protosum					; Protocol value to checksum
	movf	pktdata+ip_prot,W		; Take IP protocol value
	addwf	AA,f				; add it to AA
	skpc					; if overflowed
	goto	udp_lensum			; if not owerflowed, continue
	movlw	0x01				; add 1 to 
	addwf	BB,f				; BB
	skpnc					; if not overflowed, continue
	incf	CC,f				; if overflowed, add 1 to CC

udp_lensum					; Add length value to checksum
	movlw	pktdata+udp_len			; UDP length place in
	movwf	FSR				; FSR
	movlw	0x02				; length is 2 bytes
	call	oc_cumsum			; Calculate checksum and
				; Count UDP pseudo header checksum end

udp_pktsum					; continue data checksumming
	movlw	pktdata+udp_src			; Place the first UDP byte 
	movwf	FSR				; into FSR
	movf	pktdata+udp_len+1,W		; length of the UDP part into W
	call	oc_cumsum			; Calculate checksum

udp_getsum					; And finally get calculated
	call	oc_getsum			; checksum and palce it
	movf	BB,W				; into the UDP checksum
	movwf	pktdata+udp_cksum		; bytes in the
	movf	AA,W				; reply
	movwf	pktdata+udp_cksum+1		; packet
	return					; Checksums in place, return

;------------------------------------------------------------------------------
; Indirect address bank maintenance for packet
;------------------------------------------------------------------------------
do_indirect_bank_tests			;
	btfsc	STATUS, IRP		; if we are in bank2 do nothing
	return				; because bank3 is not used for pktdata
	movlw	0x60			; 0x5f is the last pktdata byte in
	subwf	FSR, W			; bank0
	btfss	zero			; if Z=1, zero substraction, we
	goto	indi_bank2_test		; 
	movlw	pktdata2		; have to switch to bank1
	movwf	FSR			; 

indi_bank2_test				; 
	movlw	0xf0			; 0xef is the last pktdata byte in
	subwf	FSR, W			; bank1
	btfss	zero			; if Z=1, zero substraction, we
	return				; All done
	movlw	pktdata3		; have to switch to bank2
	movwf	FSR			; 
	bsf	STATUS, IRP		; Set indirect page 2+3
	return				; All done

;------------------------------------------------------------------------------
; Send prepared packet to Ethernet card
;------------------------------------------------------------------------------
send_pkt

	ifdef	DEBUG0
	movlw	A'T'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'X'			;
	call	serial_SEND_byte	; For debug 
	movlw	A':'			;
	call	serial_SEND_byte	; For debug 
	endif
	
	page1				; Set PCLATCH to program page 1
	call	set_zero_io		; set address to I/O + 0
	page0				; Set PCLATCH to program page 0
        movf    pktdata+eth_len_l,W	; First send the packet
	movwf	byte_tmp		; Low byte here 
	ifdef	DEBUG2
	call	serial_SEND_byte	; For debug 
        movf    pktdata+eth_len_h,W	; 
	call	serial_SEND_byte	; For debug 
	endif
        movf    pktdata+eth_len_h,W	; High byte here
	call	OutWord			; First send the packet
	movlw	0x00			; Next send two zero bytes
	movwf	byte_tmp		;
	ifdef	DEBUG2
	call	serial_SEND_byte	; For debug 
	movlw	0x00			; 
	call	serial_SEND_byte	; For debug 
	endif
	movlw	0x00			; 
	call	OutWord			; Next send two zero bytes

	bcf	zero			; clear zero bit
	btfsc	pktdata+eth_len_l, 0	; is length odd?? make it even because
	incf	pktdata+eth_len_l, F	; we are sending words to card, not
	btfsc	zero			; bytes. Last 8 data bits should be
	incf	pktdata+eth_len_h, F	; cleared before in upper OSI level

        movlw   pktdata+eth_dest_0	; Address of the dest. put
        movwf   FSR			; in the FSR, for loop sending
        movf    pktdata+eth_len_l,W	; Packet size put
        movwf   g_len_low		; here
        movf    pktdata+eth_len_h,W	; Packet size put
        movwf   g_len_hi		; here
	clrf	g_val_low		; and clear bytes send counter
	clrf	g_val_hi		; and clear bytes send counter
	bcf	html_pkt_snd_f		; Clear flag for sending html filebytes
	btfss	html_wx_rep		; If sending WX page skip
	goto	send_pkt_lp1		; If not sending WX page, continue
	clrf	dataF			; clear pointer into WX data
	
	
send_pkt_lp1				; Main sending loop
	btfsc	html_pkt_snd_f		; If we are sending html file
	goto	send_pkt_html_file	; go to the appropriate part
        movf    INDF,W			; take a byte for sending
        incf    FSR,F			; increase pointer
	movwf	byte_tmp		;
	ifdef	DEBUG2
	call	serial_SEND_byte	; For debug 
        movf    INDF,W			; take a byte for sending
	call	serial_SEND_byte	; For debug 
	endif
        movf    INDF,W			; take a byte for sending
        incf    FSR,F			; increase pointer
	call	OutWord			; send the word

	call	do_indirect_bank_tests	; indirect address bank maintenance
	goto	tx_len_tests		; and go to the length test

send_pkt_html_file			; Sends contents of html file
	call	readF			; Read a byte from file
	btfsc	html_wx_rep		; If sending WX page test if
	call	wx_replace_byte		; we have to replace char with WX data
	movwf	byte_tmp		; 
	call	inc_file_address	; go to the next byte
	btfsc	html_odd_len		; Has html file odd length?
	goto	send_pkt_html_test_odd	; Yes,test if we are at the last 2bytes

send_pkt_html_read_hi_byte		; 
	call	readF			; Read a byte from file
	btfsc	html_wx_rep		; If sending WX page test if
	call	wx_replace_byte		; we have to replace char with WX data 
	call	inc_file_address	; go to the next byte
	goto	send_html_file_word	; and send 2 bytes to the card

			; If odd length of html file, we have to clear last
			; 8 bits in the last word sent! 
send_pkt_html_test_odd			; Test if we are at the last 2 bytes
	movf	g_len_hi,f		; is high byte zero?
	btfss	zero			; Skip if Yes
	goto	send_pkt_html_read_hi_byte	; High byte is not zero!
	movlw	0x02			; High byte is zero, equals low size
	xorwf	g_len_low, W		; byte to 2? Are we at the end of file?
	btfss	zero			; 
	goto	send_pkt_html_read_hi_byte	; Low size is not 0x02!
	movlw	0x00			; Low size byte is 0x02,send 0x00 byte!

send_html_file_word			; Sends a word from html file
	call	OutWord			; send the word

tx_len_tests				; 
	movlw	0x02			; add sended 2 bytes to counter
	addwf	g_val_low,f		; increase counter
	btfsc	carry			; if we overflowed
	incf	g_val_hi, f		; increase high byte
	movlw	0x02			; 
	subwf	g_len_low,f		; decrease counter by sended 2 bytes
	btfss	carry			; if we overflowed
	decf	g_len_hi, f		; decrease low byte
	btfss	html_file_rep		; Includes reply hmtl file?
	goto	tx_len_tests2		; no, html file is not in reply
	btfsc	html_pkt_snd_f		; If flag is already set, continue
	goto	tx_len_tests2		; with length tests
	movlw	D'54'			; 14MAC+20IP+20TCP bytes
	subwf	g_val_low, W		; substract it from sent bytes
	btfsc	carry			; 
	bsf	html_pkt_snd_f		; Set flag, MAC,IP,TCP head sent...

tx_len_tests2				; 
	movf	g_len_hi,f		; is high byte zero?
	btfss	zero			; 
	goto	send_pkt_lp1		; no, high byte is not zero
	movf	g_len_low,f		; is low byte zero?
	btfss	zero			; 
	goto	send_pkt_lp1		; no, there are still unsend bytes

send_pkt_pad				; 
	movf	g_val_low,W		; Take count of the bytes 
	andlw	0x03			; sended, mask 3 from this 
	bz	send_pkt_exit		; if result is zero => end
	movlw	0x00			; else send one more byte
	movwf	byte_tmp		;
	ifdef	DEBUG2
	call	serial_SEND_byte	; For debug 
	movlw	0x00			; else send one more byte
	call	serial_SEND_byte	; For debug 
	endif
	movlw	0x00			; else send one more byte
	call	OutWord			; containing 0x00 value
	incf	g_val_low,f		; increase counter 
	incf	g_val_low,f		; increase counter 
	goto	send_pkt_pad		; loop until packet_size%4 is not zero

send_pkt_exit				; the whole packet was sent
	bcf	STATUS, IRP		; Set indirect page 0+1
	ifdef	DEBUG0
	call	serialTXcr		; For debug
	endif
        return				; All done

wx_replace_byte				; Is WX pattern in W? If Yes,replace it
	xorlw	wx_pattern		; Take the pattern
	btfss	zero			; Skip if pattern found
	goto	wx_no_replace		; Pattern not found
	movlw	wx_bytes_to_replace	; Take max replace count
	subwf	dataF, W		; Do we reach the maximum number?
	btfsc	carry			; skip if not
	goto	wx_replace_end		; Do not try to replace next byte!
	movf	dataF, W		; 
	bank3				; bank3
	call	wx_byte_table		; Get the right WX byte
	bank0				; bank0
	incf	dataF, F		; increase pointer

wx_replace_end				; 
        return				; All done

wx_no_replace				; No pattern found, 
	xorlw	wx_pattern		; restore byte
        return				; All done
	

;------------------------------------------------------------------------------
; ARP Request packet received, process the response to it
;------------------------------------------------------------------------------
do_arp
	ifdef	DEBUG0
	movlw	A'A'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'R'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'P'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif

	movf	pktdata+arp_op+1,W	; Take low arp Opcode byte
	sublw	0x01			; is it 1 (ARP Request)?
	skpz				; 
	goto	do_arp_rep		; If not, check reply
	call	arp_check_IP		; Verify if packet is for my IP
	xorlw	0x01			; 0x01 means Ok
	skpz				; 
	return				; IP differs, return 

	call	reply_set_mac		; Request is for me, set my MAC
	clrf	pktdata+arp_op+0	; Clear high byte
	movlw	0x02			; Set low byte to 0x02
	movwf	pktdata+arp_op+1	; It means ARP Reply
	movf	pktdata+arp_src_hw+0,W	; Place source HW address
	movwf	pktdata+arp_dst_hw+0	; into destination HW address
	movf	pktdata+arp_src_hw+1,W	; This is done in the ARP data
	movwf	pktdata+arp_dst_hw+1	; HW address area in the
	movf	pktdata+arp_src_hw+2,W	; ARP packet
	movwf	pktdata+arp_dst_hw+2	; 
	movf	pktdata+arp_src_hw+3,W	; 
	movwf	pktdata+arp_dst_hw+3	; 
	movf	pktdata+arp_src_hw+4,W	; 
	movwf	pktdata+arp_dst_hw+4	; 
	movf	pktdata+arp_src_hw+5,W	; 
	movwf	pktdata+arp_dst_hw+5	; 
	movf	pktdata+arp_src_ip+0,W	; Place source IP address
	movwf	pktdata+arp_dst_ip+0	; into destination place
	movf	pktdata+arp_src_ip+1,W	; This is done in the ARP data
	movwf	pktdata+arp_dst_ip+1	; Protocol address area in the
	movf	pktdata+arp_src_ip+2,W	; ARP packet
	movwf	pktdata+arp_dst_ip+2	; 
	movf	pktdata+arp_src_ip+3,W	; 
	movwf	pktdata+arp_dst_ip+3	; 
	movf	MAC0,W			; 
	movwf	pktdata+arp_src_hw+0	; Place my MAC address into
	movf	MAC1,W			; ARP source HW address 
	movwf	pktdata+arp_src_hw+1	; fields
	movf	MAC2,W			; 
	movwf	pktdata+arp_src_hw+2	; 
	movf	MAC3,W			; 
	movwf	pktdata+arp_src_hw+3	; 
	movf	MAC4,W			; 
	movwf	pktdata+arp_src_hw+4	; 
	movf	MAC5,W			; 
	movwf	pktdata+arp_src_hw+5	; 
	movf	IP0,W			; 
	movwf	pktdata+arp_src_ip+0	; Place my IP address into
	movf	IP1,W			; ARP source Protocol address
	movwf	pktdata+arp_src_ip+1	; fields
	movf	IP2,W			; 
	movwf	pktdata+arp_src_ip+2	; 
	movf	IP3,W			; 
	movwf	pktdata+arp_src_ip+3	; 
	movlw	D'42'			; Size of the response packet
	movwf	pktdata+eth_len_l	; is 42 bytes, set it in the
	clrf	pktdata+eth_len_h	; MAC (Ethernet) header
	call	send_pkt		; Send pckt (without checksum)
	return				; All done

;------------------------------------------------------------------------------
; ARP Reply packet received, process the response from it
;------------------------------------------------------------------------------
do_arp_rep				; Process ARP reply packet
	movf	pktdata+arp_op+1,W	; Take low arp Opcode byte
	sublw	0x02			; is it 2 (ARP Reply)?
	skpz				; 
	return				; If not, return
	call	arp_check_IP		; Verify if packet is for my IP
	xorlw	0x01			; 0x01 means Ok
	skpz				; 
	return				; IP differs, return 

	bsf	STATUS, IRP		; set indirect page 2+3
	movlw	GW_IP0			; Get Gateway IP location (in bank2)
	movwf	FSR			; Set indirect access to it
	movf	pktdata+arp_src_ip,W	; Verify that arp packet is 
	subwf	INDF,W			; response for my GW IP
	skpz				; 
	goto	arp_rep_end		; IP differs, return
	incf	FSR, F			; 
	movf	pktdata+arp_src_ip+1,W	; 
	subwf	INDF,W			; 
	skpz				; 
	goto	arp_rep_end		; IP differs, return 
	incf	FSR, F			; 
	movf	pktdata+arp_src_ip+2,W	; 
	subwf	INDF,W			; 
	skpz				; 
	goto	arp_rep_end		; IP differs, return 
	incf	FSR, F			; 
	movf	pktdata+arp_src_ip+3,W	; 
	subwf	INDF,W			; 
	skpz				; 
	goto	arp_rep_end		; IP differs, return 

	movlw	GW_MAC0			; Get Gateway MAC location (in bank2)
	movwf	FSR			; Set indirect access to it
	movf	pktdata+arp_src_hw+0, W	; Place my MAC address into
	movwf	INDF			; ARP source HW address 
	incf	FSR, F			; 
	movf	pktdata+arp_src_hw+1, W	; fields
	movwf	INDF			; 
	incf	FSR, F			; 
	movf	pktdata+arp_src_hw+2, W	; 
	movwf	INDF			; 
	incf	FSR, F			; 
	movf	pktdata+arp_src_hw+3, W	; 
	movwf	INDF			; 
	incf	FSR, F			; 
	movf	pktdata+arp_src_hw+4, W	; 
	movwf	INDF			; 
	incf	FSR, F			; 
	movf	pktdata+arp_src_hw+5, W	; 
	movwf	INDF			; 
	incf	FSR, F			; 
	bsf	gw_mac_ok		; Set flag, that we have GW's MAC
	bank3				; 
	clrf	tmr_counter		; clear owerflow counter
	bank0				; 

	ifdef	DEBUG0
	movlw	A'G'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'W'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	call	serialTXcr		; 
	endif

	ifdef	DEBUG3
	movlw	0x10			; 
	movwf	FSR			; 
	bsf	STATUS, IRP		; indirect bank 2+3
	clrf	CC			; 
	page1				; Set PCLATCH to program page 1
	call	send_RAM_reg_loop	; 
	page0				; Set PCLATCH to program page 0
	endif

arp_rep_end				;
	bcf	STATUS, IRP		; set indirect page 0+1
	return				; All done


;------------------------------------------------------------------------------
arp_check_IP				; 
	movf	pktdata+arp_dst_ip,W	; Verify that arp packet is 
	subwf	IP0,W			; really for my IP
	skpz				; 
	retlw	0x00			; IP differs, return
	movf	pktdata+arp_dst_ip+1,W	; 
	subwf	IP1,W			; 
	skpz				; 
	retlw	0x00			; IP differs, return 
	movf	pktdata+arp_dst_ip+2,W	; 
	subwf	IP2,W			; 
	skpz				; 
	retlw	0x00			; IP differs, return 
	movf	pktdata+arp_dst_ip+3,W	; 
	subwf	IP3,W			; 
	skpz				; 
	retlw	0x00			; IP differs, return 
	retlw	0x01			; IP is mine!

;------------------------------------------------------------------------------
; ARP Reply packet received, process the response from it
;------------------------------------------------------------------------------
do_arp_gw_req				; Construct ARP request packet
	call	clear_packet_area	; Clear RX packet RAM area
	movlw	0xff			; Destination MAC is broadcast
	movwf	pktdata+eth_dest_0	; 
	movwf	pktdata+eth_dest_1	; 
	movwf	pktdata+eth_dest_2	; 
	movwf	pktdata+eth_dest_3	; 
	movwf	pktdata+eth_dest_4	; 
	movwf	pktdata+eth_dest_5	; 
	call	reply_set_my_mac	; Set my MAC as source in Eth part
	movlw	0x08			; Set Eth type to ARP
	movwf	pktdata+eth_typ_0	; 
	movlw	0x06			; Set Eth type to ARP
	movwf	pktdata+eth_typ_1	; 
	movlw	0x00			; Set Hw type to Ethernet
	movwf	pktdata+arp_hw+0	; 
	movlw	0x01			; Set Hw type to Ethernet
	movwf	pktdata+arp_hw+1	; 
	movlw	0x08			; Set Protocol type to IP
	movwf	pktdata+arp_prot+0	; 
	movlw	0x00			; Set Protocol type to IP
	movwf	pktdata+arp_prot+1	; 
	movlw	0x06			; Set Hw address length
	movwf	pktdata+arp_hwlen	; 
	movlw	0x04			; Set Protocol address length
	movwf	pktdata+arp_protlen	; 
	movlw	0x00			; Set ARP Opcode
	movwf	pktdata+arp_op+0	; 
	movlw	0x01			; Set ARP Opcode as request
	movwf	pktdata+arp_op+1	; 
	movf	pktdata+eth_src_0, W	; Set my MAC as ARP hw source
	movwf	pktdata+arp_src_hw+0	; 
	movf	pktdata+eth_src_1, W	; 
	movwf	pktdata+arp_src_hw+1	; 
	movf	pktdata+eth_src_2, W	; 
	movwf	pktdata+arp_src_hw+2	; 
	movf	pktdata+eth_src_3, W	; 
	movwf	pktdata+arp_src_hw+3	; 
	movf	pktdata+eth_src_4, W	; 
	movwf	pktdata+arp_src_hw+4	; 
	movf	pktdata+eth_src_5, W	; 
	movwf	pktdata+arp_src_hw+5	; 
	movf	IP0, W			; Set my IP as ARP protocol source
	movwf	pktdata+arp_src_ip+0	; 
	movf	IP1, W			; 
	movwf	pktdata+arp_src_ip+1	; 
	movf	IP2, W			; 
	movwf	pktdata+arp_src_ip+2	; 
	movf	IP3, W			; 
	movwf	pktdata+arp_src_ip+3	; 
	bsf	STATUS, IRP		; set indirect page 2+3
	movlw	GW_IP0			; Get Gateway IP location (in bank2)
	movwf	FSR			; Set indirect access to it
	movf	INDF, W			; Set Gateway IP as ARP protocol dest
	movwf	pktdata+arp_dst_ip+0	; 
	incf	FSR, F			; 
	movf	INDF, W			;
	movwf	pktdata+arp_dst_ip+1	; 
	incf	FSR, F			; 
	movf	INDF, W			; 
	movwf	pktdata+arp_dst_ip+2	; 
	incf	FSR, F			; 
	movf	INDF, W			; 
	movwf	pktdata+arp_dst_ip+3	; 
	incf	FSR, F			; 
	bcf	STATUS, IRP		; set indirect page 0+1
	clrf	pktdata+eth_len_h	; Set packet length to
	movlw	D'42'			; 42 bytes
	movwf	pktdata+eth_len_l	;
	
	call	send_pkt		; Send prepared packet 
	return				; All done

;##################################################################3
;
; Start of 3c509-related subroutines
;
;##################################################################3

;------------------------------------------------------------------------------
; Send a Word or a Byte to 3c509
;------------------------------------------------------------------------------
OutWord					; Writes a word to card
	movwf	TMP			; store byte
	bank1				; bank1

	movlw	0x00			; Set ports
	movwf	TRISB			; as output
	bcf	STATUS, RP0		; bank0
	movf	TMP,W			; restore byte
	movwf	PORTB			; copy it to port B
	movf	byte_tmp,W		; restore byte
	
OutByte					; Writes a byte to card
	movwf	TMP			; store byte
	bank1				; bank1
	movlw	0x00			; Set port D
	movwf	TRISD			; as output
	bcf	STATUS, RP0		; bank0
	movf	TMP,W			; restore byte
	movwf	PORTD			; copy it to port D
	nop				; Wait a while
	nop				; 
	nop				; 
	nop				; 
	nop				; 
	bcf	iow			; Hi card, a byte is ready for You...
	ifdef	XTAL4MHZ
	movlw	D'15'			; Wait 50us (at 4MHz Xtal)
	endif
	ifdef	XTAL10MHZ
	movlw	D'38'			; Wait 50us (at 10MHz Xtal)
	endif
	ifdef	XTAL12MHZ
	movlw	D'48'			; Wait 50us (at 12MHz Xtal)
	endif
	ifdef	XTAL20MHZ
	movlw	D'75'			; Wait 50us (at 20MHz Xtal)
	endif
	movwf	AA			; 
obA1:	decfsz	AA,f			; 
	goto 	obA1			; 
	
	bsf	iow			; Clear "byte is ready for write" signal
	nop				; Wait a while
	nop				; 
	nop				; 
	nop				; 

	movlw	0xff			; Set port D as input
	bank1				; bank1
	movwf TRISD			; set port D
	movwf TRISB			; set port B

	bcf	STATUS, RP0		; bank0

	return				; All done

;------------------------------------------------------------------------------
; Get a Word or a Byte from 3c509
;------------------------------------------------------------------------------
InByte					; Reads a byte from card
	movlw	0xff			; Set port D as input
	bank1				; bank1
	movwf TRISD			; set port D
	movwf TRISB			; set port B

	bcf	STATUS, RP0		; bank0

	nop				; Wait a while
	nop				; 
	nop				; 
	nop				; 
	nop				; 
	nop				; 
	bcf	ior			; Hi card, I am about to read Your byte

	ifdef	XTAL4MHZ
	movlw	D'15'			; Wait 50us (at 4MHz Xtal)
	endif
	ifdef	XTAL10MHZ
	movlw	D'38'			; Wait 50us (at 10MHz Xtal)
	endif
	ifdef	XTAL12MHZ
	movlw	D'48'			; Wait 50us (at 12MHz Xtal)
	endif
	ifdef	XTAL20MHZ
	movlw	D'75'			; Wait 50us (at 20MHz Xtal)
	endif
	movwf	AA			; 
ibA1:	decfsz	AA,f			; 
	goto 	ibA1			; 

	movf	PORTB,W			; Read a byte from card
	movwf	byte_tmp		; And store it here
	movf	PORTD,W			; Read a byte from card

	nop				; Wait a while
	bsf	ior			; Byte was readed, clear read flag.
	nop				; Wait a while
	nop				; 
	nop				; 
	nop				; 
	return				; All done
	

;------------------------------------------------------------------------------
rx_complete			; Is RX active? Int should be in HIGH state.
	btfss	int		; Test the Interrupt line
	retlw	0x00		; Nothing received!

	page1			; Set PCLATCH to program page 1
	call	set_command_io	; set address I/O + 0E (Commnad)
	page0			; Set PCLATCH to program page 0
	call	InByte		; Get Interrupt status
	movwf	TMP		; save status
	;movlw	0x30		; RX complete (0x10) + RX early (0x20)
	movwf	byte_tmp	; Low byte here
	movlw	0x68		; Send 0x68 (acknowledge interrupt)
	call	OutWord		; to address I/O + 0E
	movlw	0x10		; RX complete 
	andwf	TMP, W		; test if int was RX complete
	btfsc	zero		; 
	retlw	0x00		; Packet was not completely received!
	retlw	0x01		; Packet completely received!


;------------------------------------------------------------------------------
; Discard top packet from card's FIFO (after it's reading from card)
;------------------------------------------------------------------------------
rx_discard_top_packet		; Discard top packet from card's FIFO
	page1			; Set PCLATCH to program page 1
	call	set_command_io	; set address I/O + 0E (Commnad)
	page0			; Set PCLATCH to program page 0
	movlw	0x00		; Send 0x00 (request for top rx packet)
	movwf	byte_tmp	; 
	movlw	0x40		; Send 0x40 
	call	OutWord		; to address I/O + 0E
	return			; All done

;------------------------------------------------------------------------------
clear_packet_area		; 
	movlw	D'222'		; Clear RX packet RAM area
	movwf	AA		; it takes 222bytes (62+80+80)
	movlw	pktdata+2	; Take start of packet area
	movwf	FSR		; and put it into FSR

clrpkt				; 
	clrf	INDF		; clear rx buffer
	incf	FSR, F		; 

	call	do_indirect_bank_tests	; indirect address bank maintenance

cl_len_tests			;
	decfsz	AA,f		; Top of the buffer reached?
	goto	clrpkt		; Not yet
	bcf	STATUS, IRP	; Set indirect page 0+1
	return			; All done

;------------------------------------------------------------------------------
; Receive (read) packet received by card to PIC
;------------------------------------------------------------------------------
rx_packet				; Receive packet from card
	page1				; Set PCLATCH to program page 1
	call	set_zero_io		; set address to I/O + 0
	page0				; Set PCLATCH to program page 0
	bsf	a3			; a3 HIGH = address I/O + 8
	call	InByte			; get the first word (low 8 bits)
	movwf	pktdata+eth_len_l	; first byte is the length of the pckt
	
	movf	byte_tmp, W		; 
	movwf	pktdata+eth_len_h	; store the status of the receiving
	btfsc	pktdata+eth_len_h,7	; Was a packet completely received?
	goto	norxpkt			; No.

	btfsc	pktdata+eth_len_h,6	; Was there an error in the recv packet?
	goto	badrxpkt		; Yes.

	ifdef	DEBUG0
	movlw	A'R'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'X'			;
	call	serial_SEND_byte	; For debug 
	movlw	A':'			;
	call	serial_SEND_byte	; For debug 
	endif

	movf	pktdata+eth_len_h,w	; Mask out status bits and
	andlw	B'00000111'		; leave only length bits
	movwf	pktdata+eth_len_h	; in this byte
	tstf	pktdata+eth_len_h	; is high length byte zero?
	bnz	longpkt			; No eth_len_h <> 0, goto to longpkt

	movf	pktdata+eth_len_l,W	; And now the low length byte...
	sublw	D'222'			; Is the size of packet > 62 bytes?
	bnc	longpkt			; Yes, packet is longer then 62 bytes
	tstf	pktdata+eth_len_l	; Is packet length = 0 ?
	bz	badrxpkt		; Yes packet length = 0, badrxpkt
	goto	readrxfifo		; Ok, go to packet reading

longpkt
	movlw	D'222'			; Set length to max (62 bytes in bank0)
	movwf	pktdata+eth_len_l	; (or 222 bytes if using banks 0,1,2)

readrxfifo			
	call	clear_packet_area	; Clear RX packet RAM area
	page1				; Set PCLATCH to program page 1
	call	set_zero_io		; set address to I/O + 0
	page0				; Set PCLATCH to program page 0
	movlw	pktdata+eth_dest_0	; Here starts RAM buffer for received
	movwf	FSR			; packet
	movf	pktdata+eth_len_l,W	; take low packet length byte
	movwf	g_len_low		; store the length there

rx_lp					; Packet reading loop
	call	InByte			; Read a byte
	movwf	INDF			; Store it in the buffer
	ifdef	DEBUG2
	call	serial_SEND_byte	; For debug 
	endif
	incf	FSR,f			; Increment the buffer
	movf	byte_tmp, W		; 
	movwf	INDF			; Store it in the buffer
	ifdef	DEBUG2
	call	serial_SEND_byte	; For debug 
	endif
	incf	FSR,f			; Increment the buffer

	decf	g_len_low, f		;
	movf	g_len_low, f		;
	btfsc	STATUS, Z		;
	goto	rx_fin_odd		; We have odd packet length

	call	do_indirect_bank_tests	; indirect address bank maintenance

rx_len_tests				;
	decfsz	g_len_low,f		; Whole "length" readed?
	goto	rx_lp			; Not yet
	goto	rx_fin			; Yes whole length readed!

rx_fin_odd				;
	decf	FSR, F			; clear odd byte at the end of 
	clrf	INDF			; received packet

rx_fin					;
	bcf	STATUS, IRP		; Set indirect page 0+1
	call	rx_discard_top_packet	; Discard the readed packet from card
	ifdef	DEBUG0
	call	serialTXcr		; 
	endif
	retlw	0x01			; All done

;------------------------------------------------------------------------------
; Packet has zero length or packet was received with error
;------------------------------------------------------------------------------
badrxpkt				; length of packet is too long or zero
	clrf	pktdata+eth_len_h	; clear high
	clrf	pktdata+eth_len_l	; and low length bytes
	call	rx_discard_top_packet	; Discard top received packet
	retlw	0x00			; All done

;------------------------------------------------------------------------------
; No packet received
;------------------------------------------------------------------------------
norxpkt					; packet was not received
	clrf	pktdata+eth_len_h	; clear high 
	clrf	pktdata+eth_len_l	; and low length bytes
	retlw	0x00			; All done



;##################################################################
; Start of delay subroutines (not always
; of the advertised value, but typically
; close enough) It seems to be counted for
; 4 MHz crystal
;##################################################################3

Delay50uS:
	ifdef	XTAL4MHZ
	movlw	D'15'	; for 4MHz
	endif
	ifdef	XTAL10MHZ
	movlw	D'38'	; for 10MHz
	endif
	ifdef	XTAL12MHZ
	movlw	D'48'	; for 12MHz
	endif
	ifdef	XTAL20MHZ
	movlw	D'75'	; for 20MHz
	endif
	movwf	AA
lpA1:	decfsz	AA,f
	goto 	lpA1
	nop
	return

Delay10mS:
	ifdef	XTAL4MHZ
	movlw	D'10'	; for 4MHz and 1000uS inside cycle
	endif
	ifdef	XTAL10MHZ
	movlw	D'25'	; for 10MHz and 1000uS inside cycle
	endif
	ifdef	XTAL12MHZ
	movlw	D'35'	; for 12MHz and 1000uS inside cycle
	endif
	ifdef	XTAL20MHZ
	movlw	D'50'	; for 20MHz and 1000uS inside cycle
	endif
	movwf	BB
lpB2:
	movlw	D'249'	; 249*4 + 2 + 2 =1000uS
	movwf	AA
lpA2:
	nop	
	decfsz	AA,f	
	goto 	lpA2
	decfsz	BB,f
	goto	lpB2
	return		; 10mS and 2uS elapsed

Delay500uS:
	call	Delay50uS
	call	Delay50uS
	call	Delay50uS
	call	Delay50uS
	call	Delay50uS
	call	Delay50uS
	call	Delay50uS
	call	Delay50uS
	call	Delay50uS
	call	Delay50uS
	return

Delay100mS:
	movlw	0x01	; for 4MHz or 10MHz and 1000uS inside cycle
	ifdef	XTAL20MHZ
	movlw	0x02	; for 20MHz and 1000uS inside cycle
	endif
	movwf	CC

lpC4:
	ifdef	XTAL4MHZ
	movlw	D'100'	; for 4MHz and 1000uS inside cycle
	endif
	ifdef	XTAL10MHZ
	movlw	D'250'	; for 10MHz and 1000uS inside cycle
	endif
	ifdef	XTAL12MHZ
	movlw	D'250'	; for 12MHz and 1000uS inside cycle
	endif
	ifdef	XTAL20MHZ
	movlw	D'250'	; for 20MHz and 1000uS inside cycle
	endif
	movwf	BB
lpB4:
	movlw	D'249'	; 249*4 + 2 + 2 =1000uS
	movwf	AA
lpA4:
	nop	
	decfsz	AA,f
	goto 	lpA4
	decfsz	BB,f
	goto	lpB4
	decfsz	CC,f
	goto	lpC4
	return		; 100ms and 2uS elapsed

Delay1S:
	call	Delay100mS
	call	Delay100mS
	call	Delay100mS
	call	Delay100mS
	call	Delay100mS
	call	Delay100mS
	call	Delay100mS
	call	Delay100mS
	call	Delay100mS
	call	Delay100mS
	return


;------------------------------------------------------------------------------
; Utiliras para enviar y recibir via el peurto RS-232
;------------------------------------------------------------------------------
;serial_SEND_byte:		; Envia un caracter desde W
;serial_Transmit:		; via la USAR del PIC 16F877
;	bank1			;
;
;serial_Transmit_Loop:		; 
;	btfss	TXSTA, TRMT	; est lista la USART para TX el siguiente byte?
;	goto	serial_Transmit_Loop	; An no!
;	bank0			;
;	movwf	TXREG		; Pon un byte de W a TXREG
;	return			; Todo hecho

serial_SEND_byte:			; Envia un byte almacenado en W
serial_Transmit:			; al puerto RS-232
	movwf	serialData		; guarda el byte a transmitir
	movlw	0x0a			; 8 bits 1 stop and 1 start bit
	movwf	serialCount		;
	bcf	STATUS,C		; Limpia la bandera carry

serial_Transmit_Loop:
	bcf	serial_out		; El primer bit es cero
	btfss	STATUS,C		; Salta si carry es 1
	bsf	serial_out		; si el bit es 0, enviamos HIGH
	movf	delay_time, W		; RS232 delay para la velocidad apropiada
	call	serial_Delay		; espera un tiempo de un bit
	bsf	STATUS,C		; Pon a 1 la bandera de carry
	rrf	serialData,F		; rotar los bits del byte que se enva
	decfsz	serialCount,F		; decrementa el contador de bits
	goto	serial_Transmit_Loop	; ve por el siguiente bit
	return				; Todo hecho

;------------------------------------------------------------------------------
serial_Delay:
	movwf	serialDelayCount	; Pon un valor de delay desde W, aqui
	nop				; esto ayuda en altas velocidades

serial_Delay_Loop:			; Bucle Substractivo
	decfsz	serialDelayCount,F	; decrementa contador
	goto	serial_Delay_Loop	; procesa el siguiente bucle
	return				; Todo hecho

;------------------------------------------------------------------------------
serial_Receive:				; Estamos recibiendo datos?
	bcf	serialData,00		; Limpia la bandera de recepcin
	btfsc	serial_in		; Si estamos recibiendo datos, serial_in = 1
	retlw	0x00			; De otra manera, regresa
	movlw	0x09			; Los datos a recibir son 9
	movwf	serialCount		; bits

	movf	delay_time3, W		; espera 1/3 del tiempo delay de un bit del RS232
	call	serial_Delay		; porque este es el primer bit

serial_Receive_Loop:			; Bucle de recepcion del RS-232
	bcf	STATUS,C		; Limpia la bandera Carry
	btfss	serial_in		; Si la entrada es 0
	bsf	STATUS,C		; Pon a 1 la bandera Carry
	rrf	serialData,F		; y rota el regstro del byte de recepcin

	movf	delay_time, W		; RS232 delay: W <-- delay_time
	call	serial_Delay		; espera por la longitud de los bis
	nop				; manten la temporizacin de recepcin
	decfsz	serialCount,F		; Tenemos todos los bits? 
	goto	serial_Receive_Loop	; An no
	movf	serialData,W		; Pon el byte recibido en W

serial_Receive_Stop:			; Espera la llegda del bit stop, se espera recibir
	btfsc	serial_in		; un 0 en serial_in
	goto	serial_Receive_Stop	; hay un 1 presente (stop bit)
	bsf	serialData,00		; pon el bit 0 a 1 (Bandera para byte Recibido)
	return				; Regresa


serial_Receive_BYTE:			; Recive un byte desde el puerto RS232
	call	serial_Receive		; Lee el byte
	btfss	serialData,00		; Fu leido el byte?
	goto	serial_Receive_BYTE	; No, espera el dato
	return				; Byte leido, regresa
		
Send_Version:				; Envia la versin de firmware por el puerto RS232
		;bcf	INTCON, GIE	;
		clrf	AA		;

;------------------------------------------------------------------------------
Send_Version_serial:			;
	movf	AA, W			; toma el byte de offset
	call	Version_Data		; toma un byte
	iorlw	000H			; cero = fin de envo
	btfsc	STATUS,Z		; 
	goto	Send_Version_END	; Todo el texto enviado!
	call	serial_SEND_byte	; Enva el byte
	incf	AA, F			; incrementa el apuntador
	goto	Send_Version_serial	; y procesa el siguiente byte

Send_Version_END:			; Envia un CR+LF adicional
	call	serialTXcr		;
	call	serialTXlf		;
	;bsf	INTCON, GIE		; 
	return				; Todo hecho

serialTXcr:
	movlw	0x0D			; Envia CR
	goto	serial_SEND_byte		

serialTXlf:
	movlw	0x0A			; Envia LF
	goto	serial_SEND_byte


;------------------------------------------------------------------------------
; WX string on the RS232 maintenance
; data format: <Esc>w000/000g000t...b.....h..<CR>z...C
;                    |	         |   |     |     |   |
; offset in RAM reg: 0          12  16     22   25  29 
;------------------------------------------------------------------------------
Write_WX_to_RAM:			; write WX bytes into RAM
	bsf	led			; Switch on the WX LED
	bank3				; Select page 3 (180H - 1FFH)
	movlw	wx_data			; WX data starts here
	movwf	FSR			; place it in FSR
	bsf	STATUS, IRP		; Set indirect page 2+3

Write_WX_to_RAM_Loop:			; Loop for writting data into RAM
	bank0				; Select page 0 (00H - 7FH)
	call	serial_Receive_BYTE	; receive a byte from RS232
	bank3				; Select page 3 (180H - 1FFH)
	movwf	INDF			; store it to the FSR address
	incf	FSR, F			; point to the next byte
	xorwf	wx_data_endR, W		; Take end byte of WX string
	btfsc	STATUS,Z		; equals it to the received byte?
	goto	Write_WX_to_RAM_end	; Yes, end the loop!
	movlw	wx_data+WX_DATA_MAX_LEN	; last wx_data byte address
	subwf	FSR, W			; do we reach the last byte?
	btfss	STATUS, C		; 
	goto	Write_WX_to_RAM_Loop	; Not yet

Write_WX_to_RAM_end			; 
	bank0				; Select page 0 (00H - 7FH)
	bcf	STATUS, IRP		; Set indirect page 0+1
		; All WX data received...
	bcf	led			; Switch off the WX LED
	goto	Main_RX_Eth		; And go back to the main loop

;------------------------------------------------------------------------------
; Utilities for R/W Flash Program memory
;------------------------------------------------------------------------------
;
; lectura de FLASH
; al ingresar 	adresseH = MSB adresse
;		adresseL = LSB adresse	
; al salir	W=octet lu		
;

readF:				; Lee la memoria de programa de la flash,de acuerdo con MICROCHIP doc.
	movf adresseH,w
	bank2
	movwf EEADRH
	bank0
	movf adresseL,w
	bank2
	movwf EEADR
	bank3
	bsf EECON1,EEPGD
	bsf EECON1,RD
	nop
	nop
	bank2
	movf EEDATA,W
	bank0
	return

;
; ecritura de FLASH
; al ingresar 	adresseH = MSB adresse
;		adresseL = LSB adresse
; 		dataF	 = octet  ecrire		
;
writeF:				; Escribe la memoria de programa de la flash, de acuerdo con MICROCHIP doc.
	movf adresseH,w
	bank2
	movwf EEADRH
	bank0
	movf adresseL,w
	bank2
	movwf EEADR
	bank0
	movf dataF,w
	bank2
	movwf EEDATA
	bank3
	bsf EECON1,EEPGD
	bsf EECON1,WREN
	bcf INTCON,GIE
	movlw 55
	movwf EECON2
	movlw 0AA
	movwf EECON2
	bsf EECON1,WR	
	nop
	nop
	bcf EECON1,WREN
	bank0	
	return

;##################################################################
	org 0x0800			; Program page1 (subroutines)
;##################################################################

;------------------------------------------------------------------------------
; Rutinas de inicializacion de la NIC 3c509
;------------------------------------------------------------------------------
Init3c509b
	bsf	reset			; Envia seal de Reset a la tarjeta
	page0				; Set PCLATCH to program page 0
	call	Delay10mS		; espera 10ms
	bcf	reset			; y libera la seal de Reset
	call	Delay100mS		; espera 200ms
	call	Delay100mS		; 
	page1				; Set PCLATCH to program page 1
	call	send_id_seq		; Envia alguna id_seq a la tarjeta
	movlw	0xc0			; Reset Global
	page0				; Set PCLATCH to program page 0
	call	OutByte			; 
	call	Delay100mS		; 
	page1				; Set PCLATCH to program page 1
	call	send_id_seq		; Envia alguna id_seq a la tarjeta
	movlw	0xd0			; Limpia tagas de la tarjeta
	page0				; Set PCLATCH to program page 0
	call	OutByte			; 
	page1				; Set PCLATCH to program page 1
	call	read_id_mfg		; 
	page1				; Set PCLATCH to program page 1
	call	read_oem_mac		; Salva los registros del PIC para MAC addr
	page1				; Set PCLATCH to program page 1
	call	read_io_int		;
	page1				; Set PCLATCH to program page 1
	call	activate_adapter	; Activa la tarjeta para RX/TX
	clrf	reg_al			; borra las banderas de protocolo syn TCP
	return				; Todo hecho

;------------------------------------------------------------------------------
send_id_seq				; Envia id_seq para inicializar la tarjeta
	bcf	a9			; a9 LOW (0x100-0x10F)
	call	set_zero_io		; poner la direccin en I/O + 0
	movlw	0x00			;
	page0				; Set PCLATCH to program page 0
	call	OutByte			; Envia 0x00
	movlw	0x00			;
	call	OutByte			; Envia 0x00
	movlw	0xff			;
	movwf	CC			;
	movlw	0xff			;
	movwf	reg_al			;
seq1
	page0				; Set PCLATCH to program page 0
	movf	reg_al,W		;
	call	OutByte			; Envia 256 bytes como id_seq
	page1				; Set PCLATCH to program page 1
	bcf	STATUS,C		;
	rlf	reg_al,f		;
	bnc	seq2			;
	movlw	0xcf			;
	xorwf	reg_al,f		;
seq2
	decfsz	CC,f			;
	goto	seq1			;
	return				; Todo hecho

;------------------------------------------------------------------------------
read_id_mfg				;
	ifdef	DEBUG0
	movlw	A'I'			;
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; Para debug 
	movlw	A'D'			;
	call	serial_SEND_byte	; Para debug 
	movlw	A':'			;
	call	serial_SEND_byte	; Para debug 
	endif
	movlw	0x87			; Pon la Direccin en la EEPROM de la tarjeta
	page1				; Set PCLATCH to program page 1
	call	read_eeprom_word	; Lee la palabra desde la EEPROM de la tarjeta
	movf	iohigh,W		; 
	ifdef	DEBUG0
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; para debug 
	endif
	movf	iolow,W			; 
	ifdef	DEBUG0
	call	serial_SEND_byte	; Para debug 
	call	serialTXcr		; 
	endif
	
	page1				; Set PCLATCH to program page 1
	movf	iohigh,W		; Si la tarjeta est lista, esta deber
	xorlw	0x6d			; devolver 0x6d50 cuando la direccin de la EEPROM
	btfss	STATUS, Z		; 0x07 es requerida
	goto	card_not_found_loop	; Oh, la tarjeta no est lista
	movf	iolow,W			; 
	xorlw	0x50			; 
	btfss	STATUS, Z		; 
	goto	card_not_found_loop	; Oh, la tarjeta no est lista
	page0				; Set PCLATCH to program page 0
	bsf	led			; 
	call	Delay100mS		; Flashea 2 vecs para indicar
	bcf	led			; que la tarjeta fue encontrada!
	call	Delay100mS		;
	bsf	led			; 
	call	Delay100mS		;
	bcf	led			;
	return				; Todo hecho

card_not_found_loop			; Flasheo constante para indicar que 
	page0				; Set PCLATCH to program page 0
	bsf	led			; la tarjeta 3c509 no fue encontrada
	call	Delay100mS		; (la tarjeta no responde despues de 
	bcf	led			; enviar la secuencia inicial)
	call	Delay100mS		;
	page1				; Set PCLATCH to program page 1
	goto	card_not_found_loop	; 

;------------------------------------------------------------------------------
read_io_int				;
	ifdef	DEBUG1
	movlw	A'I'			;
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; Para debug 
	movlw	A'/'			;
	call	serial_SEND_byte	; Para debug 
	movlw	A'O'			;
	call	serial_SEND_byte	; Para debug 
	movlw	A':'			;
	call	serial_SEND_byte	; Para debug 
	movlw	0x88			; Pon la direccion en la EEPROM de la tarjeta
	page1				; Set PCLATCH to program page 1
	call	read_eeprom_word	; Lee la palabra desde la EEPROM de la tarjeta
	movf	iohigh,W		; 
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; Para debug 
	movf	iolow,W			; 
	call	serial_SEND_byte	; Para debug 
	call	serialTXcr		; 
	
	movlw	A'I'			;
	call	serial_SEND_byte	; Para debug 
	movlw	A'N'			;
	call	serial_SEND_byte	; Para debug 
	movlw	A'T'			;
	call	serial_SEND_byte	; Para debug 
	movlw	A':'			;
	call	serial_SEND_byte	; Para debug 
	movlw	0x89			; Pon la direccin en la EEPROM de la tarjeta
	page1				; Set PCLATCH to program page 1
	call	read_eeprom_word	; Lee la palabra de la EEPROM de la tarjeta
	movf	iohigh,W		; 
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; Para debug 
	movf	iolow,W			; 
	call	serial_SEND_byte	; Para debug 
	call	serialTXcr		; 
	endif
	return				;
	
;------------------------------------------------------------------------------
read_oem_mac				; Reads card's MAC to PIC's registers
	ifdef	DEBUG1
	page0				; Set PCLATCH to program page 0
	movlw	A'M'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'A'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'C'			;
	call	serial_SEND_byte	; For debug 
	movlw	A':'			;
	call	serial_SEND_byte	; For debug 
	endif
	movlw	0x8a			; Set address in card's EEPROM
	page1				; Set PCLATCH to program page 1
	call	read_eeprom_word	; Read the word from card's EEPROM
	movf	iohigh,W		; Store the obtained word
	movwf	MAC0			; 
	ifdef	DEBUG1
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; For debug 
	endif
	movf	iolow,W			; 
	movwf	MAC1			; 
	ifdef	DEBUG1
	call	serial_SEND_byte	; For debug 
	endif
	
	movlw	0x8b			; Set address in card's EEPROM
	page1				; Set PCLATCH to program page 1
	call	read_eeprom_word	; Read the word from card's EEPROM
	movf	iohigh,W		; Store the obtained word
	movwf	MAC2			; 
	ifdef	DEBUG1
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; For debug 
	endif
	movf	iolow,W			; 
	movwf	MAC3			; 
	ifdef	DEBUG1
	call	serial_SEND_byte	; For debug 
	endif

	movlw	0x8c			; Set address in card's EEPROM
	page1				; Set PCLATCH to program page 1
	call	read_eeprom_word	; Read the word from card's EEPROM
	movf	iohigh,W		; Store the obtained word
	movwf	MAC4			; 
	ifdef	DEBUG1
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; For debug 
	endif
	movf	iolow,W			; 
	movwf	MAC5			; 
	ifdef	DEBUG1
	call	serial_SEND_byte	; For debug 
	call	serialTXcr		; 
	endif


	return				; All done

;------------------------------------------------------------------------------
read_eeprom_word		; Read a byte from card's EEPROM
	page0			; Set PCLATCH to program page 0
	call	OutByte		; Send addres of byte to card
	call	Delay500uS	; wait for 500us
	clrf	iolow		; Clear registers for
	clrf	iohigh		; 16bit word reception
				
	movlw	D'16'		; response is 
	movwf	DD		; 16 bytes long
rde1
	page0			; Set PCLATCH to program page 0
	call	InByte		; Read a byte from card
	movwf	TMP		; store it there
	rrf	TMP,f		; rotate it right (lowest bit is "the one")
	rlf	iolow,f		; and rotate registers to get the whole
	rlf	iohigh,f	; received byte from card response
	page1			; Set PCLATCH to program page 1

	decfsz	DD,f		; All 16 bytes processed?
	goto	rde1		; Not yet!
	return			; All done

;------------------------------------------------------------------------------
get_act_window				; Gets actual window number
	ifdef	DEBUG1
	bsf	a0			; a0 
	bsf	a1			; a1 
	bsf	a2			; a2 
	bsf	a3			; a3 0x0f = Get actual Window number
	movlw	A'W'			;
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; For debug 
	movlw	A':'			;
	call	serial_SEND_byte	; For debug 
	call	InByte			; get the first word (low 8 bits)
	call	serial_SEND_byte	; For debug 
	call	serialTXcr		; 
	endif
	return				; All done

;------------------------------------------------------------------------------
get_status_reg				; Gets status reg content
	ifdef	DEBUG1
	bcf	a0			; a0 
	bsf	a1			; a1 
	bsf	a2			; a2 
	bsf	a3			; a3 0x0f = Get actual Window number
	page0				; Set PCLATCH to program page 0
	movlw	A'S'			;
	call	serial_SEND_byte	; For debug 
	movlw	A':'			;
	call	serial_SEND_byte	; For debug 
	call	InByte			; get the first word (low 8 bits)
	movwf	TMP			; Low byte here 
	movf	byte_tmp, W		; Take high byte
	call	serial_SEND_byte	; For debug 
	movf	TMP, W			; Take low byte
	call	serial_SEND_byte	; For debug 
	call	serialTXcr		; 
	endif
	return				; All done

;------------------------------------------------------------------------------
read_W0_mfg				;
	movlw	0x00			; 
	call	set_window		; Select window 0 (Setup registers)
	ifdef	DEBUG0
	movlw	A'i'			;
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; For debug 
	movlw	A'd'			;
	call	serial_SEND_byte	; For debug 
	movlw	A':'			;
	call	serial_SEND_byte	; For debug 
	endif
	page1				; Set PCLATCH to program page 1
	call	set_zero_io		; set address to I/O + 0
	page0				; Set PCLATCH to program page 0
	call	InByte			; get the first word (low 8 bits)
	movwf	TMP			; Low byte here 
	movf	byte_tmp, W		; Take high byte
	ifdef	DEBUG0
	call	serial_SEND_byte	; For debug 
	endif
	movf	TMP, W			; Take low byte
	ifdef	DEBUG0
	call	serial_SEND_byte	; For debug 
	call	serialTXcr		; 
	endif
	page1				; Set PCLATCH to program page 1
	movf	byte_tmp,W		; If the card is ready, it should
	xorlw	0x6d			; return 0x6d50 
	btfss	STATUS, Z		; 
	goto	card_bad_config_loop	; Sorry, but card is not ready.
	movf	TMP,W			; 
	xorlw	0x50			; 
	btfss	STATUS, Z		; 
	goto	card_bad_config_loop	; Sorry, but card is not ready.
	return				; All done

card_bad_config_loop			; Loop flashing to indicate that
	page0				; Set PCLATCH to program page 0
	bsf	led			; 3c509 card was not found
	call	Delay100mS		; (card does not respond after
	bcf	led			; sending initial sequence)
	call	Delay100mS		;
	bsf	led			; 3c509 card was not found
	call	Delay100mS		; (card does not respond after
	bcf	led			; sending initial sequence)
	call	Delay1S			; 
	page1				; Set PCLATCH to program page 1
	goto	card_bad_config_loop	; 

;------------------------------------------------------------------------------
read_w2_mac				; Read MAC from card's EEPROM(Window 2)
	ifdef	DEBUG1
	movlw	A'W'			;
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; For debug 
	movlw	A'2'			;
	call	serial_SEND_byte	; For debug 
	movlw	A':'			;
	call	serial_SEND_byte	; For debug 
	page1				; Set PCLATCH to program page 1
	call	set_zero_io		; set address to I/O + 0
	page0				; Set PCLATCH to program page 0
	call	InByte			; send to I/O + 0
	call	serial_SEND_byte	; For debug 
	movf	byte_tmp, W		; 
	call	serial_SEND_byte	; For debug 
	bsf 	a1			; address I/O + 2
	call	InByte			; send to I/O + 2	
	call	serial_SEND_byte	; For debug 
	movf	byte_tmp, W		; 
	call	serial_SEND_byte	; For debug 
	bcf 	a1			; 
	bsf 	a2			; address I/O + 4
	call	InByte			; send to I/O + 4
	call	serial_SEND_byte	; For debug 
	movf	byte_tmp, W		; 
	call	serial_SEND_byte	; For debug 
	call	serialTXcr		; 
	endif
	return				; All done

;------------------------------------------------------------------------------
activate_adapter		; Activates the card for RX/TX
	;movlw	0xf0		; (I/O 0x300 >> 4) | 0xe0 = 0xf0 
	movlw	0xff		; ACTIVATE_VULCAN
	page0			; Set PCLATCH to program page 0
	call	OutByte		; Activate card in EEPROM iobase
 	bsf	a9		; a9 HIGH (0x300-0x30F)

	;call	rx_reset	; Reset Receive
	;call	tx_reset	; Reset Transmit

	page1				; Set PCLATCH to program page 1
	call	read_W0_mfg	; Show if 3c509 is responding correctly
	;page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 

	movlw	0x04		; 
	page1				; Set PCLATCH to program page 1
	call	set_window	; Select card window 4
	;page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	page1				; Set PCLATCH to program page 1
	call	set_link_jabber	; for link jabber setup
	movlw	0x00		; 
	page1				; Set PCLATCH to program page 1
	call	set_window	; Select card window 0 (Setup registers)
	;page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	page1				; Set PCLATCH to program page 1
	call	rx_enable	; Enable Receive
	;page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	page1				; Set PCLATCH to program page 1
	call	tx_enable	; Enable Transmit
	;page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	page1				; Set PCLATCH to program page 1
	call	set_int_mask_RXC	; Enable RX complete interrupts
	page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	;page1				; Set PCLATCH to program page 1
	call	set_RZ_mask	; Set Read Zero mask
	page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	;page1				; Set PCLATCH to program page 1
	;call	set_TX_START	; Set TX START after 0 bytes
	page1				; Set PCLATCH to program page 1
	call	set_RX_EARLY	; Set RX START after 0 bytes
	;page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	page1				; Set PCLATCH to program page 1
	call	set_enable_adapter	; Enable adapter
	;page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	page1				; Set PCLATCH to program page 1
	call	set_rx_filter	; Set normal / promiscuit mode
	;page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	movlw	0x02		; 
	page1				; Set PCLATCH to program page 1
	call	set_window	; Select card window 2 for 
	;page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	page1				; Set PCLATCH to program page 1
	call	set_stored_mac	; setting the cards address
	page1				; Set PCLATCH to program page 1
	call	read_w2_mac	; For debug
	movlw	0x01		; 
	page1				; Set PCLATCH to program page 1
	call	set_window	; Select card window 1 for normal operation
	;page1				; Set PCLATCH to program page 1
	;call	get_status_reg	; 
	return			; All done

;------------------------------------------------------------------------------
set_enable_adapter		; Enables card after initialization
	call	set_zero_io	; set address to I/O + 0
	bsf	a2		; a2 HIGH = address I/O + 4
	movlw	0x01		; Send 0x01
	movwf	byte_tmp	; Low byte here
	movlw	0x00		; Send 0x00
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 4
	return			; All done

;------------------------------------------------------------------------------
rx_reset			; Reset receive
	call	set_command_io	; set address I/O + 0E (Commnad)
	movlw	0x00		; Send 0x00
	movwf	byte_tmp	; Low byte here
	movlw	0x28		; Send 0x28
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0E
	return			; All done

;------------------------------------------------------------------------------
tx_reset			; Reset transmit
	call	set_command_io	; set address I/O + 0E (Commnad)
	movlw	0x00		; Send 0x00
	movwf	byte_tmp	; Low byte here
	movlw	0x58		; Send 0x58
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0E
	return			; All done
	
;------------------------------------------------------------------------------
set_rx_filter			; Set type of RX filter
	call	set_command_io	; set address I/O + 0E (Commnad)
	movlw	0x05		; 0x01=station, 0x04=bcast ; movlw 0x01
	movwf	byte_tmp	; Low byte here
	movlw	0x80		; Send 0x80
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0E
	return			; All done

;------------------------------------------------------------------------------
set_stored_mac			; Write MAC from card's EEPROM to Window 2
	call	set_zero_io	; set address to I/O + 0
	movf	MAC0,W		; MAC 0 address
	movwf	byte_tmp	; Low byte here
	movf	MAC1,W		; MAC 1 address
	page0			; Set PCLATCH to program page 0
	call	OutWord		; send to I/O + 0

	bsf 	a1		; address I/O + 2
	movf	MAC2,W		; MAC 2 address
	movwf	byte_tmp	; Low byte here
	movf	MAC3,W		; MAC 3 address
	call	OutWord		; send to I/O + 2

	bcf 	a1		; 
	bsf 	a2		; address I/O + 4
	movf	MAC4,W		; MAC 4 address
	movwf	byte_tmp	; Low byte here
	movf	MAC5,W		; MAC 5 address
	call	OutWord		; send to I/O + 4
	return			; All done

;------------------------------------------------------------------------------
set_link_jabber			; Set link jabber
	bcf	a0		; a0 LOW
	bsf	a1		; a1 HIGH
	bcf	a2		; a2 LOW
	bsf	a3		; a3 HIGH = address I/O + 0A
	movlw	0xC0		; 0xC0 (Enable link beat,jabber for TP media)
	movwf	byte_tmp	; Low byte here
	movlw	0x80		; Send 0x80 (media TP)
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0A
	return			; All done
				  
;------------------------------------------------------------------------------
rx_enable			; Enable receive
	call	set_command_io	; set address I/O + 0E (Commnad)
	movlw	0x00		; Send 0x00
	movwf	byte_tmp	; Low byte here
	movlw	0x20		; Send 0x20 
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0E
	return			; All done

;------------------------------------------------------------------------------
tx_enable			; Enable transmit
	call	set_command_io	; set address I/O + 0E (Commnad)
	movlw	0x00		; Send 0x00 
	movwf	byte_tmp	; Low byte here
	movlw	0x48		; Send 0x48 
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0E
	return			; All done


;------------------------------------------------------------------------------
		; Window 0: Setup/configuration 
		; Window 1: Normal operation
		; Window 2: Station address setup/read
		; Window 4: Diagnostics + Various transcvr/media bits
set_window			; Set window to number in W
	movwf	byte_tmp	; Low byte here (Window number)
	call	set_command_io	; set address I/O + 0E (Commnad)
	movlw	0x08		; Send 0x08 (Set_window_command + byte_tmp)
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0E
	return			; All done

;------------------------------------------------------------------------------
set_int_mask_RXC		; Mask Interrupts to only RX complete
	call	set_command_io	; set address I/O + 0E (Commnad)
	movlw	0x30		; Send 0x10 (RX complete) + 0x20 (RX early)
	movwf	byte_tmp	; Low byte here
	movlw	0x70		; Send 0x70 (set int mask cmd)
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0E
	return			; All done

;------------------------------------------------------------------------------
set_RZ_mask			; 
	call	set_command_io	; set address I/O + 0E (Commnad)
	movlw	0xfe		; Send 0xfe
	movwf	byte_tmp	; Low byte here
	movlw	0x78		; Send 0x78 (set read zero mask)
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0E
	return			; All done

;------------------------------------------------------------------------------
set_RX_EARLY			; 
	call	set_command_io	; set address I/O + 0E (Commnad)
	movlw	0x00		; Send 0x00
	movwf	byte_tmp	; Low byte here
	movlw	0x88		; Send 0x70 (set RX start after 0 bytes)
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0E
	return			; All done

;------------------------------------------------------------------------------
set_TX_START			; 
	call	set_command_io	; set address I/O + 0E (Commnad)
	movlw	0x00		; Send 0x00 
	movwf	byte_tmp	; Low byte here
	movlw	0x98		; Send 0x98 (set TX start after 0 bytes)
	page0			; Set PCLATCH to program page 0
	call	OutWord		; to address I/O + 0E
	return			; All done

;------------------------------------------------------------------------------
set_command_io			; Sets address for commnads (write)
set_status_io			; Same address is for status (read)
	bcf	a0		; a0 LOW
	bsf	a1		; a1 HIGH
	bsf	a2		; a2 HIGH
	bsf	a3		; a3 HIGH = address I/O + 0E
	return			; All done

;------------------------------------------------------------------------------
set_zero_io			; Same address is for status (read)
	bcf	a0		; a0 LOW
	bcf	a1		; a1 LOW
	bcf	a2		; a2 LOW
	bcf	a3		; a3 LOW = address I/O + 00
	return			; All done


;------------------------------------------------------------------------------
WX_RAMinit:				; Fills WX RAM area with '-' (0x2d)
	bank3				; Select page 3 (180H - 1FFH)
	movlw	wx_data			; Start position
	movwf	FSR			; place here
	bsf	STATUS, IRP		; Set indirect page 2+3

WX_nextRAMB:				; Loop itself
	movlw	a'-'			; fill area with '-' (0x2d)
	movwf	INDF			; 
	incf	FSR, F			; next register
	movlw	wx_data+WX_DATA_MAX_LEN	; last byte for wx_data
	subwf	FSR, W			; do we reach the last byte?
	btfss	STATUS, C		; 
	goto	WX_nextRAMB		; Not yet

	bank0				; Select page 0 (00H - 7FH)
	bcf	STATUS, IRP		; Set indirect page 0+1
	return				; All done

;------------------------------------------------------------------------------
RAMsend					; Sends RAM registers to Rs-232
	movlw	0x00			; Start with reg 0x00
	movwf	FSR			; 
	bcf	STATUS, IRP		; Set indirect page 0+1

dalsiRAMB:				; Sending loop
	movf	INDF,W			; Take the byte from the FSR address
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; Send on the RS-232 line
	page1				; Set PCLATCH to program page 1
	incf	FSR,F			; next REG
	btfss	STATUS,Z		; all REGs sent?
	goto	dalsiRAMB		; not yet
	btfsc	STATUS, IRP		; are we in indirect page 0+1?
	goto	RAMsend_end		; All 4 pages sent.
	bsf	STATUS, IRP		; Set indirect page 2+3
	goto	dalsiRAMB		; loop for the next register

RAMsend_end				; 
	bcf	STATUS, IRP		; Set indirect page 0+1
	page0				; Set PCLATCH to program page 0
	goto	Main_RX_Eth		; All 4 pages sent, continue

;------------------------------------------------------------------------------
WX_RAMsend:				; Sends WX registers on the RS232
	bank3				; Select page 3 (180H - 1FFH)
	movlw	wx_data			; WX data start position
	movwf	FSR			; into FSR
	bsf	STATUS, IRP		; Set indirect page 2+3

WX_dalsiRAMB:				; Sending loop
	movf	INDF, W			; Take the byte from the FSR address
	bank0				; Select page 0 (00H - 7FH)
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; Send on the RS-232 line
	page1				; Set PCLATCH to program page 1
	bank3				; Select page 3 (180H - 1FFH)
	movf	INDF, W			; Take the byte from the FSR address
	incf	FSR, F			; point to the next REG
	xorwf	wx_data_endR, W		; is the actual byte WX data end byte?
	btfsc	STATUS,Z		; 
	goto	WX_end_R		; Yes, stop looping
	movlw	wx_data+WX_DATA_MAX_LEN	; address of the last byte for wx_data
	subwf	FSR, W			; do we reach the last byte?
	btfss	STATUS, C		; 
	goto	WX_dalsiRAMB		; Not yet

WX_end_R:
	bank0				; Select page 0 (00H - 7FH)
	bcf	STATUS, IRP		; Set indirect page 0+1
	page0				; Set PCLATCH to program page 0
	call	serialTXcr		; Send CR
	call	serialTXlf		; Send LF
	goto	Main_RX_Eth		; And go back to the Main loop

;------------------------------------------------------------------------------
; DS1302 RTC Routines			RTC string on the RS232 
;     					(sec,min,hour,day,month,weekday,year)
; Vcc2 -|\/|- Vcc1
;   X1 -|  |- SCLK  (RC3) 		data format: <Esc>tSSmmhhDDMMwwYY
;   X2 -|  |- I/O   (RC5)		                   |     |      |
;  GND -|__|- /RST  (RA0) 		offset in RAM reg: 0     6     13
;------------------------------------------------------------------------------
RTC_init				; Initializes DS1302 after powerup
	bank0				; 
	movlw	EE_RTC_ACT		; Is RTC connected?
	call	EEPROM_Read_Byte	; Take the RTC byte from EEPROM
	bank3				;
	movwf	rtc_act			; Place it here to know RTC state
	btfsc	rtc_act, 0		; Is RTC connected? 1=Yes, 0=No
	goto	RTC_needs_initialization	; Configuration says Yes!
	bank0				; 
	return				; RTC is not in configuration!

;------------------------------------------------------------------------------
RTC_needs_initialization		; Needs RTC to be initialized?
	bank1				;
	bcf	sclk			; Make this pin Output (bank1=TRIS reg)
	bcf	rtcres			; Make this pin Output (bank1=TRIS reg)
	bcf	rtcdata			; Make this pin Output (bank1=TRIS reg)
	bank0				; 
	bcf	sclk			; Clock signal LOW
	bcf	rtcres			; /Reset signal LOW
	bcf	rtcdata			; Data I/O signal LOW
	bank3				;
	
	btfsc	rtc_act, 1		; Needs RTC to be initialized? 1=Yes
	goto	RTC_do_init		; Configuration says Yes!
	goto	RTC_get_time		; RTC is backupped, get it's time
	
RTC_do_init				; 
	movlw	rtc_data		; Start of rtc_clock_data 
	movwf	FSR			; into FSR
	bsf	STATUS, IRP		; Set indirect page 2+3
	movlw	rtc_clock_len-1		; take length of clock data-controlREG
	movwf	dataF			; and place it here
	movlw	0x01			; Set clock to 1.1.2001 01:01:01
	bank3				; Select page 3 (180H - 1FFH)

RTC_RAM_init_loop			; 
	movwf	INDF			; Place byte in RAM
	incf	FSR, F			; increase pointer
	decfsz	dataF, F		; All bytes filled?
	goto	RTC_RAM_init_loop	; Not yet!
	clrf	INDF			; set RTC control reg to zero

	ifdef	RTC_TRCHARGE_ENABLE	; 
	call	RTC_enable_WR		; Enable wtite to RCT
	movlw	b'10010000'		; Trickle Charge register write command
	call	RTC_send_byte		; Send one byte to RTC
	movlw	rtc_trchrge		; Enable trickle charge
	call	RTC_send_byte		; Send one byte to RTC
	bcf	rtcres			; /Reset signal LOW
	bcf	sclk			; Clock signal LOW
	page0				; Set PCLATCH to program page 0
	call	Delay50uS		; Wait a while
	endif				; 

;------------------------------------------------------------------------------
RTC_set_clock				; Set RTC clock to time in RAM regs	
	page1				; Set PCLATCH to program page 1
	call	RTC_enable_WR		; Enable wtite to RCT
	movlw	b'10111110'		; Clock burst write command
	call	RTC_send_byte		; Send one byte to RTC
	movlw	rtc_data		; Start of rtc_clock_data 
	movwf	FSR			; into FSR
	bsf	STATUS, IRP		; Set indirect page 2+3
	movlw	rtc_clock_len-1		; take length of clock data-controlREG
	movwf	dataF			; and place it here

RTC_set_clock_loop			; 
	movf	INDF, W			; Take a byte from RAM
	call	RTC_send_byte		; Send one byte to RTC
	incf	FSR, F			; increase pointer
	decfsz	dataF, F		; All bytes filled?
	goto	RTC_set_clock_loop	; Not yet!
	bcf	STATUS, IRP		; Set indirect page 0+1
	movlw	0x80			; Set control reg to disable write
	call	RTC_send_byte		; Send one byte to RTC
	bank3				;
	bcf	rtc_act, 1		; RTC does not need initialization
	goto	RTC_stop_communication	; Finaly end the session with RTC

;------------------------------------------------------------------------------
RTC_enable_WR				; Enable Write to RTC
	call	RTC_set_output		; Set RTC Data line as Output
	bank0				; 
	bcf	sclk			; Clock signal LOW
	bsf	rtcres			; /Reset signal HIGH
	page0				; Set PCLATCH to program page 0
	call	Delay50uS		; Wait a while
	page1				; Set PCLATCH to program page 1
	movlw	b'10001110'		; Control register write command
	call	RTC_send_byte		; Send one byte to RTC
	movlw	0x00			; Set control reg to enable write
	call	RTC_send_byte		; Send one byte to RTC
	bcf	rtcres			; /Reset signal LOW
	bcf	sclk			; Clock signal LOW
	page0				; Set PCLATCH to program page 0
	call	Delay50uS		; Wait a while
	bsf	rtcres			; /Reset signal HIGH
	call	Delay50uS		; Wait a while
	page1				; Set PCLATCH to program page 1
	return				; All done

;------------------------------------------------------------------------------
RTC_set_output				; Set Data pin as Output
	bank1				;
	bcf	rtcdata			; Make this pin Output (bank1=TRIS reg)
	bank0				; 
	return				; All done

;------------------------------------------------------------------------------
RTC_set_input				; Set Data pin as Input
	bank1				;
	bsf	rtcdata			; Make this pin Input (bank1=TRIS reg)
	bank0				; 
	return				; All done

;------------------------------------------------------------------------------
RTC_send_byte				; Send a byte in W to RTC
	movwf	serialData		; save the byte for transmit
	movlw	0x08			; 8 bits 
	movwf	serialCount		;
	bcf	STATUS,C		; Clear carry flag
	rrf	serialData, F		; take the first bit

RTC_send_byte_loop:
	bcf	sclk			; togle clock LOW
	bcf	rtcdata			; Prepare zero bit
	btfsc	STATUS,C		; Skip if carry set
	bsf	rtcdata			; if bit is 1, we send HIGH
	nop				; wait a while
	nop				; wait a while
	bsf	sclk			; togle clock HIGH
	rrf	serialData, F		; rotate bits of the send byte 
	decfsz	serialCount, F		; decrease bit sent counter
	goto	RTC_send_byte_loop	; go for the next bit
	bcf	sclk			; togle clock LOW
	return				; All done

;------------------------------------------------------------------------------
RTC_get_byte				; Get a byte from RTC
	clrf	serialData		; clear the byte for receive
	movlw	0x08			; 8 bits 
	movwf	serialCount		;

RTC_get_byte_loop:
	bcf	sclk			; togle clock LOW
	bcf	STATUS,C		; Clear carry flag
	nop				; wait a while
	btfsc	rtcdata			; Skip if bit from RTC is zero
	bsf	STATUS,C		; if bit is 1, we store HIGH
	bsf	sclk			; togle clock HIGH
	rrf	serialData, F		; rotate bits of the received byte 
	decfsz	serialCount, F		; decrease bit sent counter
	goto	RTC_get_byte_loop	; go for the next bit
	movf	serialData, W		; Place received byte in W
	return				; All done

;------------------------------------------------------------------------------
RTC_stop_communication			; Stop communication with RTC
	bank0				; 
	bcf	rtcres			; /Reset signal LOW
	bcf	sclk			; Clock signal LOW
	bcf	rtcdata			; Data signal LOW
	goto	RTC_set_input		; And set I/O as input

;------------------------------------------------------------------------------
RTC_get_time				; Place time from RTC to RAM regs
	call	RTC_set_output		; Set RTC Data line as Output
	bank0				; 
	bcf	sclk			; Clock signal LOW
	bsf	rtcres			; /Reset signal HIGH
	page0				; Set PCLATCH to program page 0
	call	Delay50uS		; Wait a while
	page1				; Set PCLATCH to program page 1
	movlw	b'10111111'		; Clock burst read command
	call	RTC_send_byte		; Send one byte to RTC
	call	RTC_set_input		; Set RTC Data line as Input
	movlw	rtc_data		; Start of rtc_clock_data 
	movwf	FSR			; into FSR
	bsf	STATUS, IRP		; Set indirect page 2+3
	movlw	rtc_clock_len		; take length of clock data+controlREG
	movwf	dataF			; and place it here

RTC_get_clock_loop			; 
	call	RTC_get_byte		; Get one byte from RTC
	movwf	INDF			; Store it here
	incf	FSR, F			; increase pointer
	decfsz	dataF, F		; All bytes filled?
	goto	RTC_get_clock_loop	; Not yet!
	call	RTC_stop_communication	; Finaly end the session with RTC

	bank3				; 
	movlw	time_data		; Time data ASCII formated starts here
	movwf	FSR			; into FSR
	swapf	rtc_data+rtc_date, W	; Date hi digit
	andlw	b'00000011'		; mask for date hi digit
	call	place_time_digit	; Place it in formated string
	movf	rtc_data+rtc_date, W	; Date lo digit
	call	place_time_digit	; Place it in formated string
	movlw	a'.'			; dot
	call	place_time_data		; Place it in formated string
	swapf	rtc_data+rtc_month, W	; Month, hi digit
	andlw	b'00000001'		; mask for hi digit
	call	place_time_digit	; Place it in formated string
	movf	rtc_data+rtc_month, W	; Month, lo digit
	call	place_time_digit	; Place it in formated string
	movlw	a'.'			; dot
	call	place_time_data		; Place it in formated string
	swapf	rtc_data+rtc_year, W	; Year, hi digit
	call	place_time_digit	; Place it in formated string
	movf	rtc_data+rtc_year, W	; Year, lo digit
	call	place_time_digit	; Place it in formated string
	movlw	0x20			; space
	call	place_time_data		; Place it in formated string
	swapf	rtc_data+rtc_hr, W	; Hour, hi digit
	andlw	b'00000011'		; mask for hi digit
	call	place_time_digit	; Place it in formated string
	movf	rtc_data+rtc_hr, W	; Hour, lo digit
	call	place_time_digit	; Place it in formated string
	movlw	a':'			; colon
	call	place_time_data		; Place it in formated string
	swapf	rtc_data+rtc_min, W	; Minute, hi digit
	andlw	b'00000111'		; mask for hi digit
	call	place_time_digit	; Place it in formated string
	movf	rtc_data+rtc_min, W	; Minute, lo digit
	call	place_time_digit	; Place it in formated string
	movlw	a':'			; colon
	call	place_time_data		; Place it in formated string
	swapf	rtc_data+rtc_sec, W	; Second, hi digit
	andlw	b'00000111'		; mask for hi digit
	call	place_time_digit	; Place it in formated string
	movf	rtc_data+rtc_sec, W	; Second, lo digit
	call	place_time_digit	; Place it in formated string
	movlw	0x20			; space
	call	place_time_data		; Place it in formated string
	movf	rtc_data+rtc_day, W	; Weekday, lo digit
	andlw	b'00000111'		; mask for lo digit
	call	place_time_digit	; Place it in formated string
	bank0				; 
	return				; All done

place_time_digit			; 
	andlw	b'00001111'		; BCD digit mask
	addlw	ASCII_OFFSET		; Add ascii offset
place_time_data				; 
	movwf	INDF			; Place result here
	incf	FSR,F			; Increase pointer
	return				; All done

;------------------------------------------------------------------------------
RTC_set_232				; Expects string: ssmmhhDDMMWWYY
	movlw	rtc_data		; Start of rtc_clock_data 
	movwf	FSR			; into FSR
	bsf	STATUS, IRP		; Set indirect page 2+3
	movlw	rtc_clock_len-1		; take length of clock data-controlREG
	movwf	dataF			; and place it here
	bank0				; Select page 3 (180H - 1FFH)

RTC_set_232_loop			; 
	page0				; Set PCLATCH to program page 0
	call	serial_Receive_BYTE	; receive a byte from RS232
	;page1				; Set PCLATCH to program page 1
	andlw	b'00001111'		; apply BCD mask
	movwf	INDF			; result put here
	swapf	INDF, F			; and move hi digit in high nibble
	;page0				; Set PCLATCH to program page 0
	call	serial_Receive_BYTE	; receive a byte from RS232
	page1				; Set PCLATCH to program page 1
	andlw	b'00001111'		; apply BCD mask
	iorwf	INDF, F			; place lo digit in low nibble
	incf	FSR, F			; increase pointer
	decfsz	dataF, F		; All bytes filled?
	goto	RTC_set_232_loop	; Not yet!
	clrf	INDF			; set RTC control reg to zero
	call	RTC_set_clock		; Set RTC clock to time in RAM regs	
	page0				; Set PCLATCH to program page 0
	goto	Main_RX_Eth		; And go back to the Main loop

;------------------------------------------------------------------------------
RTC_get_232				; 
	call	RTC_get_time		; Place time from RTC to RAM regs
	movlw	time_data		; Time data ASCII formated starts here
	movwf	FSR			; into FSR
	movlw	d'19'			; Time string length
	movwf	dataF			; place here

RTC_get_232_loop			; 
	movf	INDF, W			; Take byte from the Time string
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; Send length as the first byte
	page1				; Set PCLATCH to program page 1
	incf	FSR, F			; Increase pointer
	decfsz	dataF, F		; Decrease counter
	goto	RTC_get_232_loop	; We do not send all bytes yet.
	
	bcf	STATUS, IRP		; Set indirect page 0+1
	page0				; Set PCLATCH to program page 0
	call	serialTXcr		; Send CR
	call	serialTXlf		; Send LF
	goto	Main_RX_Eth		; And go back to the Main loop

;------------------------------------------------------------------------------
RTC_send_sync_pkt			; 
	ifdef	DEBUG0
	page0				; Set PCLATCH to program page 0
	bank0				; 
	call	serialTXcr		; For debug
	movlw	A'u'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'd'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'p'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif
	page2				; Set PCLATCH to program page 2
	goto	UDP_send_daytime_pkt	; Prepare and send daytime UDP packet

;------------------------------------------------------------------------------
time_to_smt_pkt				; Place time on the top of smt packet
	movf	FSR, W			; Destination address in packet
	movwf	adresseH		; store here
	movlw	time_data		; Time data ASCII formated starts here
	movwf	adresseL		; store here 
	movlw	d'17'			; Time string length (without weekday)
	movwf	dataF			; place here

time_to_smt_pkt_loop			; 
	movf	adresseL, W		; Take source address
	movwf	FSR			; make it actual
	bsf	STATUS, IRP		; Set indirect page 2+3
	movf	INDF, W			; Take byte from the Time string
	incf	adresseL, F		; Increase source address
	movwf	byte_tmp		; Save the byte
	movf	adresseH, W		; Take destination address
	movwf	FSR			; make it actual
	bcf	STATUS, IRP		; Set indirect page 0+1
	movf	byte_tmp, W		; Take stored byte
	movwf	INDF			; Place it in the packet
	incf	adresseH, F		; Increase destination address
	incf	AA, F			; Increase pkt bytes counter
	movlw	0x60			; 0x5f is the last pktdata byte in
	subwf	adresseH, W		; bank0
	btfss	zero			; if Z=1, zero substraction, we
	goto	time_to_smt_pkt_next	; 
	movlw	pktdata2		; have to switch to bank1
	movwf	adresseH		; 

time_to_smt_pkt_next			; 
	decfsz	dataF, F		; Decrease counter
	goto	time_to_smt_pkt_loop	; We do not placed all bytes yet.
	movlw	a'>'			; Take timestamp separator
	incf	FSR, F			; Increase destination address
	movwf	INDF			; Place separator in the packet
	incf	AA, F			; Increase pkt bytes counter
	incf	FSR, F			; Increase destination address
	return				; All done
;------------------------------------------------------------------------------
		; EEPROM.ASM

Write_Config_to_EEPROM:		; Writes config data from RS232 to EEPROM
	movlw	0x01			; 
	movwf	TMP			; start on address 0x01

Write_Config_to_EEPROM_Loop:		; Write loop
	page0				; Set PCLATCH to program page 0
	call	serial_Receive_BYTE	; receive a byte from RS232
	page1				; Set PCLATCH to program page 1
	bank2				; bank2
	movwf	EEDATA			; Place in here
	bank0				; bank0
	movf	TMP,W			; Address in EEPROM place in W
	call	EEPROM_Write_Byte	; write one byte into EEPROM
	incf	TMP,F			; point to the next byte
	btfss	STATUS, Z		; all 255 bytes written?
	goto	Write_Config_to_EEPROM_Loop	; not yet
	page0				; Set PCLATCH to program page 0
	goto	Main_RX_Eth		; And go back to the Main loop

Send_Config_from_EEPROM:	; Sends config data from EEPROM to the RS232
	movlw	EE_LEN			; EEPROM data length
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; Send length as the first byte
	page1				; Set PCLATCH to program page 1
	movlw	0x01			; 
	movwf	TMP			; start on address 0x01

Send_Config_from_EEPROM_Loop:		; Read loop
	movf	TMP,W			; Take EEPROM byte address
	call	EEPROM_Read_Byte	; read a byte fro this addres
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; and send it via RS-232
	page1				; Set PCLATCH to program page 1
	incf	TMP,F			; point to the next byte
	btfss	STATUS, Z		; all 255 bytes sent?
	goto	Send_Config_from_EEPROM_Loop	; not yet
	page0				; Set PCLATCH to program page 0
	goto	Main_RX_Eth		; All done, go back to the Main loop

;------------------------------------------------------------------------------
EEPROM_Read_Byte:			; reads a byte from W address in EEPROM
	andlw	0xff			; mask bits for 256 bytes length
	bank2				; bank2
	movwf	EEADR			; address of the byte store here
	bsf	STATUS,RP0		; bank3
	bcf	EECON1,EEPGD		; from EEPROM 
	bsf	EECON1,RD		; we want to read
	bcf	STATUS,RP0		; bank2
	movf	EEDATA,W		; read a byte
	bank0				; bank0
	return				; Return from subroutine


EEPROM_Write_Byte:			; writes byte into W address in EEPROM
	andlw	0xff			; mask bits for 256 bytes length
	bsf	STATUS,RP0		; bank1
	bsf	STATUS,RP1		; bank3
	btfsc	EECON1, WR		; Wait for previous write to complete
	goto	$-1			; if it is not finished yet
	bcf	STATUS,RP0		; bank2
	movwf	EEADR			; address of the byte store here
	bsf	STATUS,RP0		; bank3
	bcf	EECON1,EEPGD		; into EEPROM 
	bsf	EECON1,WREN		; enable write
	bcf	INTCON,GIE		; Disable Interrupts
	movlw	0x55			; REQUIRED WRITE SEQUENCE START
	movwf	EECON2			; Write 55h
	movlw	0xAA			; 
	movwf	EECON2			; Write AAh
	bsf	EECON1,WR		; Set WR bit, write starts here!
	bcf	EECON1,WREN		; Disable EEPROM write
	;bsf	INTCON,GIE		; Enable Interrupts if You need it.

;EEPROM_Write_Complete_Loop:		; waits for write to complete
;		btfsc	EECON1,WR	; write complete?
;		goto	EEPROM_Write_Complete_Loop	; not yet
;		bcf	STATUS,RP0	; write done, Select page 0 (00H - 7FH)
	bank0				; bank0
	return				; Return from subroutine

;------------------------------------------------------------------------------
send_RAM_reg			; Sends RAM regs in ascii format (bank0+2)
	page0				; Set PCLATCH to program page 0
	call	serialTXcr		; 
	;call	serialTXlf		; 
	movlw	0x20			; 
	movwf	FSR			; 
	clrf	CC			; 
	
send_RAM_reg_loop			; 
	page1				; Set PCLATCH to program page 1
	movlw 	0xf0			; 
	andwf	INDF, W			; 
	movwf	BB			; 
	swapf	BB, F			; 
	movlw	0x30			; 
	addwf	BB, F			; 
	movlw	0x3a			; 
	subwf	BB, W			; 
	btfsc	carry			; 
	call	send_RAM_add_offset	; 
	movf	BB, W			; 
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; 
	page1				; Set PCLATCH to program page 1
	movlw 	0x0f			; 
	andwf	INDF, W			; 
	movwf	BB			; 
	movlw	0x30			; 
	addwf	BB, F			; 
	movlw	0x3a			; 
	subwf	BB, W			; 
	btfsc	carry			; 
	call	send_RAM_add_offset	; 
	movf	BB, W			; 
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; 
	movlw	0x20			; 
	call	serial_SEND_byte	; 
	page1				; Set PCLATCH to program page 1
	incf	CC, F			; 
	btfss	CC, 4			; 
	goto	send_RAM_reg_test	; 
	page0				; Set PCLATCH to program page 0
	call	serialTXcr		; 
	;call	serialTXlf		; 
	page1				; Set PCLATCH to program page 1
	clrf	CC			; 

send_RAM_reg_test			; 
	incf	FSR, F			; 
	movlw	0x00			; 
	subwf	FSR, W			; 
	btfsc	zero			; 
	goto	send_RAM_reg_end	; 
	movlw	0x80			; 
	subwf	FSR, W			; 
	btfss	zero			; 
	goto	send_RAM_reg_loop	; 
	page0				; Set PCLATCH to program page 0
	call	serialTXcr		; 
	page1				; Set PCLATCH to program page 1
	goto	send_RAM_reg_loop	; 
		
send_RAM_reg_end			; 
	page0				; Set PCLATCH to program page 0
	call	serialTXcr		; 
	;call	serialTXlf		; 
	return				;

send_RAM_add_offset			;
	movlw	0x07			; 
	addwf	BB, F			; 
	return


;------------------------------------------------------------------------------
; SMT measurement
;------------------------------------------------------------------------------
stridaL		equ	g_val_low		; 
stridaH		equ	g_val_hi		; 
countL		equ	g_len_low		; 
countH		equ	g_len_hi		; 

Start_Mereni					; SMT-160 temperature measure.
		bank3				; bank3
		MOVLW	010H			; for better results measure
		MOVWF	poc_mer			; 16 times, store it in counter
		CLRF	SOUCET			; Clear result bytes
		CLRF	horniSOUCET		;
		MOVLW	020H			; space (positive degres mark)
		MOVWF	pod_nulou		; store here

Mer_smycka					; Measurement loop
		bank0				; bank0 - for I/O access
		CALL	Mereni			; Measure subroutine
		MOVF	stridaH, W		; Take high nibble
		bank3				; back to bank3
		BCF	STATUS, 0		; clear C
		ADDWF	SOUCET, F		; add measured value to result
		BTFSC	STATUS,0		; if we overflowed, increase
		INCF	horniSOUCET, F		; high byte
		DECFSZ	poc_mer, F		; count down measure tasks
		GOTO 	Mer_smycka		; we are not finished yet
			; Measure finished, clear appropriate SMC flag
		btfss	TMP, 0			; Is SMT present?
		goto	$+3			; jump if not
		bcf	TMP, 0			; clear this SMT and
		goto	Count_SMT_tempr		; count measured value
		btfss	TMP, 1			; Is SMT present?
		goto	$+3			; jump if not
		bcf	TMP, 1			; clear this SMT and
		goto	Count_SMT_tempr		; count measured value
		btfss	TMP, 2			; Is SMT present?
		goto	$+3			; jump if not
		bcf	TMP, 2			; clear this SMT and
		goto	Count_SMT_tempr		; count measured value
		btfss	TMP, 3			; Is SMT present?
		goto	$+3			; jump if not
		bcf	TMP, 3			; clear this SMT and
		goto	Count_SMT_tempr		; count measured value
		btfss	TMP, 4			; Is SMT present?
		goto	$+3			; jump if not
		bcf	TMP, 4			; clear this SMT and
		goto	Count_SMT_tempr		; count measured value
		btfss	TMP, 5			; Is SMT present?
		goto	$+3			; jump if not
		bcf	TMP, 5			; clear this SMT and
		goto	Count_SMT_tempr		; count measured value
		btfss	TMP, 6			; Is SMT present?
		goto	$+3			; jump if not
		bcf	TMP, 6			; clear this SMT and
		goto	Count_SMT_tempr		; count measured value
	
Count_SMT_tempr					; 
		; We need to count an average from ours 16 measurements
		; and we get amount of half-degrees from -68C
		; So we will divide by 32 and multiplly by 10 for getting
		; amount of 0.1 degrees (from -68C) and finnaly we add
		; -680 to this result to get amount of 0.1 degrees 
		; This is what we want... measure in tens of degree precission
		; 10/32 = 0.3125
		; 0.3125 = 7D 20 00 00 float (counted by fprep.exe)
		;   -680 = 88 AA 00 00 float (counted by fprep.exe)

		CLRF	AARGB0			; clear high nibble
		MOVF	horniSOUCET, W		; take high byte
		MOVWF	AARGB1			; place it here
		MOVF	SOUCET, W		; take low byte
		MOVWF	AARGB2			; place it here
		CALL	FLO2432			; convert it to the float
		; 0.3125 = 7B 20 00 00 float (z fprep.exe)
		MOVLW	07DH			; Prepare multiplying by
		MOVWF	BEXP			; 0.3125
		MOVLW	020H			; 
		MOVWF	BARGB0			; 
		CLRF	BARGB1			; 
		CLRF	BARGB2			; 
		CALL	FPM32			; Multiply
		;     -680 = 88 AA 00 00 float (z fprep.exe)
		MOVLW	088H			; Prepare addition of number
		MOVWF	BEXP			; -680
		MOVLW	0AAH			; 
		MOVWF	BARGB0			; 
		CLRF	BARGB1			; 
		CLRF	BARGB2			; 
		CALL	FPA32			; Add
		CALL	INT3224			; convert float to int
		MOVF	AARGB0, W		; and store it for 
		MOVWF	temp0			; conversion to 
		MOVF	AARGB1, W		; degrees of
		MOVWF	temp1			; Fahrenheit
		MOVF	AARGB2, W		; into these temporary
		MOVWF	temp2			; registers
		BTFSS	AARGB0, 7		; do we have negative degrees?
		GOTO	C_kladne		; No, positice Celsius degrees
		COMF	AARGB0, F		; Negative deg, do complement
		COMF	AARGB1, F		; Negative deg, do complement
		COMF	AARGB2, F		; Negative deg, do complement
		MOVLW	02DH			; negative degrres mark "-"
		MOVWF	pod_nulou		; store here 

C_kladne					; And finnaly convert counted
		MOVF	AARGB1, W		; 16 bit result into ASCII 
		MOVWF	AARGB0			; 
		MOVF	AARGB2, W		; 
		MOVWF	AARGB1			; 
		movlw	03H	         	; We convert into 3 digits
		movwf	digit_count     	; decimal number
		movlw	desetiny		; decenas comienzan aqui
		bsf	STATUS, IRP		; indirect page 2+3
		CALL	int16ascii		; Convert into ASCII
		bcf	STATUS, IRP		; indirect page 0+1

		RETURN				; And return from subroutine

;------------------------------------------------------------------------------
Mereni		; Based on the routine by Ondrej Vitous from HW server
		MOVLW	020H			; 108ms (425*32*8us) measure
		MOVWF	countL			; 435*32*8=1360=3520 +
		MOVLW	036H			; cycle for countL
		MOVWF	countH			; and countH = 3620h

		CLRF	stridaL			; Clear counters 
		CLRF	stridaH			; Vynulovat citace stridy
		BCF	STATUS, 2		; clear Z flag
		
		btfsc	TMP, 0			; Is SMT present?
		goto	smycka0_9		; measure it's value
		btfsc	TMP, 1			; Is SMT present?
		goto	smycka1_9		; measure it's value
		btfsc	TMP, 2			; Is SMT present?
		goto	smycka2_9		; measure it's value
		btfsc	TMP, 3			; Is SMT present?
		goto	smycka3_9		; measure it's value
		btfsc	TMP, 4			; Is SMT present?
		goto	smycka4_9		; measure it's value
		btfsc	TMP, 5			; Is SMT present?
		goto	smycka5_9		; measure it's value
		btfsc	TMP, 6			; Is SMT present?
		goto	smycka6_9		; measure it's value

Mereni_end					; 
		BCF	STATUS, 0		; 1 clear C
		RLF	stridaL, F		; 1 and rotate
		RLF	stridaH, F		; 1 rotate using
		RLF	stridaL, F		; 1 16bit arithmetics
		RLF	stridaH, F		; 1 (it is multiplying
		RLF	stridaL, F		; 1 by 32*, and we have amount
		RLF	stridaH, F		; 1 of half-degres of celsius)
		RETURN				; So return

;------------------------------------------------------------------------------
smycka0_9	nop				; 1 every 256 cycle = 9us
smycka0_8	BTFSC	smt_0			; 1/2
		INCF	stridaL, F		; 2	increase, if SMT_IN = 1
		BTFSC	STATUS, 2		; 3/4 
		INCF	stridaH, F		; 4	inc if we overflowed...
			
		DECFSZ	countL, F		; 5/6
		GOTO	smycka0_9		; 6+1=7
		DECFSZ	countH, F		; 7/8
		GOTO	smycka0_8		; 7+1=8
		goto	Mereni_end		; Finished, go back

;------------------------------------------------------------------------------
smycka1_9	nop				; 1 every 256 cycle = 9us
smycka1_8	BTFSC	smt_1			; 1/2
		INCF	stridaL, F		; 2	increase, if SMT_IN = 1
		BTFSC	STATUS, 2		; 3/4 
		INCF	stridaH, F		; 4	inc if we overflowed...
			
		DECFSZ	countL, F		; 5/6
		GOTO	smycka1_9		; 6+1=7
		DECFSZ	countH, F		; 7/8
		GOTO	smycka1_8		; 7+1=8
		goto	Mereni_end		; Finished, go back

;------------------------------------------------------------------------------
smycka2_9	nop				; 1 every 256 cycle = 9us
smycka2_8	BTFSC	smt_2			; 1/2
		INCF	stridaL, F		; 2	increase, if SMT_IN = 1
		BTFSC	STATUS, 2		; 3/4 
		INCF	stridaH, F		; 4	inc if we overflowed...
			
		DECFSZ	countL, F		; 5/6
		GOTO	smycka2_9		; 6+1=7
		DECFSZ	countH, F		; 7/8
		GOTO	smycka2_8		; 7+1=8
		goto	Mereni_end		; Finished, go back

;------------------------------------------------------------------------------
smycka3_9	nop				; 1 every 256 cycle = 9us
smycka3_8	BTFSC	smt_3			; 1/2
		INCF	stridaL, F		; 2	increase, if SMT_IN = 1
		BTFSC	STATUS, 2		; 3/4 
		INCF	stridaH, F		; 4	inc if we overflowed...
			
		DECFSZ	countL, F		; 5/6
		GOTO	smycka3_9		; 6+1=7
		DECFSZ	countH, F		; 7/8
		GOTO	smycka3_8		; 7+1=8
		goto	Mereni_end		; Finished, go back

;------------------------------------------------------------------------------
smycka4_9	nop				; 1 every 256 cycle = 9us
smycka4_8	BTFSC	smt_4			; 1/2
		INCF	stridaL, F		; 2	increase, if SMT_IN = 1
		BTFSC	STATUS, 2		; 3/4 
		INCF	stridaH, F		; 4	inc if we overflowed...
			
		DECFSZ	countL, F		; 5/6
		GOTO	smycka4_9		; 6+1=7
		DECFSZ	countH, F		; 7/8
		GOTO	smycka4_8		; 7+1=8
		goto	Mereni_end		; Finished, go back

;------------------------------------------------------------------------------
smycka5_9	nop				; 1 every 256 cycle = 9us
smycka5_8	BTFSC	smt_5			; 1/2
		INCF	stridaL, F		; 2	increase, if SMT_IN = 1
		BTFSC	STATUS, 2		; 3/4 
		INCF	stridaH, F		; 4	inc if we overflowed...
			
		DECFSZ	countL, F		; 5/6
		GOTO	smycka5_9		; 6+1=7
		DECFSZ	countH, F		; 7/8
		GOTO	smycka5_8		; 7+1=8
		goto	Mereni_end		; Finished, go back

;------------------------------------------------------------------------------
smycka6_9	nop				; 1 every 256 cycle = 9us
smycka6_8	BTFSC	smt_6			; 1/2
		INCF	stridaL, F		; 2	increase, if SMT_IN = 1
		BTFSC	STATUS, 2		; 3/4 
		INCF	stridaH, F		; 4	inc if we overflowed...
			
		DECFSZ	countL, F		; 5/6
		GOTO	smycka6_9		; 6+1=7
		DECFSZ	countH, F		; 7/8
		GOTO	smycka6_8		; 7+1=8
		goto	Mereni_end		; Finished, go back


;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;	RCS Header $Id: math16.inc 2.4 1997/02/11 16:58:49 F.J.Testa Exp $

;	$Revision: 2.4 $

;       MATH16 INCLUDE FILE
;
;       IMPORTANT NOTE: The math library routines can be used in a dedicated application on
;       an individual basis and memory allocation may be modified with the stipulation that
;       on the PIC17, P type registers must remain so since P type specific instructions
;	were used to realize some performance improvements.

;*****************************************************************************
;
;       GENERAL MATH LIBRARY DEFINITIONS
;
;	general literal constants

;	define assembler constants

B0		equ	0
B1		equ	1
B2		equ	2
B3		equ	3
B4		equ	4
B5		equ	5
B6		equ	6
B7		equ	7

MSB		equ	7
LSB		equ	0


;     define commonly used bits

;     STATUS bit definitions

		#define	_C		STATUS,0
		#define	_Z		STATUS,2

;
;       general register variables (od 0x20H do 0x34H vcetne)
;

;;ACCB7           equ     0x20
;;ACCB6           equ     0x21
;;ACCB5           equ     0x22
;;ACCB4           equ     0x23
;;ACCB3           equ     0x24
;;ACCB2           equ     0x25
;;ACCB1           equ     0x26
;;ACCB0           equ     0x27
;;ACC             equ     0x27    ; most significant byte of contiguous 8 byte accumulator
;
;SIGN            equ     0x29    ; save location for sign in MSB
;
;;TEMPB3          equ     0x30
;;TEMPB2          equ     0x31
;;TEMPB1          equ     0x32
;;TEMPB0          equ     0x33
;TEMP            equ     0x33    ; temporary storage
;
;       binary operation arguments
;
;;AARGB7          equ     0x20
;;AARGB6          equ     0x21
;AARGB5          equ     0x22
;AARGB4          equ     0x23
;AARGB3          equ     0x24
;AARGB2          equ     0x25
;AARGB1          equ     0x26
;AARGB0          equ     0x27
;AARG            equ     0x27    ; most significant byte of argument A
;
;BARGB3          equ     0x2B
;BARGB2          equ     0x2C
;BARGB1          equ     0x2D
;BARGB0          equ     0x2E
;BARG            equ     0x2E    ; most significant byte of argument B
;
;       Note that AARG and ACC reference the same storage locations
;
;*****************************************************************************
;
;       FIXED POINT SPECIFIC DEFINITIONS
;
;       remainder storage
;
;;REMB3           equ     0x20
;;REMB2           equ     0x21
;REMB1           equ     0x22
;REMB0           equ     0x23    ; most significant byte of remainder
REMB1           equ     AARGB5
REMB0           equ     AARGB4    ; most significant byte of remainder

;LOOPCOUNT       equ     0x34    ; loop counter
;
;*****************************************************************************
;
;       FLOATING POINT SPECIFIC DEFINITIONS
;
;       literal constants
;
EXPBIAS         equ     D'127'
;
;       biased exponents
;
;EXP             equ     0x28    ; 8 bit biased exponent
EXP             equ     AEXP    ; 8 bit biased exponent

;AEXP            equ     0x28    ; 8 bit biased exponent for argument A

;BEXP            equ     0x2F    ; 8 bit biased exponent for argument B
;
;       floating point library exception flags
;
;FPFLAGS         equ     0x2A    ; floating point library exception flags
                
IOV             equ     0       ; bit0 = integer overflow flag

FOV             equ     1       ; bit1 = floating point overflow flag

FUN             equ     2       ; bit2 = floating point underflow flag

FDZ             equ     3       ; bit3 = floating point divide by zero flag

NAN		equ	4	; bit4 = not-a-number exception flag

DOM		equ	5	; bit5 = domain error exception flag

RND             equ     6       ; bit6 = floating point rounding flag, 0 = truncation
                                ; 1 = unbiased rounding to nearest LSB

SAT             equ     7       ; bit7 = floating point saturate flag, 0 = terminate on
                                ; exception without saturation, 1 = terminate on
                                ; exception with saturation to appropriate value

;******************************************************************************

;	RCS Header $Id: fp32.a16 2.8 1996/10/07 13:50:59 F.J.Testa Exp $

;	$Revision: 2.8 $

;       PIC16 32 BIT FLOATING POINT LIBRARY
;
;       Unary operations: both input and output are in AEXP,AARG
;
;       Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;
;       All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;       for an error condition specified in FPFLAGS.
;
;       All timings are worst case cycle counts
;
;         Routine               Function
;
;       FLO2432         24 bit integer to 32 bit floating point conversion
;       FLO32
;
;               Timing:            RND
;                               0       1
;
;                       0       104     104
;                  SAT
;                       1       110     110
;
;       NRM3232   32 bit normalization of unnormalized 32 bit floating point numbers
;       NRM32
;
;               Timing:            RND
;                               0       1
;
;                       0       90      90
;                  SAT
;                       1       96      96
;
;
;       INT3224         32 bit floating point to 24 bit integer conversion
;       INT32
;
;
;               Timing:            RND
;                               0       1
;
;                       0       104      112
;                  SAT
;                       1       104      114
;
;
;       NRM4032 32 bit normalization of unnormalized 40 bit floating point numbers
;
;               Timing:            RND
;                               0       1
;
;                       0       112     128
;                  SAT
;                       1       112     135
;
;
;       FPA32           32 bit floating point add
;
;               Timing:            RND
;                               0       1
;
;                       0       251     265
;                  SAT
;                       1       251     271
;
;       FPS32           32 bit floating point subtract
;
;               Timing:            RND
;                               0       1
;
;                       0       253     267
;                  SAT
;                       1       253     273
;
;       FPM32           32 bit floating point multiply
;
;               Timing:            RND
;                               0       1
;
;                       0       574     588
;                  SAT
;                       1       574     591
;
;******************************************************************************
;******************************************************************************
;
;       32 bit floating point representation
;
;       EXPONENT        8 bit biased exponent
;
;                       It is important to note that the use of biased exponents produces
;                       a unique representation of a floating point 0, given by
;                       EXP = HIGHBYTE = MIDBYTE = LOWBYTE = 0x00, with 0 being
;                       the only number with EXP = 0.
;
;       HIGHBYTE        8 bit most significant byte of fraction in sign-magnitude representation,
;                       with SIGN = MSB, implicit MSB = 1 and radix point to the right of MSB
;
;       MIDBYTE         8 bit middle significant byte of sign-magnitude fraction
;
;       LOWBYTE         8 bit least significant byte of sign-magnitude fraction
;
;       EXPONENT        HIGHBYTE        MIDBYTE         LOWBYTE
;
;       xxxxxxxx        S.xxxxxxx       xxxxxxxx        xxxxxxxx
;
;                        |
;                      RADIX
;                      POINT
;
;
;******************************************************************************
;******************************************************************************

;       Integer to float conversion

;       Input:  24 bit 2's complement integer right justified in AARGB0, AARGB1, AARGB2

;       Use:    CALL    FLO2432 or      CALL    FLO32

;       Output: 32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  FLOAT( AARG )

;       Max Timing:     14+90 = 104 clks                SAT = 0
;                       14+96 = 110 clks                SAT = 1

;       Min Timing:     6+28 = 34 clks                  AARG = 0
;                       6+18 = 24 clks

;       PM: 14+38 = 52                                  DM: 7

;------------------------------------------------------------------------------

FLO2432
FLO32           MOVLW           D'23'+EXPBIAS		; initialize exponent and add bias
                MOVWF           EXP
                CLRF            SIGN
                BTFSS           AARGB0,MSB		; test sign
                GOTO            NRM3232
                COMF            AARGB2,F                ; if < 0, negate and set MSB in SIGN
                COMF            AARGB1,F
                COMF            AARGB0,F
                INCF            AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F
                BSF             SIGN,MSB

;******************************************************************************

;       Normalization routine

;       Input:  32 bit unnormalized floating point number in AEXP, AARGB0, AARGB1,
;               AARGB2, with sign in SIGN,MSB

;       Use:    CALL    NRM3232 or      CALL    NRM32

;       Output: 32 bit normalized floating point number in AEXP, AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  NORMALIZE( AARG )

;       Max Timing:     21+6+7*8+7 = 90 clks            SAT = 0
;                       21+6+7*8+1+12 = 96 clks SAT = 1

;       Min Timing:     22+6 = 28 clks                  AARG = 0
;                       5+9+4 = 18 clks

;       PM: 38                                          DM: 7

;------------------------------------------------------------------------------

NRM3232

NRM32           CLRF            TEMP			; clear exponent decrement
                MOVF            AARGB0,W		; test if highbyte=0
                BTFSS           _Z
                GOTO            NORM3232
                MOVF            AARGB1,W		; if so, shift 8 bits by move
                MOVWF           AARGB0
                MOVF            AARGB2,W
                MOVWF           AARGB1
                CLRF            AARGB2
                BSF             TEMP,3                  ; increase decrement by 8

                MOVF            AARGB0,W		; test if highbyte=0
                BTFSS           _Z
                GOTO            NORM3232
                MOVF            AARGB1,W		; if so, shift 8 bits by move
                MOVWF           AARGB0
                CLRF            AARGB1
                BCF             TEMP,3                  ; increase decrement by 8
                BSF             TEMP,4
        
                MOVF            AARGB0,W		; if highbyte=0, result=0
                BTFSC           _Z
                GOTO            RES032

NORM3232        MOVF            TEMP,W
                SUBWF           EXP,F
                BTFSS           _Z
                BTFSS           _C
                GOTO            SETFUN32

                BCF             _C                      ; clear carry bit

NORM3232A       BTFSC           AARGB0,MSB		; if MSB=1, normalization done
                GOTO            FIXSIGN32
                RLF             AARGB2,F                ; otherwise, shift left and 
                RLF             AARGB1,F                ; decrement EXP
                RLF             AARGB0,F
                DECFSZ          EXP,F
                GOTO            NORM3232A

                GOTO            SETFUN32                ; underflow if EXP=0

FIXSIGN32       BTFSS           SIGN,MSB
                BCF             AARGB0,MSB              ; clear explicit MSB if positive
                RETLW           0

RES032          CLRF            AARGB0                  ; result equals zero
                CLRF            AARGB1
                CLRF            AARGB2
		CLRF		AARGB3
                CLRF            EXP
                RETLW           0

;******************************************************************************

;       Normalization routine

;       Input:  40 bit unnormalized floating point number in AEXP, AARGB0, AARGB1,
;               AARGB2, AARGB3 with sign in SIGN,MSB

;       Use:    CALL    NRM4032

;       Output: 32 bit normalized floating point number in AEXP, AARGB0, AARGB1, AARGB2,
;               AARGB3

;       Result: AARG  <--  NORMALIZE( AARG )

;       Max Timing:     38+6*9+12+8 = 112 clks  RND = 0
;                       38+6*9+12+24 = 128 clks RND = 1, SAT = 0
;                       38+6*9+12+31 = 135 clks RND = 1, SAT = 1

;       Min Timing:     33+6 = 39 clks                  AARG = 0
;                       5+9+8 = 22 clks

;       PM: 66                                          DM: 8

;------------------------------------------------------------------------------

NRM4032         CLRF            TEMP			; clear exponent decrement
                MOVF            AARGB0,W		; test if highbyte=0
                BTFSS           _Z
                GOTO            NORM4032
                MOVF            AARGB1,W		; if so, shift 8 bits by move
                MOVWF           AARGB0
                MOVF            AARGB2,W
                MOVWF           AARGB1
                MOVF            AARGB3,W
                MOVWF           AARGB2
                CLRF            AARGB3
                BSF             TEMP,3                  ; increase decrement by 8

                MOVF            AARGB0,W		; test if highbyte=0
                BTFSS           _Z
                GOTO            NORM4032
                MOVF            AARGB1,W		; if so, shift 8 bits by move
                MOVWF           AARGB0
                MOVF            AARGB2,W
                MOVWF           AARGB1
                CLRF            AARGB2
                BCF             TEMP,3                  ; increase decrement by 8
                BSF             TEMP,4
        
                MOVF            AARGB0,W		; test if highbyte=0
                BTFSS           _Z
                GOTO            NORM4032
                MOVF            AARGB1,W		; if so, shift 8 bits by move
                MOVWF           AARGB0
                CLRF            AARGB1
                BSF             TEMP,3                  ; increase decrement by 8
        
                MOVF            AARGB0,W		; if highbyte=0, result=0
                BTFSC           _Z
                GOTO            RES032

NORM4032        MOVF            TEMP,W
                SUBWF           EXP,F
                BTFSS           _Z
                BTFSS           _C
                GOTO            SETFUN32

                BCF             _C                      ; clear carry bit

NORM4032A       BTFSC           AARGB0,MSB		; if MSB=1, normalization done
                GOTO            NRMRND4032
                RLF             AARGB3,F                ; otherwise, shift left and 
                RLF             AARGB2,F                ; decrement EXP
                RLF             AARGB1,F
                RLF             AARGB0,F
                DECFSZ          EXP,F
                GOTO            NORM4032A

                GOTO            SETFUN32                ; underflow if EXP=0

NRMRND4032      BTFSC           FPFLAGS,RND
                BTFSS           AARGB2,LSB
                GOTO            FIXSIGN32
		BTFSS		AARGB3,MSB		; round if next bit is set
                GOTO            FIXSIGN32
		INCF		AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F

                BTFSS           _Z                      ; has rounding caused carryout?
                GOTO            FIXSIGN32
                RRF             AARGB0,F                ; if so, right shift
                RRF             AARGB1,F
                RRF             AARGB2,F
                INCF            EXP,F
                BTFSC           _Z                      ; check for overflow
                GOTO            SETFOV32
                GOTO            FIXSIGN32

;******************************************************************************
;******************************************************************************

;       Float to integer conversion

;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2

;       Use:    CALL    INT3224         or      CALL    INT32

;       Output: 24 bit 2's complement integer right justified in AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  INT( AARG )

;       Max Timing:     40+6*7+6+16 = 104 clks		RND = 0
;                       40+6*7+6+24 = 112 clks		RND = 1, SAT = 0
;                       40+6*7+6+26 = 114 clks  	RND = 1, SAT = 1

;       Min Timing:     4 clks

;       PM: 82                                          DM: 6

;------------------------------------------------------------------------------

INT3224
INT32
		MOVF		EXP,W			; test for zero argument
		BTFSC		_Z
		RETLW		0x00

		MOVF            AARGB0,W		; save sign in SIGN
                MOVWF           SIGN
                BSF             AARGB0,MSB		; make MSB explicit

                MOVLW           EXPBIAS+D'23'		; remove bias from EXP
                SUBWF           EXP,F
                BTFSS           EXP,MSB
                GOTO            SETIOV3224
		COMF		EXP,F
		INCF		EXP,F

                MOVLW           8                       ; do byte shift if EXP >= 8
                SUBWF           EXP,W
                BTFSS           _C
                GOTO            TSHIFT3224
                MOVWF           EXP
                RLF             AARGB2,F                ; rotate next bit for rounding
                MOVF            AARGB1,W
                MOVWF           AARGB2
                MOVF            AARGB0,W
                MOVWF           AARGB1
                CLRF            AARGB0

                MOVLW           8                       ; do another byte shift if EXP >= 8
                SUBWF           EXP,W
                BTFSS           _C
                GOTO            TSHIFT3224
                MOVWF           EXP
                RLF             AARGB2,F		; rotate next bit for rounding
                MOVF            AARGB1,W
                MOVWF           AARGB2
                CLRF            AARGB1

                MOVLW           8                       ; do another byte shift if EXP >= 8
                SUBWF           EXP,W
                BTFSS           _C
                GOTO            TSHIFT3224
                MOVWF           EXP
                RLF             AARGB2,F                ; rotate next bit for rounding
                CLRF            AARGB2
		MOVF		EXP,W
		BTFSS		_Z
		BCF		_C
		GOTO		SHIFT3224OK

TSHIFT3224      MOVF            EXP,W                   ; shift completed if EXP = 0
                BTFSC           _Z
                GOTO            SHIFT3224OK

SHIFT3224       BCF             _C
                RRF             AARGB0,F                ; right shift by EXP
                RRF             AARGB1,F
                RRF             AARGB2,F
                DECFSZ          EXP,F
                GOTO            SHIFT3224

SHIFT3224OK     BTFSC           FPFLAGS,RND
                BTFSS           AARGB2,LSB
                GOTO            INT3224OK
                BTFSS           _C
                GOTO            INT3224OK
                INCF            AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F
                BTFSC           AARGB0,MSB		; test for overflow
                GOTO            SETIOV3224

INT3224OK       BTFSS           SIGN,MSB                ; if sign bit set, negate               
                RETLW           0
                COMF            AARGB0,F
                COMF            AARGB1,F
                COMF            AARGB2,F
                INCF            AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F
                RETLW           0

IRES03224	CLRF            AARGB0			; integer result equals zero
                CLRF            AARGB1
                CLRF            AARGB2
                RETLW           0

SETIOV3224	BSF             FPFLAGS,IOV             ; set integer overflow flag
                BTFSS           FPFLAGS,SAT             ; test for saturation
                RETLW           0xFF                    ; return error code in WREG

                CLRF            AARGB0			; saturate to largest two's
                BTFSS           SIGN,MSB                ; complement 24 bit integer
                MOVLW           0xFF
                MOVWF           AARGB0			; SIGN = 0, 0x 7F FF FF
                MOVWF           AARGB1			; SIGN = 1, 0x 80 00 00
                MOVWF           AARGB2
                RLF             SIGN,F
                RRF             AARGB0,F
                RETLW           0xFF                    ; return error code in WREG

;******************************************************************************
;******************************************************************************

;       Floating Point Multiply

;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;               32 bit floating point number in BEXP, BARGB0, BARGB1, BARGB2

;       Use:    CALL    FPM32

;       Output: 32 bit floating point product in AEXP, AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  AARG * BARG

;       Max Timing:     26+23*22+21+21 = 574 clks       RND = 0
;                       26+23*22+21+35 = 588 clks       RND = 1, SAT = 0
;                       26+23*22+21+38 = 591 clks       RND = 1, SAT = 1

;       Min Timing:     6+6 = 12 clks                   AARG * BARG = 0
;                       24+23*11+21+17 = 315 clks

;       PM: 94                                          DM: 14

;------------------------------------------------------------------------------

FPM32           MOVF            AEXP,W                  ; test for zero arguments
                BTFSS           _Z
                MOVF            BEXP,W
                BTFSC           _Z
                GOTO            RES032

M32BNE0         MOVF            AARGB0,W
                XORWF           BARGB0,W
                MOVWF           SIGN                    ; save sign in SIGN

                MOVF            BEXP,W
                ADDWF           EXP,F
                MOVLW           EXPBIAS-1
                BTFSS           _C
                GOTO            MTUN32

                SUBWF           EXP,F
                BTFSC           _C
                GOTO            SETFOV32                ; set multiply overflow flag
                GOTO            MOK32

MTUN32          SUBWF           EXP,F
                BTFSS           _C
                GOTO            SETFUN32

MOK32		MOVF		AARGB0,W
		MOVWF		AARGB3
		MOVF		AARGB1,W
		MOVWF		AARGB4
		MOVF		AARGB2,W
		MOVWF		AARGB5
		BSF             AARGB3,MSB              ; make argument MSB's explicit
                BSF             BARGB0,MSB
                BCF             _C
                CLRF            AARGB0			; clear initial partial product
                CLRF            AARGB1
                CLRF            AARGB2
                MOVLW           D'24'
                MOVWF           TEMP                    ; initialize counter

MLOOP32         BTFSS           AARGB5,LSB              ; test next bit
                GOTO            MNOADD32

MADD32          MOVF            BARGB2,W
                ADDWF           AARGB2,F
                MOVF            BARGB1,W
                BTFSC           _C
                INCFSZ          BARGB1,W
                ADDWF           AARGB1,F

                MOVF            BARGB0,W
                BTFSC           _C
                INCFSZ          BARGB0,W
                ADDWF           AARGB0,F

MNOADD32        RRF             AARGB0,F
                RRF             AARGB1,F
                RRF             AARGB2,F
                RRF             AARGB3,F
                RRF             AARGB4,F
                RRF             AARGB5,F
                BCF             _C
                DECFSZ          TEMP,F
                GOTO            MLOOP32

                BTFSC           AARGB0,MSB               ; check for postnormalization
                GOTO            MROUND32
                RLF             AARGB3,F
                RLF             AARGB2,F
                RLF             AARGB1,F
                RLF             AARGB0,F
                DECF            EXP,F

MROUND32        BTFSC           FPFLAGS,RND
                BTFSS           AARGB2,LSB
                GOTO            MUL32OK
		BTFSS		AARGB3,MSB
                GOTO            MUL32OK
		INCF		AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F

                BTFSS           _Z                      ; has rounding caused carryout?
                GOTO            MUL32OK
                RRF             AARGB0,F                ; if so, right shift
                RRF             AARGB1,F
                RRF             AARGB2,F
                INCF            EXP,F
                BTFSC           _Z                      ; check for overflow
                GOTO            SETFOV32

MUL32OK         BTFSS           SIGN,MSB
                BCF             AARGB0,MSB		; clear explicit MSB if positive

                RETLW           0  

SETFOV32        BSF             FPFLAGS,FOV             ; set floating point underflag
                BTFSS           FPFLAGS,SAT             ; test for saturation
                RETLW           0xFF                    ; return error code in WREG

                MOVLW           0xFF
                MOVWF           AEXP                    ; saturate to largest floating
                MOVWF           AARGB0                  ; point number = 0x FF 7F FF FF
                MOVWF           AARGB1                  ; modulo the appropriate sign bit
                MOVWF           AARGB2
                RLF             SIGN,F
                RRF             AARGB0,F
                RETLW           0xFF                    ; return error code in WREG

;******************************************************************************

SETFUN32        BSF             FPFLAGS,FUN             ; set floating point underflag
                BTFSS           FPFLAGS,SAT             ; test for saturation
                RETLW           0xFF                    ; return error code in WREG

                MOVLW           0x01                    ; saturate to smallest floating
                MOVWF           AEXP                    ; point number = 0x 01 00 00 00
                CLRF            AARGB0                  ; modulo the appropriate sign bit
                CLRF            AARGB1
                CLRF            AARGB2
                RLF             SIGN,F
                RRF             AARGB0,F
                RETLW           0xFF                    ; return error code in WREG

;******************************************************************************
;******************************************************************************

;       Floating Point Subtract

;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;               32 bit floating point number in BEXP, BARGB0, BARGB1, BARGB2

;       Use:    CALL FPS32

;       Output: 32 bit floating point sum in AEXP, AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  AARG - BARG

;       Max Timing:     2+251 = 253 clks                RND = 0
;                       2+265 = 267 clks                RND = 1, SAT = 0
;                       2+271 = 273 clks                RND = 1, SAT = 1

;       Min Timing:     2+12 = 14 clks

;       PM: 2+146 = 148                         DM: 14

;------------------------------------------------------------------------------

FPS32           MOVLW           0x80
                XORWF           BARGB0,F

;******************************************************************************

;       Floating Point Add

;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;               32 bit floating point number in BEXP, BARGB0, BARGB1, BARGB2

;       Use:    CALL FPA32

;       Output: 32 bit floating point sum in AEXP, AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  AARG - BARG

;       Max Timing:     31+41+6*7+6+41+90 = 251 clks            RND = 0
;                       31+41+6*7+6+55+90 = 265 clks            RND = 1, SAT = 0
;                       31+41+6*7+6+55+96 = 271 clks            RND = 1, SAT = 1

;       Min Timing:     8+4 = 12 clks

;       PM: 146                                                 DM: 14

;------------------------------------------------------------------------------

FPA32           MOVF            AARGB0,W                ; exclusive or of signs in TEMP
                XORWF           BARGB0,W
                MOVWF           TEMP

		CLRF		AARGB3			; clear extended byte
		CLRF		BARGB3

                MOVF            AEXP,W                  ; use AARG if AEXP >= BEXP
                SUBWF           BEXP,W
                BTFSS           _C
                GOTO            USEA32

                MOVF            BEXP,W                  ; use BARG if AEXP < BEXP
                MOVWF           AARGB5			; therefore, swap AARG and BARG
                MOVF            AEXP,W
                MOVWF           BEXP
                MOVF            AARGB5,W
                MOVWF           AEXP

                MOVF            BARGB0,W
                MOVWF           AARGB5
                MOVF            AARGB0,W
                MOVWF           BARGB0
                MOVF            AARGB5,W
                MOVWF           AARGB0

                MOVF            BARGB1,W
                MOVWF           AARGB5
                MOVF            AARGB1,W
                MOVWF           BARGB1
                MOVF            AARGB5,W
                MOVWF           AARGB1

                MOVF            BARGB2,W
                MOVWF           AARGB5
                MOVF            AARGB2,W
                MOVWF           BARGB2
                MOVF            AARGB5,W
                MOVWF           AARGB2

USEA32          MOVF            BEXP,W                  ; return AARG if BARG = 0
                BTFSC           _Z
                RETLW           0x00

                MOVF            AARGB0,W
                MOVWF           SIGN                    ; save sign in SIGN
                BSF             AARGB0,MSB              ; make MSB's explicit
                BSF             BARGB0,MSB

                MOVF            BEXP,W                  ; compute shift count in BEXP
                SUBWF           AEXP,W
                MOVWF           BEXP
                BTFSC           _Z
                GOTO            ALIGNED32

                MOVLW           8
                SUBWF           BEXP,W
                BTFSS           _C                      ; if BEXP >= 8, do byte shift
                GOTO            ALIGNB32
                MOVWF           BEXP
                MOVF            BARGB2,W		; keep for postnormalization
		MOVWF		BARGB3
                MOVF            BARGB1,W
		MOVWF		BARGB2
                MOVF            BARGB0,W
		MOVWF		BARGB1
                CLRF            BARGB0

                MOVLW           8
                SUBWF           BEXP,W
                BTFSS           _C                      ; if BEXP >= 8, do byte shift
                GOTO            ALIGNB32
                MOVWF           BEXP
                MOVF            BARGB2,W		; keep for postnormalization
		MOVWF		BARGB3
                MOVF            BARGB1,W
		MOVWF		BARGB2
                CLRF            BARGB1

                MOVLW           8
                SUBWF           BEXP,W
                BTFSS           _C                      ; if BEXP >= 8, BARG = 0 relative to AARG
                GOTO            ALIGNB32
                MOVF            SIGN,W
                MOVWF           AARGB0
                RETLW           0x00

ALIGNB32        MOVF            BEXP,W                  ; already aligned if BEXP = 0
                BTFSC           _Z
                GOTO            ALIGNED32

ALOOPB32        BCF             _C                      ; right shift by BEXP
                RRF             BARGB0,F
                RRF             BARGB1,F
		RRF		BARGB2,F
		RRF		BARGB3,F
                DECFSZ          BEXP,F
                GOTO            ALOOPB32

ALIGNED32       BTFSS           TEMP,MSB                ; negate if signs opposite
                GOTO            AOK32

		COMF		BARGB3,F
		COMF		BARGB2,F
                COMF            BARGB1,F
                COMF            BARGB0,F
                INCF            BARGB3,F
                BTFSC           _Z
                INCF            BARGB2,F
		BTFSC		_Z
		INCF		BARGB1,F
		BTFSC		_Z
		INCF		BARGB0,F

AOK32
                MOVF   		BARGB3,W
                ADDWF   	AARGB3,F
                MOVF            BARGB2,W
                BTFSC           _C
                INCFSZ          BARGB2,W
                ADDWF           AARGB2,F
                MOVF            BARGB1,W
                BTFSC           _C
                INCFSZ          BARGB1,W
                ADDWF           AARGB1,F
                MOVF            BARGB0,W
                BTFSC           _C
                INCFSZ          BARGB0,W
                ADDWF           AARGB0,F

                BTFSC           TEMP,MSB
                GOTO            ACOMP32
                BTFSS           _C
                GOTO            NRMRND4032

                RRF             AARGB0,F               ; shift right and increment EXP
                RRF             AARGB1,F
                RRF             AARGB2,F
		RRF		AARGB3,F
                INCFSZ          AEXP,F
                GOTO            NRMRND4032
                GOTO            SETFOV32

ACOMP32         BTFSC           _C
                GOTO            NRM4032			; normalize and fix sign

		COMF		AARGB3,F
                COMF            AARGB2,F		; negate, toggle sign bit and
                COMF            AARGB1,F		; then normalize
                COMF            AARGB0,F
                INCF            AARGB3,F
                BTFSC           _Z
                INCF            AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F

                MOVLW           0x80
                XORWF           SIGN,F
                GOTO            NRM32


		; AN670 32BIT NUMBER TO ASCII (ale zatim jen do cisla 9999!)

;****************************************************************************
;*		Floating Point to ASCII
;*
;* INPUT: 32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;*        For this example, the number must be between 0.000 and 9.999.
;*        (You can easily change the number range to suit your needs.)
;*
;* OUTPUT: ones, tenths, hundredths, thousandths (ASCII digit in each)
;*         
;*
;* USED: INDF,FSR,AARG,BARG,REMB,digit_count
;*
;* PROCEDURE: The floating point number in AARG is multiplied by 1000.
;*            Then the product is divided by 10 three times.  After each
;*            divide, the remainder is kept.  
;*
;*            After each digit is obtained, 30H is added to it to make it
;*            an ASCII representation of that number.  Then, the ASCII
;*            value is stored in register RAM at the locations specified
;*            in the "cblock."      
;*
;*	Note: The ASCII decimal point is not generated by this routine.
;*            You must output the decimal point in the correct decimal
;*            position for your application.  For this example, the
;*            decimal point is after the first digit: ones.
;*            
;* The following files are needed for this routine to function.
;*  p16c74a.inc		-- or any other midrange processor include file 
;*                         include the processor file in your "main" file
;*
;*  math16.inc          -- constant and variable definitions for 
;*                         AN575 floating point routines and
;*                         AN617 fixed point routines, both are used
;*                         in this float to ASCII routine
;*                         "include" this file in your "main" program
;*
;*  fxd26.a16 		-- fixed point 32/16 divide, included at the end
;*                         of this routine.
;*
;*  fp32.a16		-- 32 bit float to 32 bit integer conversion 
;*                         included at the end of this program. 
;* 
;****************************************************************************


;RAM Register Definitions       Your "main" program must have a "cblock"
                                ;directive with a RAM address so the 
                                ;following "cblock" will be located in RAM

;digit_count	equ	035H	;counter used to cycle through each digit 

int16ascii			; Prevod 16bitove binarni hodnoty na ascii
	; nasledujici tri radky udelat pred volanim rutiny, tak aby rutina dostala 
	; adresu kam ukladat vysledek ve 'W' a pocet mist byl v digit_count
	;movlw	SIG_FIG         ;load counter with the number of
	;movwf	digit_count     ;significant figures the decimal number	
	;movlw	last_digit	;

	movwf	FSR		;pointer = address of smallest digit

flo_asclp
	clrf	BARGB0		;Make the divisor 10.  
	movlw	d'10'
	movwf	BARGB1

	call	FXD1616U	;divide (16 bit fixed) / 10 (to get remainder)
	
	movf	REMB1,w		;put remainder in w register
	movwf	INDF		;put number into appropriate digit position

	movlw	0x30
	addwf	INDF,f		;add 30H to decimal number to convert to ASCII

	decf	FSR,f		;move pointer to next digit 	

	decfsz	digit_count,f
	goto	flo_asclp

	return


;******************************************************************************
;******************************************************************************

;	RCS Header $Id: fxd66.a16 2.4 1997/02/27 01:20:22 F.J.Testa Exp $

;	$Revision: 2.4 $

;       16/16 PIC16 FIXED POINT DIVIDE ROUTINES
;
;       Input:  fixed point arguments in AARG and BARG
;
;       Output: quotient AARG/BARG followed by remainder in REM
;
;       All timings are worst case cycle counts
;
;       It is useful to note that the additional unsigned routines requiring a non-power of two
;       argument can be called in a signed divide application where it is known that the
;       respective argument is nonnegative, thereby offering some improvement in
;       performance.
;
;         Routine            Clocks     Function
;
;       FXD1616U    373 16 bit/16 bit -> 16.16 unsigned fixed point divide
;
;       The above timings are based on the looped macros. If space permits,
;       approximately 65-69 clocks can be saved by using the unrolled macros.
;

;******************************************************************************
;******************************************************************************

;       16/16 Bit Division Macros


UDIV1616L       macro

;       restore = 23 clks,  nonrestore = 17 clks

;       Max Timing:     2+15*23+22 = 369 clks

;       Min Timing:     2+15*17+16 = 273 clks

;       PM: 24                                  DM: 7

                MOVLW           D'16'
                MOVWF           LOOPCOUNT

LOOPU1616       RLF             AARGB0,W
                RLF             REMB1, F
                RLF             REMB0, F
                MOVF            BARGB1,W
                SUBWF           REMB1, F
                MOVF            BARGB0,W
                BTFSS           _C
                INCFSZ          BARGB0,W
                SUBWF           REMB0, F

                BTFSC           _C
                GOTO            UOK66LL         
                MOVF            BARGB1,W
                ADDWF           REMB1, F
                MOVF            BARGB0,W
                BTFSC           _C
                INCFSZ          BARGB0,W
                ADDWF           REMB0, F

                BCF             _C

UOK66LL RLF             AARGB1, F
                RLF             AARGB0, F

                DECFSZ          LOOPCOUNT, F
                GOTO            LOOPU1616

                endm

;******************************************************************************
;******************************************************************************
        
;       16/16 Bit Unsigned Fixed Point Divide 16/16 -> 16.16

;       Input:  16 bit unsigned fixed point dividend in AARGB0, AARGB1
;               16 bit unsigned fixed point divisor in BARGB0, BARGB1

;       Use:    CALL    FXD1616U

;       Output: 16 bit unsigned fixed point quotient in AARGB0, AARGB1
;               16 bit unsigned fixed point remainder in REMB0, REMB1

;       Result: AARG, REM  <--  AARG / BARG

;       Max Timing:     2+369+2 = 373 clks

;       Min Timing:     2+273+2 = 277 clks

;       PM: 2+24+1 = 27         DM: 7

FXD1616U        CLRF            REMB0
                CLRF            REMB1

                UDIV1616L

                RETLW           0x00

;******************************************************************************
;******************************************************************************

	org	0x1000			; Program memory page 2

;------------------------------------------------------------------------------
; DayTime protocol routines 
;------------------------------------------------------------------------------
UDP_synchonize_RTC			; DayTime pkt: Fri Dec  2 14:48:37 2005
 	clrf	TMP			; Clear table position counter
	movlw	0x01			; Set result number to one
	movwf	byte_tmp		; and save it here

find_day_loop				; Find day number in Day table
	call	DayTab			; Take a letter from table
	xorwf	pktdata+udp_data+0, W	; Day first letter
	btfss	zero			; 
	goto	inc3dTMP		; If not equal jump to next day
	incf	TMP, F			; Go to next letter
	call	DayTab			; Take a letter from table
	xorwf	pktdata+udp_data+1, W	; Day second letter
	btfss	zero			; 
	goto	inc2dTMP		; If not equal jump to next day
	incf	TMP, F			; Go to next letter
	call	DayTab			; Take a letter from table
	xorwf	pktdata+udp_data+2, W	; Day third letter
	btfss	zero			; 
	goto	inc1dTMP		; If not equal jump to next day
	goto	have_weekday		; We found weekday!
inc3dTMP				;
	incf	TMP, F			; Jump over 3 bytes
inc2dTMP				;
	incf	TMP, F			; Jump over 2 bytes
inc1dTMP				;
	incf	TMP, F			; Jump over 1 byte
	incf	byte_tmp, F		; Increase weekday number
	movlw	d'8'			; Stop if 8 weekday reached
	subwf	byte_tmp, W		; 
	btfss	carry			; 
	goto	find_day_loop		; We do not processes all days
	goto	DayTime_bad_format	; If weekday not found jump there

have_weekday				; We have weekday number
	movf	byte_tmp, W		; take it 
	bank3				;
	movwf	rtc_data+rtc_day	; and store in the right position
	bank0				; 

 	clrf	TMP			; Clear table position counter
	movlw	0x01			; Set result number to one
	movwf	byte_tmp		; and save it here

find_month_loop				; 
	call	MonthTab		; Take a letter from table
	xorwf	pktdata+udp_data+4, W	; Month first letter
	btfss	zero			; 
	goto	inc3mTMP		; If not equal jump to next month
	incf	TMP, F			; Go to next letter
	call	MonthTab		; Take a letter from table
	xorwf	pktdata+udp_data+5, W	; Month second letter
	btfss	zero			; 
	goto	inc2mTMP		; If not equal jump to next month
	incf	TMP, F			; Go to next letter
	call	MonthTab		; Take a letter from table
	xorwf	pktdata+udp_data+6, W	; Month third letter
	btfss	zero			; 
	goto	inc1mTMP		; If not equal jump to next month
	goto	have_month		; We have month number!
inc3mTMP				;
	incf	TMP, F			; Jump over 3 bytes
inc2mTMP				;
	incf	TMP, F			; Jump over 2 bytes
inc1mTMP				;
	incf	TMP, F			; Jump over 1 byte
	incf	byte_tmp, F		; Increase month number
	movlw	d'13'			; Stop if 13 month reached
	subwf	byte_tmp, W		; 
	btfss	carry			; 
	goto	find_month_loop		; We do not processes all month
	goto	DayTime_bad_format	; If month not found jump here

have_month				; We have month number
	movf	byte_tmp, W		; take it
	sublw	d'9'			; if month is (10, 11, 12)
	btfsc	carry			; 
	goto	month_ok		; month < 10, BCD is correcr
	movlw	d'10'			; month > 9, set correct
	subwf	byte_tmp, F		; BCD number in
	bsf	byte_tmp, 4		; byte_tmp reg
	

month_ok				; We have month in right
	movf	byte_tmp, W		; BCD format here
	bank3				;
	movwf	rtc_data+rtc_month	; so store in the right position
	movlw	pktdata+udp_data+8	; Day first digit address
	movwf	FSR			; place here
	call	get_daytime_number	; Get daytime value from udp packet
	movwf	rtc_data+rtc_date	; and store BCD in the right position
	incf	FSR, F			;
	incf	FSR, F			; Hour first digit address
	call	get_daytime_number	; Get daytime value from udp packet
	movwf	rtc_data+rtc_hr		; and store BCD in the right position
	incf	FSR, F			;
	incf	FSR, F			; Minute first digit address
	call	get_daytime_number	; Get daytime value from udp packet
	movwf	rtc_data+rtc_min	; and store BCD in the right position
	incf	FSR, F			;
	incf	FSR, F			; Second first digit address
	call	get_daytime_number	; Get daytime value from udp packet
	movwf	rtc_data+rtc_sec	; and store BCD in the right position
	movlw	pktdata+0x82	 	; Year first digit address (0xa2)
	movwf	FSR			; place here
	call	get_daytime_number	; Get daytime value from udp packet
	movwf	rtc_data+rtc_year	; and store BCD in the right position
	clrf	rtc_data+rtc_cntr	; set RTC control reg to zero
	page1				; Set PCLATCH to program page 1
	call	RTC_set_clock		; Set RTC clock to time in RAM regs	
	return				; All done

get_daytime_number			; Get daytime value from udp packet
	movf	INDF, W			; Day first number
	xorlw	0x20			; is it space?
	movlw	0x30			; prepare zero in w
	btfss	zero			; skip if it was space
	movf	INDF, W			; Day first number
	andlw	0x0f			; BCD mask
	movwf	byte_tmp		; place digit here
	swapf	byte_tmp, F		; and make it tens
	incf	FSR, F			; Take second digit
	movf	INDF, W			; Day second number
	andlw	0x0f			; BCD mask
	iorwf	byte_tmp, W		; and 
	return				; All done

DayTime_bad_format			; 
	;call	sendRS232badDayTime	; 
	return				; All done
	
;------------------------------------------------------------------------------
DayTab					; 
	movf	TMP, W			; 
	addwf	PCL,F			; Relative addressing
	retlw	a'M'			; Mon
	retlw	a'o'			; 
	retlw	a'n'			; 
	retlw	a'T'			; Tue
	retlw	a'u'			; 
	retlw	a'e'			; 
	retlw	a'W'			; Wen
	retlw	a'e'			; 
	retlw	a'd'			; 
	retlw	a'T'			; Thu
	retlw	a'h'			; 
	retlw	a'u'			; 
	retlw	a'F'			; Fri
	retlw	a'r'			; 
	retlw	a'i'			; 
	retlw	a'S'			; Sat
	retlw	a'a'			; 
	retlw	a't'			; 
	retlw	a'S'			; Sun
	retlw	a'u'			; 
	retlw	a'n'			; 

;------------------------------------------------------------------------------
MonthTab				; 
	movf	TMP, W			; 
	addwf	PCL,F			; Relative addressing
	retlw	a'J'			; Jan
	retlw	a'a'			; 
	retlw	a'n'			; 
	retlw	a'F'			; Feb
	retlw	a'e'			; 
	retlw	a'b'			; 
	retlw	a'M'			; Mar
	retlw	a'a'			; 
	retlw	a'r'			; 
	retlw	a'A'			; Apr
	retlw	a'p'			; 
	retlw	a'r'			; 
	retlw	a'M'			; May
	retlw	a'a'			; 
	retlw	a'y'			; 
	retlw	a'J'			; Jun
	retlw	a'u'			; 
	retlw	a'n'			; 
	retlw	a'J'			; Jul
	retlw	a'u'			; 
	retlw	a'l'			; 
	retlw	a'A'			; Aug
	retlw	a'u'			; 
	retlw	a'g'			; 
	retlw	a'S'			; Sep
	retlw	a'e'			; 
	retlw	a'p'			; 
	retlw	a'O'			; Oct
	retlw	a'c'			; 
	retlw	a't'			; 
	retlw	a'N'			; Nov
	retlw	a'o'			; 
	retlw	a'v'			; 
	retlw	a'D'			; Dec
	retlw	a'e'			; 
	retlw	a'c'			; 

;------------------------------------------------------------------------------
UDP_send_daytime_pkt			; Prepare and send daytime UDP packe
	page0				; Set PCLATCH to program page 0
	bank0				; 
	call	clear_packet_area	; Clear RX packet RAM area
	call	set_my_src_ip		; Set my IP as source in packet
	call	reply_set_my_mac	; Set my MAC as source in packet
	page2				; Set PCLATCH to program page 2
	call	set_gw_mac_dst		; Set GW MAC as destination + IP type
	call	set_ip_header		; Sets constant bytes in IP header
	call	set_dtime_ip_dst	; Set DayTime IP as destination IP 
	movlw	D'17'			; 17 = protocol UDP
	movwf	pktdata+ip_prot		; Set IP protocol to UDP
	movlw	D'8'			; Set UDP length
	movwf	pktdata+udp_len+1	; to 8 bytes (no udp data in packet)
	addlw	D'20'			; Set IP Total length to
	movwf	pktdata+ip_len+1	; 28 bytes
	clrf	pktdata+eth_len_h	; Length of Ethernet packet
	addlw	D'14'			; udp08+ip20+mac14=42
	movwf	pktdata+eth_len_l	; 
	movlw	0x49			; Set UDP source port
	movwf	pktdata+udp_src+0	; to
	movlw	0x19			; 18713 (my favourite number :-))
	movwf	pktdata+udp_src+1	; 
	page0				; Set PCLATCH to program page 0
	call	udp_do_cksum		; Count and fill UDP checksum
	call	send_pkt		; Send packet
	return				; All done

;------------------------------------------------------------------------------
set_ip_header				; Sets constant bytes in IP header
	movlw	0x45			; IP ver. 4, header len=5x4bytes
	movwf	pktdata+ip_verlen	;
	movwf	pktdata+ip_id+1		; Identification will be 0x0045
	movwf	pktdata+ip_ttl		; TTL will be 0x45
	bsf	pktdata+ip_froff, 6	; Set do not fragment flag 
	return				; All done

;------------------------------------------------------------------------------
set_gw_mac_dst				; Set GW MAC as destination + IP type
	movlw	GW_MAC0			; GW MAC addres
	movwf	FSR			; into FSR
	bsf	STATUS, IRP		; indirect page 2+3
	movf	INDF, W			; Take address
	movwf	pktdata+eth_dest_0	; place it in the packet
	incf	FSR, F			; next byte
	movf	INDF, W			; Take address
	movwf	pktdata+eth_dest_1	; place it in the packet 
	incf	FSR, F			; next byte
	movf	INDF, W			; Take address
	movwf	pktdata+eth_dest_2	; place it in the packet 
	incf	FSR, F			; next byte
	movf	INDF, W			; Take address
	movwf	pktdata+eth_dest_3	; place it in the packet 
	incf	FSR, F			; next byte
	movf	INDF, W			; Take address
	movwf	pktdata+eth_dest_4	; place it in the packet 
	incf	FSR, F			; next byte
	movf	INDF, W			; Take address
	movwf	pktdata+eth_dest_5	; place it in the packet 
	incf	FSR, F			; next byte
	movf	INDF, W			; Take address
	bcf	STATUS, IRP		; indirect page 0+1
	movlw	0x08			; Set Eth type to IP
	movwf	pktdata+eth_typ_0	; 
	movlw	0x00			; Set Eth type to IP
	movwf	pktdata+eth_typ_1	; 
	return				; All done
;------------------------------------------------------------------------------
set_dtime_ip_dst			; dtime IP+PORT from EEPROM to packet
	page1				; Set PCLATCH to program page 1
	movlw	EE_DTIME_IP0		; Load dtime server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+ip_dst+0	; 
	movlw	EE_DTIME_IP1		; Load dtime server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+ip_dst+1	; 
	movlw	EE_DTIME_IP2		; Load dtime server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+ip_dst+2	; 
	movlw	EE_DTIME_IP3		; Load dtime server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+ip_dst+3	; 
	movlw	EE_DTIME_PORTH		; Load dtime server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+udp_dst+0	; 
	movlw	EE_DTIME_PORTL		; Load dtime server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+udp_dst+1	; 
	return				; All done

;------------------------------------------------------------------------------
; Aprsd Telnet client
;------------------------------------------------------------------------------
TCP_send_aprsd_SYN_pkt			; Prepare and send SYN to aprsd srv
	ifdef	DEBUG0
	page0				; Set PCLATCH to program page 0
	bank0				; 
	call	serialTXcr		; For debug
	movlw	A'a'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'p'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'r'			;
	call	serial_SEND_byte	; For debug 
	movlw	A's'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'd'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif

	page0				; Set PCLATCH to program page 0
	bank0				; 
	call	clear_packet_area	; Clear RX packet RAM area
	call	set_my_src_ip		; Set my IP as source in packet
	call	reply_set_my_mac	; Set my MAC as source in packet
	page2				; Set PCLATCH to program page 2
	call	set_gw_mac_dst		; Set GW MAC as destination + IP type
	call	set_ip_header		; Sets constant bytes in IP header
	call	set_aprsd_ip_dst	; Set Aprsd IP as destination IP 
	movlw	D'6'			; 6 = protocol TCP
	movwf	pktdata+ip_prot		; Set IP protocol to UDP
	movlw	b'00000010'		; Set SYN in packet
	movwf	pktdata+tcp_ctrl	; packet
	movlw	0x02			; Option Max Segment size (4B)
	movwf	pktdata+tcp_data+0	; kind = 0x02 (MSS)
	movlw	0x04			;
	movwf	pktdata+tcp_data+1	; length = 0x04 bytes
	clrf	pktdata+tcp_data+2	; We have small buffer
	movlw	D'188'			; bank 1,2 = 80+80
	movwf	pktdata+tcp_data+3	; bank 0 = 28 TCP bytes => 188
	movlw	0x48			; Set TCP source port
	movwf	pktdata+tcp_src_port+0	; to
	movlw	0xf1			; 18673 (my favourite number :-))
	movwf	pktdata+tcp_src_port+1	; 
	movlw	0x45			; Set TCP seqn
	movwf	pktdata+tcp_seqn+2	; 
	movlw	D'168'			; Window set to 168 bytes (80+80+8)
	movwf	pktdata+tcp_window+1	; 
	page0				; Set PCLATCH to program page 0
	call	tcp_syn_MSS_done	; Do checksums, ... and send the pkt
	page0				; Set PCLATCH to program page 0
	goto	Main			; And continue looping

;------------------------------------------------------------------------------
set_aprsd_ip_dst			; dtime IP+PORT from EEPROM to packet
	page1				; Set PCLATCH to program page 1
	movlw	EE_APRSD_IP0		; Load dtime server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+ip_dst+0	; 
	movlw	EE_APRSD_IP1		; Load aprsd server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+ip_dst+1	; 
	movlw	EE_APRSD_IP2		; Load aprsd server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+ip_dst+2	; 
	movlw	EE_APRSD_IP3		; Load aprsd server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+ip_dst+3	; 
	movlw	EE_APRSD_PORTH		; Load aprsd server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+tcp_dst_port+0	; 
	movlw	EE_APRSD_PORTL		; Load aprsd server address and port
	call	EEPROM_Read_Byte	; 
	movwf	pktdata+tcp_dst_port+1	; 
	return				; All done

;------------------------------------------------------------------------------
tcp_do_aprsd				; Aprsd packet with data received...
					; Data length is in TMP
	bank3				; 
	bcf	aprsd_tout		; we are receiving data!
	bcf	aprsd_UIsnd		; we do not sent <UI> yet
	btfss	aprsd_form		; Which output format to use?
	goto	tcp_do_aprsd_normal	; Normal!

tcp_do_aprsd_UIView			; Timestamp: "YYYY-MM-DD HH:MM:SS "
					; from: DD.MM.YY HH:MM:SS d
	page1				; Set PCLATCH to program page 1
	call	RTC_get_time		; Place time from RTC to RAM regs
	page0				; Set PCLATCH to program page 0
	movlw	a'2'			; 
	call	serial_SEND_byte	; Send the century
	movlw	a'0'			; 
	call	serial_SEND_byte	; 
	bsf	STATUS, IRP		; indirect page 0+1
	movlw	time_data+0x06		; Year data ASCII formated 
	movwf	FSR			; into FSR
	movf	INDF, W			; 
	call	serial_SEND_byte	; Send first digit
	incf	FSR, F			; 
	movf	INDF, W			; 
	call	serial_SEND_byte	; Send second digit
	movlw	a'-'			; 
	call	serial_SEND_byte	; Send separator
	movlw	time_data+0x03		; Month data ASCII formated 
	movwf	FSR			; into FSR
	movf	INDF, W			; 
	call	serial_SEND_byte	; Send first digit
	incf	FSR, F			; 
	movf	INDF, W			; 
	call	serial_SEND_byte	; Send second digit
	movlw	a'-'			; 
	call	serial_SEND_byte	; Send separator
	movlw	time_data+0x00		; Date data ASCII formated 
	movwf	FSR			; into FSR
	movf	INDF, W			; 
	call	serial_SEND_byte	; Send first digit
	incf	FSR, F			; 
	movf	INDF, W			; 
	call	serial_SEND_byte	; Send second digit
	movlw	time_data+0x08		; Space before Hour data ASCII formated 
	movwf	FSR			; into FSR
	movlw	0x09			; 
	movwf	byte_tmp		; " HH:MM:SS" = 9 bytes

tcp_do_aprsd_UIView_loop		; 
	movf	INDF, W			; 
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; Send byte
	page2				; Set PCLATCH to program page 2
	incf	FSR, F			; increase pointer
	decfsz	byte_tmp, F		; decrease counter
	goto	tcp_do_aprsd_UIView_loop	; Loop for the next byte
	movlw	a' '			; 
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; Send separator
	bcf	STATUS, IRP		; indirect page 0+1
	page2				; Set PCLATCH to program page 2

tcp_do_aprsd_normal			; Here do normal packet send to RS232
	bank0				; 
	movlw	0x0f			; Mask for IP header len
	andwf	pktdata+ip_verlen, W	; 
	sublw	0x05			; Std Header len
	btfsc	zero			; if len is std, continue
	goto	tcp_do_aprsd_starts	; 
	movwf	byte_tmp		; place result here
	comf	byte_tmp, F		; make result positive
	bcf	carry			; clear carry
	rlf	byte_tmp, F		; and multiply by 4 
	rlf	byte_tmp, W		; to get data offset

tcp_do_aprsd_starts			; 
	addlw	pktdata+tcp_data	; First data byte
	movwf	FSR			; Place here
	bcf	STATUS, IRP		; indirect page 0+1
	movf	TMP, W			; 
	movwf	byte_tmp		; 
	
tcp_do_aprs_rs232_loop			; Send packet data to RS-232
	movf	INDF, W			; Take a byte
	xorlw	a':'			; is it Path/Data separator?
	btfss	zero			; Skip if not
	goto	tcp_do_aprs_rs232_lp2	; 
	bank3				; 
	btfss	aprsd_form		; Which output format to use?
	goto	tcp_do_aprs_rs232_lp1	; 
	btfsc	aprsd_UIsnd		; Send <UI> only on first ":"
	goto	tcp_do_aprs_rs232_lp1	; 
	bsf	aprsd_UIsnd		; we send <UI> just now
	bank0				; 	
	page0				; Set PCLATCH to program page 0
	movlw	a' '			; 
	call	serial_SEND_byte	; Send separator
	movlw	a'<'			; 
	call	serial_SEND_byte	; Send separator
	movlw	a'U'			; 
	call	serial_SEND_byte	; Send separator
	movlw	a'I'			; 
	call	serial_SEND_byte	; Send separator
	movlw	a'>'			; 
	call	serial_SEND_byte	; Send separator
	movlw	a':'			; 
	call	serial_SEND_byte	; Send separator
	call	serialTXcr		;
	call	serialTXlf		;
	incf	FSR, F			; increase pointer
	call	do_indirect_bank_tests	; indirect address bank maintenance
	decf	byte_tmp, F		; decrement counter
	page2				; Set PCLATCH to program page 2

tcp_do_aprs_rs232_lp1			; 
	bank0				; 	

tcp_do_aprs_rs232_lp2			; 
	movf	INDF, W			; Take a byte
	page0				; Set PCLATCH to program page 0
	call	serial_SEND_byte	; 
	incf	FSR, F			; increase pointer
	call	do_indirect_bank_tests	; indirect address bank maintenance
	page2				; Set PCLATCH to program page 2
	decfsz	byte_tmp, F		; decrement bytes counter
	goto	tcp_do_aprs_rs232_loop	; And loop for the next byte

	;clrf	INDF			; clear the byte for checksumming
	page0				; Set PCLATCH to program page 0
	bcf	STATUS, IRP		; indirect page 0+1
	goto	tcp_send_ack		; And send ack for received packet

;------------------------------------------------------------------------------
store_TCP_ackn_seqn			; Store ackn and seqn
	bank0				; 
	movf	pktdata+tcp_seqn+0, W	; 
	bsf	STATUS, RP1		; bank2
	movwf	seqn1			; 
	bcf	STATUS, RP1		; bank0
	movf	pktdata+tcp_seqn+1, W	; 
	bsf	STATUS, RP1		; bank2
	movwf	seqn2			; 
	bcf	STATUS, RP1		; bank0
	movf	pktdata+tcp_seqn+2, W	; 
	bsf	STATUS, RP1		; bank2
	movwf	seqn3			; 
	bcf	STATUS, RP1		; bank0
	movf	pktdata+tcp_seqn+3, W	; 
	bsf	STATUS, RP1		; bank2
	movwf	seqn4			; 
	bcf	STATUS, RP1		; bank0
	movf	pktdata+tcp_ackn+0, W	; 
	bank3				; 
	movwf	ackn1			; 
	bank0				; 
	movf	pktdata+tcp_ackn+1, W	; 
	bank3				; 
	movwf	ackn2			; 
	bank0				; 
	movf	pktdata+tcp_ackn+2, W	; 
	bank3				; 
	movwf	ackn3			; 
	bank0				; 
	movf	pktdata+tcp_ackn+3, W	; 
	bank3				; 
	movwf	ackn4			; 
	bank0				; 
	return				; All done

;------------------------------------------------------------------------------
set_stored_acknseqn			; Stored ackn and seqn into pkt
	bank2				; 
	movf	seqn1, W		; 
	bcf	STATUS, RP1		; bank0
	movwf	pktdata+tcp_seqn+0	; 
	bsf	STATUS, RP1		; bank2
	movf	seqn2, W		; 
	bcf	STATUS, RP1		; bank0
	movwf	pktdata+tcp_seqn+1	; 
	bsf	STATUS, RP1		; bank2
	movf	seqn3, W		; 
	bcf	STATUS, RP1		; bank0
	movwf	pktdata+tcp_seqn+2	; 
	bsf	STATUS, RP1		; bank2
	movf	seqn4, W		; 
	bcf	STATUS, RP1		; bank0
	movwf	pktdata+tcp_seqn+3	; 
	bank3				; 
	movf	ackn1, W		; 
	bank0				; 
	movwf	pktdata+tcp_ackn+0	; 
	bank3				; 
	movf	ackn2, W		; 
	bank0				; 
	movwf	pktdata+tcp_ackn+1	; 
	bank3				; 
	movf	ackn3, W		; 
	bank0				; 
	movwf	pktdata+tcp_ackn+2	; 
	bank3				; 
	movf	ackn4, W		; 
	bank0				; 
	movwf	pktdata+tcp_ackn+3	; 
	return				; All done

;------------------------------------------------------------------------------
TCP_prep_aprsd_pkt			; Prepare aprsd pkt (for sending data)
	page0				; Set PCLATCH to program page 0
	bank0				; 
	call	clear_packet_area	; Clear RX packet RAM area
	call	set_my_src_ip		; Set my IP as source in packet
	call	reply_set_my_mac	; Set my MAC as source in packet
	page2				; Set PCLATCH to program page 2
	call	set_gw_mac_dst		; Set GW MAC as destination + IP type
	call	set_ip_header		; Sets constant bytes in IP header
	call	set_aprsd_ip_dst	; Set Aprsd IP as destination IP 
	page2				; Set PCLATCH to program page 2
	movlw	D'6'			; 6 = protocol TCP
	movwf	pktdata+ip_prot		; Set IP protocol to TCP
	movlw	b'00010000'		; Set ACK in packet
	movwf	pktdata+tcp_ctrl	; packet
	movlw	0x48			; Set TCP source port
	movwf	pktdata+tcp_src_port+0	; to
	movlw	0xf1			; 18673 (my favourite number :-))
	movwf	pktdata+tcp_src_port+1	; 
	movlw	D'168'			; Window set to 168 bytes (80+80+8)
	movwf	pktdata+tcp_window+1	; 
	call	set_stored_acknseqn	;
	movlw	pktdata+tcp_data	; Start of data in packet
	movwf	FSR			; place here
	bcf	STATUS, IRP		; indirect page 0+1
	return				; All done

;------------------------------------------------------------------------------
TCP_send_aprsd_AUTH_pkt			; Send authorize packet
	bank3				; 
	btfss	aprsd_TXena		; If flag not set,
	goto	TCP_send_aprsd_DISABLED	; data sending is disabled!

	ifdef	DEBUG0
	page0				; Set PCLATCH to program page 0
	bank0				; 
	call	serialTXcr		; For debug
	movlw	A'a'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'p'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'a'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'u'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	page2				; Set PCLATCH to program page 2
	endif

	call	TCP_prep_aprsd_pkt	; Prepare aprsd pkt (for sending data)
	movlw	EE_APRSD_CSTR		; Connect string EEPROM address
	movwf	byte_tmp		; place here
	clrf	AA			; data bytes counter

TCP_send_aprsd_AUTH_loop		; 
	movf	byte_tmp, W		; take EEPROM address
	page1				; Set PCLATCH to program page 1
	call	EEPROM_Read_Byte	; read its value
	page2				; Set PCLATCH to program page 2
	iorlw	0x00			; Is it end mark?
	btfsc	zero			; 
	goto	TCP_send_aprsd_AUTH_end	; End mark, stop the loop!
	call	Place_aprsd_byte	; Place byte in the pkt
	goto	TCP_send_aprsd_AUTH_loop	; And loop for the next byte

TCP_send_aprsd_AUTH_end			; 
	clrf	INDF			; Clear this for checksumming
	bcf	STATUS, IRP		; indirect page 0+1
	bank3				; 
	bsf	aprsd_auth		; set auth flag
	bank0				; 
	page0				; Set PCLATCH to program page 0
	goto	Send_tcp_final		; Send reply (data length in AA)

;------------------------------------------------------------------------------
TCP_send_aprsd_WX_pkt			; Send authorize packet
	bank3				; 
	btfss	aprsd_TXena		; If flag not set,
	goto	TCP_send_aprsd_DISABLED	; data sending is disabled!

	ifdef	DEBUG0
	page0				; Set PCLATCH to program page 0
	bank0				; 
	call	serialTXcr		; For debug
	movlw	A'a'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'p'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'w'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'x'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	page2				; Set PCLATCH to program page 2
	bank3				; 
	endif

	movf	wx_data+00, W		; Take first WX byte
	xorlw	a'-'			; if it is dash, no data received!
	btfsc	zero			; 
	goto	TCP_send_aprsd_WX_ret	; If no wx data, then return

	call	TCP_prep_aprsd_pkt	; Prepare aprsd pkt (for sending data)
	movlw	EE_APRSD_CALL		; Connect string EEPROM address
	movwf	byte_tmp		; place here
	clrf	AA			; data bytes counter

TCP_send_aprsd_WXP_loop			; 
	movf	byte_tmp, W		; take EEPROM address
	page1				; Set PCLATCH to program page 1
	call	EEPROM_Read_Byte	; read its value
	page2				; Set PCLATCH to program page 2
	iorlw	0x00			; Is it end mark?
	btfsc	zero			; 
	goto	TCP_send_aprsd_WX_data	; End mark, stop the loop!
	call	Place_aprsd_byte	; Place byte in the pkt
	goto	TCP_send_aprsd_WXP_loop	; And loop for the next byte

TCP_send_aprsd_WX_data			; 
	bank3				; 
	movf	wx_data+0x00, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x01, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x02, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x03, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x04, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x05, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x06, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x07, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x08, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x09, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x0a, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x0b, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x0c, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x0d, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x0e, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x0f, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x10, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x11, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x12, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x13, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x14, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x15, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x16, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt
	movf	wx_data+0x17, W		; Take WX byte
	call	Place_aprsd_byte	; Place byte in the pkt

TCP_send_aprsd_WX_commt			; 
	bank0				; 
	movlw	EE_APRSD_WXCOM		; Connect string EEPROM address
	movwf	byte_tmp		; place here

TCP_send_aprsd_WXC_loop			; 
	movf	byte_tmp, W		; take EEPROM address
	page1				; Set PCLATCH to program page 1
	call	EEPROM_Read_Byte	; read its value
	page2				; Set PCLATCH to program page 2
	iorlw	0x00			; Is it end mark?
	btfsc	zero			; 
	goto	TCP_send_aprsd_WX_end	; End mark, stop the loop!
	call	Place_aprsd_byte	; Place byte in the pkt
	goto	TCP_send_aprsd_WXC_loop	; And loop for the next byte

TCP_send_aprsd_WX_end			; 
	clrf	INDF			; Clear this for checksumming
	bcf	STATUS, IRP		; indirect page 0+1
	page0				; Set PCLATCH to program page 0
	goto	Send_tcp_final		; Send reply (data length in AA)

TCP_send_aprsd_DISABLED			; data sending is disabled!
TCP_send_aprsd_WX_ret			; no wx data in buffer!
	page0				; Set PCLATCH to program page 0
	return				; No WX data, so return...

;------------------------------------------------------------------------------
Place_aprsd_byte			; Place byte in W into aprsd pkt
	movwf	INDF			; place value in the packet
	incf	byte_tmp, F		; increase EEPROM pointer
	incf	AA, F			; increase bytes counter
	incf	FSR, F			; increase pkt pointer
	page0				; Set PCLATCH to program page 0
	call	do_indirect_bank_tests	; indirect address bank maintenance
	page2				; Set PCLATCH to program page 2
	return				; All done

;------------------------------------------------------------------------------
Change_aprs_format			; Change format UIView/normal
	page0				; Set PCLATCH to program page 0
	movlw	A'O'			;
	call	serial_SEND_byte	; 
	movlw	A':'			;
	call	serial_SEND_byte	; 
	call	serial_Receive_BYTE	; receive a byte from RS232
	page2				; Set PCLATCH to program page 2
	bank2				; bank2
	movwf	EEDATA			; Place in here
	bank3				; bank2
	bcf	aprsd_form		; prepare normal format
	xorlw	a'U'			; UIView format??
	btfsc	zero			; skip if not
	bsf	aprsd_form		; set UIView format
	bank0				; bank0
	movlw	EE_APRSD_FORMAT		; Address in EEPROM place in W
	page1				; Set PCLATCH to program page 1
	call	EEPROM_Write_Byte	; write one byte into EEPROM
	page0				; Set PCLATCH to program page 0
	goto	Main_RX_Eth		; And go back to the Main loop

;------------------------------------------------------------------------------
; Send Echo Reply - response to ICMP "ping" packet
;------------------------------------------------------------------------------
do_ping
	page0				; Set PCLATCH to program page 0
	ifdef	DEBUG0
	movlw	A'P'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'I'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'N'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'G'			;
	call	serial_SEND_byte	; For debug 
	movlw	A'.'			;
	call	serial_SEND_byte	; For debug 
	endif

	call	ip_check_IP		; Verify if packet is for my IP
	xorlw	0x01			; 0x01 means Ok
	skpz				; 
	return				; IP differs, return 
	
        clrf    pktdata+icmp_cksum	; clear ICMP cheksum
        clrf    pktdata+icmp_cksum+1	; 
        clrf    pktdata+icmp_typ	; ICMP type 0 (Echo reply)
        clrf    pktdata+icmp_func	; ICMP code (0 for Echo reply)
	call	reply_set_ip		; Set dest. and sender IP
	call	reply_set_mac		; Set dest. and sender MAC
	call	ip_do_cksum		; Calculate IP checksum

	movlw	pktdata+icmp_typ	; ICMP header start 
	movwf	FSR			; place in FSR

        movlw   d'20'			; ip header length 
        subwf   pktdata+ip_len+1,W	; substract from IP packet len.
        movwf   AA			; and result place here

	btfsc	AA,0			; is the result even?
	incf	AA,W			; is odd, make it even 

        call    oc_startsum		; Compute checksum
	call	oc_getsum		; Get computed checkcum
        movf    BB,W			; Take checsum bytes 
        movwf   pktdata+icmp_cksum	; and place them into
        movf    AA,W			; ICMP header checsum area
        movwf   pktdata+icmp_cksum+1	; in the packet

	call	send_pkt		; Send packet (Echo Reply)
	return				; All done

;------------------------------------------------------------------------------
; Punto de arranque del programa y preparacion de datos
;------------------------------------------------------------------------------
Start					; Esta es la entrada al programa
	page0				; pon PCLATCH a la pagina 0 del programa
	clrf 	Flag			; Limpia el registro de banderas de eventos
	bank1				; bank1
	movlw	b'10000111'		; PORTB pullups inhabilitados...
	movwf 	OPTION_REG		; Ajusta OPTION REG
	bcf	STATUS, RP0		; bank0
	clrf 	PORTA			; Inicializa PORTA
	bank1				; bank1
	movlw 	0x06			; Configura todos los pins ANALOGICOS/DIGITALES
	movwf 	ADCON1			; como I/O digital

        movlw   b'11110001'		; PORTA 0,4,5 entradas
        movwf   TRISA           	;       1,2,3 salidas
        movlw   b'11111111'		; PORTB todas entradas
        movwf   TRISB           	; 
        movlw   b'10111000'		; PORTC 3,4,5,7 entradas
        movwf   TRISC			;       0,1,2,6 salidas
	movlw 	b'11111111'		; PORTD todas entradas
	movwf 	TRISD			; 
	movlw 	b'00000000'		; PORTE todas salidas
	movwf 	TRISE           	; 

;	movlw	D'25'			; RS232 baud 9600 @ 4MHz
;	movwf	SPBRG			; 
;	movlw	0x24			; TX enable, High Baud Rate
;	movwf	TXSTA			; selected
	bcf	STATUS, RP0		; bank0

	clrf	PORTA			; bits de PORTA a cero
	clrf	PORTB			; bits de PORTB a cero
	clrf	PORTC			; bits de PORTC a cero
	clrf	PORTE			; bits de PORTE a cero
	bsf	ior			; poner ior de ISA a inactiva
	bsf	iow			; poner iow de ISA a inactiva
	bcf	reset			; poner seal de reset ISA a inactiva

;	movlw	0x90			; Puerto Serie (RC7/RX+RC6/TX)
;	movwf	RCSTA			; y Rx continuamente habilitada
	
	ifdef	XTAL4MHZ		; 
	movlw	0x1E			; RS232 delay para 9600 a 4MHz
	endif				;
	ifdef	XTAL10MHZ		; 
	movlw	0x52			; RS232 delay para 9600 a 10MHz
	endif				;
	ifdef	XTAL12MHZ		; 
	movlw	0x63			; RS232 delay para 9600 a 12MHz
	endif				;
	ifdef	XTAL20MHZ		; 
	movlw	0xA9			; RS232 delay para 9600 a 20MHz = A9
	endif				;
	movwf	delay_time		; 
	ifdef	XTAL4MHZ		; 
	movlw	0x0A			; RS232 delay de 1/3 para 4MHz
	endif				;
	ifdef	XTAL10MHZ		; 
	movlw	0x1B			; RS232 delay de 1/3 para 10MHz
	endif				;
	ifdef	XTAL12MHZ		; 
	movlw	0x21			; RS232 delay de 1/3 para 12MHz
	endif				;
	ifdef	XTAL20MHZ		; 
	movlw	0x38			; RS232 delay de 1/3 para 20MHz = 38
	endif				;
	movwf	delay_time3		;


	page1				; Ajuste de PCLATCH para programar pagina 1
	movlw	EE_IP0			; Ajustar mi IP inicial
	call	EEPROM_Read_Byte	; 
	movwf	IP0			; 
	movlw	EE_IP1			; Ajustar mi IP inicial
	call	EEPROM_Read_Byte	; 
	movwf	IP1			; 
	movlw	EE_IP2			; Ajustar mi IP inicial
	call	EEPROM_Read_Byte	; 
	movwf	IP2			; 
	movlw	EE_IP3			; Ajustar mi IP inicial
	call	EEPROM_Read_Byte	; 
	movwf	IP3			; 
	movlw	EE_GW_IP0		; Ajustar la IP del GW por default
	call	EEPROM_Read_Byte	; 
	bank2				;
	movwf	GW_IP0			; 
	movlw	EE_GW_IP1		; Ajustar la IP del GW por default
	call	EEPROM_Read_Byte	; 
	bank2				;
	movwf	GW_IP1			; 
	movlw	EE_GW_IP2		; Ajustar la IP del GW por default
	call	EEPROM_Read_Byte	; 
	bank2				;
	movwf	GW_IP2			; 
	movlw	EE_GW_IP3		; Ajustar la IP del GW por default
	call	EEPROM_Read_Byte	; 
	bank2				;
	movwf	GW_IP3			; 
	bank0				; 
	
	movlw	EE_WX_ENDB		; Toma el byte final de WX de la EEPROM
	call	EEPROM_Read_Byte	; 
	bank3				;
	movwf	wx_data_endR		; Pone el dato WX aqui
	bank0				; 
	movlw	EE_SMT_ACT		; Toma el byte activo SMT desde la EEPROM
	call	EEPROM_Read_Byte	; 
	bank3				;
	movwf	smt_act			; Lo pone aqui para saber donde estan los SMTs
	clrf	tmr_counter		; Limpia el contador para re-enviar paquete GW
	movlw	0xec			; 
	movwf	timingL			; 0xec para 4Mhz = 10s
	movlw	0xff			; 
	movwf	timingH			; Lo pone a 0xFF para sincronia despues del arranque
	clrf	srvs_flags		; Limpia las banderas del registro de servidores
	bank0				;
	movlw	EE_APRSD_ACT		; Debo conectarme a un server aprsd?
	call	EEPROM_Read_Byte	; 
	bank3				;
	xorlw	a'Y'			; 'Y' = Yes, De otra forma No!
	btfsc	zero			; Si EEPROM dice 'Y' 
	bsf	aprsd_conn		; pon a 1 la bandera de conexion
	bank0				;
	movlw	EE_DTIME_ACT		; Debo sincronizar via server DayTimer?
	call	EEPROM_Read_Byte	; 
	bank3				;
	xorlw	a'Y'			; 'Y' = Yes, De otra forma No!
	btfsc	zero			; Si EEPROM dice 'Y' 
	bsf	dtime_conn		; pon a 1 la bandera de conexion
	bank0				;
	movlw	EE_APRSD_FORMAT		; Cual formto de salida de datos para aprsd?
	call	EEPROM_Read_Byte	; 
	bank3				;
	xorlw	a'U'			; 'U' = UIView, de otra forma normal.
	btfsc	zero			; Si EEPROM dice 'U' 
	bsf	aprsd_form		; pon a 1 la bandera de formato de dato
	bank0				;
	movlw	EE_APRSD_TX		; Esta habilitado el envo a aprsd?
	call	EEPROM_Read_Byte	; 
	bank3				;
	xorlw	a'Y'			; 'Y' = podemos enviar dato a aprsd
	btfsc	zero			; Si EEPROM dice 'Y' 
	bsf	aprsd_TXena		; pon a 1 la bandera de habilitacin
	bank0				;

	movlw	0x0b			; El timer har sobreflujo a 500ms@4MHz
	movwf	TMR1H			; 
	movlw	0xdc			; 
	movwf	TMR1L			; 
	movlw	b'00110100'		; TMR1_presc1:8 conn to intosc
	movwf	T1CON			; pero no arranques TMR1 an
	

	page1				; Ajusta PCLATCH para programar la pagina 1
	call	WX_RAMinit		; Pon los bytes de WX en RAM a '-'
	call	RTC_init		; Inicializa el RTC DS1302
	page0				; Pon PCLATCH para programar la pagina 0
	bcf	STATUS, IRP		; Direccionamiento indirecto de pagina 0+1
	call	Delay1S			; Delay de 1 segundo

	page1				; Pon PCLATCH para programar la pagina 1
	call	Init3c509b		; Inicializa la NIC 3C509b
	page0				; Pon PCLATCH para programar la pagina 0

	call	Send_Version		; Envia la version del firmware por RS232
	bsf led				; Enciende el LED
	call	Delay1S			; Delay de 1 segundo
	bcf led				; Apaga el LED
	bsf	T1CON, TMR1ON		; Arranca TMR1
	
Send_arp_gw_req				; 
	page0				; Pon PCLATCH para programar la pagina 0
	call	do_arp_gw_req		; Intenta leer la MAC de mi gateway
	clrf	TMR1H			; Borra el timer
	bcf	PIR1, TMR1IF		; Limpia la bandera de interrupciones

	page0				; Pon PCLATCH para programar la pagina 0
	goto	Main			; Regresa a main
	

;------------------------------------------------------------------------------
; Web pages area
;------------------------------------------------------------------------------
	org	0x1800			; Program memory page 3
	
	; El primer archivo es el unico devuelto, si el archivo apropiano no se encuentra!
	include	index_2.inc		; pagina index.html
	include	p3c_4bsm.inc		; pagina con la imagen jpeg
	include	wx_2.inc		; pagina de ambiente
	retlw	0xff			; mrca de fin de archivo

;------------------------------------------------------------------------------
; EEPROM area
;------------------------------------------------------------------------------

	org	0x2100			; 

EE_IP0		equ	0x01		; Guarda esta estacin
EE_IP1		equ	0x02		; La IP esta en esos 
EE_IP2		equ	0x03		; cuatro bytes
EE_IP3		equ	0x04		; 
EE_GW_IP0	equ	0x05		; Guarda el gateway
EE_GW_IP1	equ	0x06		; cuya IP de esta estacin est en estos
EE_GW_IP2	equ	0x07		; en esos cuatro bytes
EE_GW_IP3	equ	0x08		; 
EE_USE_GW	equ	0x09		; Indica si debemos usar GW MAC
					; si iniciamos una conexin...
EE_WX_ENDB	equ	0x0a		; caracter de terminacin del dato WX
EE_SMT_ACT	equ	0x0b		; Hay un SMTs conectado? 0 = NOTconn.
EE_RTC_ACT	equ	0x0c		; Estado del modulo RTC (DS1302)

EE_APRSD_ACT	equ	0x0e		; Se conecto a un server aprsd?
EE_DTIME_ACT	equ	0x0f		; Se sincronizara con un server DayTime?

EE_APRSD_IP0	equ	0x10		; Guarda el server aprsd
EE_APRSD_IP1	equ	0x11		; La direccin IP
EE_APRSD_IP2	equ	0x12		; en estos cuatro bytes
EE_APRSD_IP3	equ	0x13		; 
EE_APRSD_PORTH	equ	0x14		; y el numero del puerto TCP
EE_APRSD_PORTL	equ	0x15		; aqui
EE_DTIME_IP0	equ	0x16		; Almacena el server DayTime
EE_DTIME_IP1	equ	0x17		; Su IP
EE_DTIME_IP2	equ	0x18		; en esos cuatro bytes
EE_DTIME_IP3	equ	0x19		; 
EE_DTIME_PORTH	equ	0x1a		; y el numero del puerto TCP
EE_DTIME_PORTL	equ	0x1b		; aqui
EE_APRSD_FORMAT	equ	0x1c		; aprsd data 'U' = UIView, otro = normal
EE_APRSD_TX	equ	0x1d		; Debo transmitir algo mas al aprsd?

EE_APRSD_CSTR	equ	0x80		; cadena de conexion Aprsd
EE_APRSD_CALL	equ	0xC0		; Aprsd WX CALL+POS
EE_APRSD_WXCOM	equ	0xE8		; Aprsd WX commentarios

	; RS-232 delay time should be: ((ClockSpeed/4/SerialSpeed)-14)/3
	; RS-232 9k6 @10MHz => 52h   9k6 @4MHz => 1Eh
	; RS-232 9600 a 12 MHz ==> 63h
	; RS-232 9K6 @20MHz => A9h
	
	de	0x00			; 
	de	d'192'			; definicin de mi IP address
	de	d'168'			; 
	de	d'0'			; 
	de	d'17'			; 
	de	d'192'			; definicin de mi GW IP address
	de	d'168'			; 
	de	d'0'			; 
	de	d'1'			; 
	de	b'00000001'		; Uso de GW si se esta iniciando una conexion
	de	a'C'			; Caracter con el cual termina el dato WX
	de	b'00001010'		; SMTs: NC-RC7-RC6-RC5-RC4-RC3-RA5-RA0
	de	b'00110011'		; RTC: xx11 xxIP (Inicializa,Presentado)
	de	0x20			;
	de	a'Y'			; Connectar a un server Aprsd?
	de	a'Y'			; Sincroniza via server DayTime?
	de	d'66'			; Definicion de la IP address del server Aprsd
	de	d'206'			; 
	de	d'11'			; 
	de	d'192'			; 
	de	0x00			; puerto TCP 1316
	de	0x50			; 
	de	d'194'			; Definicion de la IP address del server DayTime
	de	d'228'			; 
	de	d'2'			; 
	de	d'8'			; 
	de	0x48			; puerto TCP 18613
	de	0xb5			; 
	de	a'N'			; N = normal, U = UIView aprs fuera de formato
	de	a'Y'			; A = TX para aprsd permitido, en otro caso, deshabilitado

	org	0x2180			; 

	de	a'u'			; cadena de conexion para Aprsd
	de	a's'			; 
	de	a'e'			; 
	de	a'r'			; 
	de	a' '			; 
	de	a'N'			; 
	de	a'O'			; 
	de	a'C'			; 
	de	a'A'			; 
	de	a'L'			; 
	de	a'L'			; 
	de	a'-'			; 
	de	a'1'			; 
	de	a' '			; 
	de	a'p'			; 
	de	a'a'			; 
	de	a's'			; 
	de	a's'			; 
	de	a' '			; 
	de	a'1'			; 
	de	a'2'			; 
	de	a'3'			; 
	de	a'4'			; 
	de	a'5'			; 
	de	a' '			; 
	de	a'v'			; 
	de	a'e'			; 
	de	a'r'			; 
	de	a's'			; 
	de	a' '			; 
	de	a'P'			; 
	de	a'I'			; 
	de	a'C'			; 
	de	a'3'			; 
	de	a'C'			; 
	de	a'o'			; 
	de	a'm'			; 
	de	0x0a			; 
	de	0x00			; marca de End

	org	0x21C0			; 

	de	a'N'			; Aprsd WX CALL+POS
	de	a'O'			; 
	de	a'C'			; 
	de	a'A'			; 
	de	a'L'			; 
	de	a'L'			; 
	de	a'-'			; 
	de	a'1'			; 
	de	a'>'			; 
	de	a'A'			; 
	de	a'P'			; 
	de	a'R'			; 
	de	a'S'			; 
	de	a'D'			; 
	de	a':'			; 
	de	a'!'			; 
	de	a'5'			; 
	de	a'0'			; 
	de	a'0'			; 
	de	a'0'			; 
	de	a'.'			; 
	de	a'0'			; 
	de	a'0'			; 
	de	a'N'			; 
	de	a'/'			; 
	de	a'0'			; 
	de	a'1'			; 
	de	a'4'			; 
	de	a'3'			; 
	de	a'0'			; 
	de	a'.'			; 
	de	a'0'			; 
	de	a'0'			; 
	de	a'E'			; 
	de	a'_'			; 
	de	0x00			; marca de End

	org	0x21E8			; 

	de	a'X'			; 
	de	a'P'			; 
	de	a'I'			; 
	de	a'C'			; 
	de	a'w'			; 
	de	a'x'			; 
	de	a'+'			; 
	de	a'P'			; 
	de	a'I'			; 
	de	a'C'			; 
	de	a'3'			; 
	de	a'C'			; 
	de	a'o'			; 
	de	a'm'			; 
	de	0x0d			; 
	de	0x0a			; 
	de	0x00			; marca de End



	end



